//********************************************************************************************
//*         Name:  hp45.c
//*
//*       Author:  Daniel F. Ramirez
//* 
//*          Date:  10/21/2001
//* 
//*   Description:  
//* 
//* 		This is PIC18F452 floating point application that emulates an HP-45 using the 
//* DIY calculator hardware.  It allows expressions to be entered using HP's RPN calculator language. 
//* The application performs polish notation to be entered from algebraic expressions by using the 
//* keypad or the serial port interface and then evaluates them. It will run on the PIC18F452 with
//* an attached keyboard and LED or LCD display.  It includes the basic arithmetic 
//* operators, "+","-","*", and "/".  
//*
//* 	The calculator supports I/O via the serial port (USART), and the Hex keypad.  It supports
//* LED or LCD displays.
//* 
//* 		Future upgrades to the software include the addition of conversions (Hex, Decimal, Binary),
//* trigonometric, scientific, and financial functions.  Other improvements include simulating
//* an HP-65 programmable calculator with RPN. These new features may require additional SRAM, 
//* FLASH or EEPROM.
//* 
//*********************************************************************************************

  
// Initialize Calculator compilation flags here

//#define SINGLE_PRECISION                               // Include Single Precision code

//#define TEST                                           // Test code

#ifdef PIC18C452

// Include function headers here

  
/*
#ifdef PIC18C252     
#include <p18c252.h>													// Register definitions for PIC18C252
#endif

#ifdef PIC18C452     
#include <p18c452.h>													// Register definitions for PIC18C452
#endif
*/
#endif
  
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
//#include <i2c.h>												  // I2C library function 

#include "flags.h"                                      	// Global flags used for MPSIM debugging 

#ifdef PIC18C452

#include "timeutil.h"                                   // Header file for timer utilities
#else
#include <stdio.h>                                      // Header file for stdio file
#endif
                        
#ifdef SERIAL_DRIVER
#include "stdio.h"                                      // Header file for stdio file
#endif

#ifdef KEYPAD_DRIVER
#include "keypad.h"                                     // Header file for keypad Driver
#endif

#ifdef LED_DRIVER
#include "max7219.h"    										  // Header file for Max7219 LED Display Drivers
#endif

#ifdef LCD_DRIVER
#include "lcd.h"    										  			// Header file for LCD Display Drivers
#endif

#define TRUE 1
#define FALSE 0

#define BufferSize		20			// Floating point buffer size
#define StackSize		 5			// HP-45 stack (5 levels)
#define RegisterSize	10			// HP-45 Registers (1-9)

#define X_LOCATION 0				// X Location in stack
#define Y_LOCATION 1				// Y Location in stack
#define Z_LOCATION 2				// Z Location in stack
#define T_LOCATION 3				// T Location in stack (top of stack)

// Define calculator keys here

// General Calculator functions

#define GOLD    'G'					// Gold function key  ??? 
#define ENTER	'E'                 // Enter key ???
#define ROLL    'R'					// Roll key  ???
#define SWAP    'S'					// Swap X with Y key ???
#define STORE   'O'					// Store X into register N, N=1..9  ???
#define RECALL  'I'					// Recall register N, N=1..9 into X ???
#define LASTX   'L'                 // Recall register Last X into X

// Math Operators 

#define PLUS	'+'					// "+" Operator
#define MINUS	'-'					// "-" Operator
#define TIMES	'*'					// "*" Operator
#define DIVIDE	'/'					// "/" Operator

// Math functions


// Declare global variables here

#define SQRT	'A'					// Square root function
#define SIN		'B'					// Sine function
#define COS		'C'					// Cosine function
#define TAN		'D'					// Tangent function


 
char af1[BufferSize];    // Holds ascii representation of an IEEE floating point number for testing

char Period[] = ".";
char Plus[] = "+";
char Minus[] = "-";
char Blank[] = " ";
char E[] = "E";

char Operators[] = {'+','-','*','/','.','=',0};          // Math operators 
//extern char Operators[];                               // Math operators declared in keypad.c

char NumericChars[] = {'.','E',0};                       // Numeric characters
unsigned char DisableLift = FALSE;								 // Disable the lift flag
 
// Power of 10 lookup tables used by the pow10 function

#ifdef PIC18C452
//     10 **                 0     1      2      3     4       5      6      7      8      9      10
const rom double PlusPowerOf10[11] = {1.0, 1.0E1, 1.0E2, 1.0E3, 1.0E4, 1.0E5, 1.0E6, 1.0E7, 1.0E8, 1.0E9, 1.0E10 };
const rom double MinusPowerOf10[11] = {1.0, 1.0E-1, 1.0E-2, 1.0E-3, 1.0E-4, 1.0E-5, 1.0E-6, 1.0E-7, 1.0E-8, 1.0E-9, 1.0E10 };
#else
//     10 **                 0     1      2      3     4       5      6      7      8      9      10
const double PlusPowerOf10[11] = {1.0, 1.0E1, 1.0E2, 1.0E3, 1.0E4, 1.0E5, 1.0E6, 1.0E7, 1.0E8, 1.0E9, 1.0E10 };
const double MinusPowerOf10[11] = {1.0, 1.0E-1, 1.0E-2, 1.0E-3, 1.0E-4, 1.0E-5, 1.0E-6, 1.0E-7, 1.0E-8, 1.0E-9, 1.0E-10 };
#endif

// Declare function headers here

void main(void);
double Calculator(double Argument1, char Operator, double Argument2);

// HP-45 Functions

void Lift(void);
void Drop(void);
void Roll(void);
void Swap(void);
 

// Floating point conversion functions

void Normalize(double *Value, int *Exponent);
unsigned long NormalizeFraction(double FractionalPart, int NumberOfDigits);

#ifdef SINGLE_PRECISION
void FloatToAscii(float Number, char * s);
void AsciiToFloat(float Number, char * s);
#endif
void DoubleToAscii(double Number, char * s);
void AsciiToDouble(double *Number, char * s);
int Member(char Ch, char * Set);

float GetFloat(void);
double GetDouble(void);

double log10(double x);
double pow10(int x);
void ConfigureInterrupts(void);
char GetKey(void);

char TheOperator;        // Current operator function key: "+","-","*","/"
char TheEqualSign;       // Equal sign, "="

double Argument1;
double Argument2;
double Result;

// Calculator specific variables

char Buffer[BufferSize];					// Floating point Buffer
char StackLabel[]={'X','Y','Z','T'};		// Stack register labels
double Stack[StackSize];					// HP-45 Stack 
double Registers[RegisterSize];				// HP-45 Registers 1-9
double X = 0.0;								// X register (Display)
double Y = 0.0;								// Y register
double LastX = 0.0;							// Last X register
byte RegisterNumber = 0;					// Register number 1..9
int CharacterCount = 0;								// Character count
int BufferCount = 0;								// Buffer count


byte KeyValue; 
extern char Ch;

#ifdef STDIO_INTERRUPTS

//********************************************************************************
// Interrupt Code
//********************************************************************************


// Designate ServiceInterrupt as an interrupt function and save key registers

//#pragma interrupt ServiceInterrupt save = PRODL,PRODH,FSR0L,FSR0H
#pragma interrupt ServiceInterrupt save = PRODL,PRODH

// Locate ISR handler code at interrupt vector

#pragma code isrcode=0x0008

void            isrhandler(void)                           // This function directs
 // execution to the
{                                                          // actual interrupt code
    _asm
    goto ServiceInterrupt
    _endasm
}

#pragma code

//********************************************************************************
//* ServiceInterrupt - Process each type of interrupt (TMR0IF..TMR3IF).
//********************************************************************************
void            ServiceInterrupt(void)
{
    // ISR code timing
 
    //SERVO_SPULSE = 1;                                            // Toggle output pin for
                                                           	 // ISR code timing 

    if (PIR1bits.RCIF)                                       // Check for incomming characters from USART  
    {
 
  	   SerialInterruptCount++;                                // Increment serial interrupt count
          
       
    	// First check to see if data is available from the USART and then read it if 
    	// it has been received.  

    	if (DataRdyUSART())
    	{
      	// Save byte read from USART into the serial input ring buffer

    		RingBuffer[EndBuffer] = ReadUSART();
    		
    		// Echo the character 
    		
    		putchar(RingBuffer[EndBuffer]);
    		
        	//RingBuffer[EndBuffer] = RCREG;

			// Check for Carriage Return character (CR) indicating the end of input
 			
 			if (RingBuffer[EndBuffer] == CR)
 			{
        		ReceivedDataFlag = TRUE;									 // Set the Received Data flag 
        		
        		printf("trace 2: Received CR ! NumberCharacters = \r\n");
        		dec(NumberCharacters);
        		printf("\r\n");

        	}

			EndBuffer++;
        	EndBuffer = EndBuffer & (BUFFER_SIZE - 1);        	 // Wrap the EndBuffer pointer if necessary
        	 
        NumberCharacters = EndBuffer - StartBuffer;                                 // Increment the number of characters currently in the buffer

       	if (NumberCharacters < 0)
        	{
        			NumberCharacters = 0;
        	}	
    	}

	   PIR1bits.RCIF = FALSE;											// Clear the serial interrupt flag
 
   }
}   
#endif
 
void main (void)
{

	int i;                                   // Index 
	byte Ch;
	
	BufferCount = 0;								// Buffer count
	
#ifdef STDIO_INTERRUPTS	
	ReceivedDataFlag =  FALSE;
	
	NumberCharacters = 0;
#endif
 
#ifdef PIC18C452

#ifdef SERIAL_DRIVER 

   // Configure USART for 20 MHz, 9600 Baud, 8-Bits, No parity, Asynchronous
   
   InitializeUsart();
    

   
#endif

      // Setup timers 

      SetupTimers();  
      
      
#ifdef LCD_DRIVER
		InitializeLCD();
		//printf("trace 1:\r\n");
		//lcd_test_1();
		//lcd_test_2();
	   lcd_clear();						// clear LCD, home cursor

#endif

#ifdef LED_DRIVER
		printf("trace 2:\r\n");
		InitializeMAX7219();
#endif


#ifdef KEYPAD_DRIVER

  // Initialize keypad, Setup timers, configure ports and initialize variables
		printf("trace 3:\r\n");

   InitializeKeypad();  
#endif




#endif

/*

    printf("PIC18C452 Start KeyScan Test ... \r\n\n");
  
again:
    
    KeyValue = GetKey() - 48;					// Convert the key value from ASCII to binary
  
    // Send the scanned key to the host

    printf("Key pressed = ");
    dec(KeyValue);
    printf("\r\n");

    goto again;

*/
#ifdef STDIO_INTERRUPTS
 
	// Configure interrupts for serial I/O and other peripherals that need them
	
	ConfigureInterrupts();
#endif

      // Wait 5 seconds 
/*      
      printf("5 Second Test ...\r\n");
      pause(5000);
      printf("End of 5 Second Test\n");
*/

	// Initialize Serial LCD Display here
	
	//printf("\r8-N-1");  
	 
	printf("DIY Calculator \r\n");	
	
	// Test the Ascii to double conversion function
/*	
	af1[0] = '9';
	af1[1] = '8';
	af1[2] = '7';
	af1[3] = '.';
	af1[4] = '6';
	af1[5] = '5';
	af1[6] = '4';
	af1[7] = 0;
	
	AsciiToDouble(&X, af1);
	

   Enter();
*/	
/* 
	Buffer[0] = '0';
	Buffer[1] = '0';
	Buffer[2] = '0';
	Buffer[3] = '0';
	Buffer[4] = '0';
	Buffer[5] = '0';
	Buffer[6] = '0';
	Buffer[7] = 0;

	Buffer[0] = '3';
	Buffer[1] = '.';
	Buffer[2] = '0';
	Buffer[3] = 0;
	
	AsciiToDouble(&X, Buffer);
	
	Buffer[0] = '4';
	Buffer[1] = '.';
	Buffer[2] = '0';
	Buffer[3] = 0;
	
	AsciiToDouble(&X, Buffer);
*/	

	// Initialize stack 
	
   Stack[X_LOCATION] = 0.0;
   Stack[Y_LOCATION] = 0.0;
   Stack[Z_LOCATION] = 0.0;
   Stack[T_LOCATION] = 0.0;


	// Test stack operations 
	
   Stack[X_LOCATION] = 1.0;
   Stack[Y_LOCATION] = 2.0;
   Stack[Z_LOCATION] = 3.0;
   Stack[T_LOCATION] = 4.0;
    
	// Make sure we can perform a stack lift initially
	
    DisableLift = FALSE;
    
	// Interpret HP-45 RPN key strokes by reading function key or digits from the keypad and immediatly
	// execute them. Floating point arguments entered from the keyboard or keypad are used to compute 
    // the desired operation using the selected Calculator function.

#ifdef LCD_DRIVER
    lcd_clear();						// clear LCD, home cursor	    
	puts("> ");			      	    // Display calculator prompt
#endif
 
	// Get first floating point argument and the corresponding argument

#ifdef SERIAL_DRIVER
	printf("\r\n> ");			    	// Display calculator prompt
		
#endif
	
	while (1)
	{
			
		// Get a keystroke from the operator using the hex key pad or
		// the serial port (USART) and store it in the floating point work buffer.

#ifdef STDIO_INTERRUPTS
 
   		if ((ReceivedDataFlag == TRUE) && (NumberCharacters >= 1))
   		{
 
			Ch = getchar();                   	// Serial Input and Output 
			//Ch = GetKey();					// Keypad Input and Serial Output  
			putch(Ch);							// Echo the character just read

#else

			Ch = getchar();                   	// Serial Input and Output 
			//Ch = GetKey();                    // Keypad Input and LED or LCD output  
			putch(Ch);							// Echo the character just read
 
#endif		

			if (isdigit(Ch) || (Ch == '.'))
			{
				// Save the key in the input buffer

				Buffer[BufferCount++] = Ch;
 			}
			else
			{
												

				if (BufferCount > 0)
				{
					// Automatically lift the stack by one unless lift has been disabled for the current operation
 				
 					Lift();
 					
					// This logic needs to be modified or moved ?????
				
					Buffer[BufferCount++] = 0;			// Terminate the string 
				
					// Convert the value stored in the buffer from ascii to float and save it in register X
 
					AsciiToDouble(&X, Buffer);
				
					// Copy number from the display register X to the Stack register X 
				
 					Stack[X_LOCATION] = X;
 				
					LastX = X;							// Save value of X into LAST_X register
					BufferCount = 0;					// Buffer count
				}	
 										
				// Interpret the HP-45 RPN command here
				
				switch (Ch)
				{
					// Stack operations

					case ENTER:						// "Enter ^" key 
					{
						// Lift stack up one level and disable automatic lift for the next operation

						Lift();
						DisableLift = TRUE;
						break;
					}

					case ROLL:						// "R !" key 
					{
						// Rotate the values in the stack while preserving all values.    

						Roll();
						break;
					}

					case SWAP:						// "X <> Y" key 
					{
						// Swap X with Y

						Swap();
						break;
					}

					case STORE:						// "STO" key 
					{
						// Store X into register N, where N=1..9

						RegisterNumber = GetKey();
						Registers[RegisterNumber] = X;
						break;
					}

					case RECALL:					// "RCL" key 
					{
						// Recall register N, where N=1..9 into X

						RegisterNumber = GetKey();
						X = Registers[RegisterNumber];
						break;
					}

					case LASTX:					// "LAST X" key 
					{
						// Recall Last X register into X

						X = LastX;
 						Stack[X_LOCATION] = X;
 						
						break;
					}

					// Basic math operations

					case PLUS:						// "+" key
					{
						// Compute the "+" operation and place result in the X register

						Stack[X_LOCATION] = Calculator(Stack[Y_LOCATION],PLUS, Stack[X_LOCATION]);
			
						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
					case MINUS:						// "-" key
					{
						// Compute the "-" operation and place result in the X register

						Stack[X_LOCATION] = Calculator(Stack[Y_LOCATION], MINUS, Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
					case TIMES:						// "*" key
					{
						// Compute the "*" operation and place result in the X register

						Stack[X_LOCATION] = Calculator(Stack[Y_LOCATION], TIMES, Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
					case DIVIDE:					// "/" key
					{
						// Compute the "/" operation and place result in the X register

						Stack[X_LOCATION] = Calculator(Stack[Y_LOCATION],DIVIDE, Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 					
				}
							
				// Copy number from the Stack register X to the display register X  
				
 				X = Stack[X_LOCATION];
				
				// Convert the value in register X from float to ascii for display to the monitor, LED or LCD.

				DoubleToAscii(X, af1);

			}
		
#ifdef LED_DRIVER	
		// Send result to LED		
		MaxPrint(af1);
#endif			
		
	
		
#ifdef SERIAL_DRIVER

		// Hit any  key to continue

		//Ch = getchar();                   // Serial Input and Output 
		//Ch = GetKey();								// Keypad Input and Serial Output  

#else

		Ch = GetKey();                      // Keypad Input and LED or LCD output  

#endif

#ifdef STDIO_INTERRUPTS
		}
#endif		

	}

#ifdef NEW_CODE

	// Get floating point arguments from the keyboard or keypad and compute the desired operation
	// using the  Calculator function.
	
	while (1)
	{
	   //lcd_clear();						// clear LCD, home cursor

		// Get first floating point argument and the corresponding argument

#ifdef SERIAL_DRIVER
		//printf("Enter Argument 1: ");
#endif

		Argument1 = GetDouble();
		
#ifdef SERIAL_DRIVER
		//printf("\r\n");
#endif

		// Get second floating point argument and the equal sign

#ifdef SERIAL_DRIVER
		//printf("Enter Argument 2: ");
#endif
		Argument2 = GetDouble();
		
#ifdef SERIAL_DRIVER
		//printf("\r\n");
#endif

		// Compute the desired operation

		Result = Calculator(Argument1, TheOperator, Argument2);
		//MyResult = (int) Result;

#ifdef SERIAL_DRIVER
		//printf("%f + %f = %f \r\n", Argument1, Argument2, Result);
#endif

		// Convert argument1 from float to ascii for display to the monitor, LED or LCD.

		DoubleToAscii(Argument1, af1);
		
#ifdef LED_DRIVER	
		// Send output to LED		
		MaxPrint(af1);
#endif

#ifdef SERIAL_DRIVER	

		// Send output to Serial Port
/*		
		puts(af1);
		printf(" ");
		putchar(TheOperator);
		printf(" ");
*/
#endif

		// Convert argument2 from float to ascii for display to the monitor, LED or LCD.

		DoubleToAscii(Argument2, af1);
		
#ifdef LED_DRIVER	
		// Send output to LED		
		MaxPrint(af1);
#endif		

#ifdef SERIAL_DRIVER
		// Send argument 2 to Serial Port
/*
		puts(af1);
		printf(" = ");
*/
#endif		
		
		// Convert result from float to ascii for display to the monitor, LED or LCD.

		DoubleToAscii(Result, af1);
		
#ifdef LED_DRIVER	
		// Send result to LED		
		MaxPrint(af1);
#endif			
		
#ifdef SERIAL_DRIVER
		// Send result to Serial Port
		puts(af1);
		//printf("\r\n");
		
		
#ifdef SERIAL_DRIVER

		// Hit any  key to continue

		//Ch = getchar();                   // Serial Input and Output 
		Ch = GetKey();								// Keypad Input and Serial Output  

#else

		Ch = GetKey();                      // Keypad Input and LED or LCD output  

#endif

#endif

	}

#ifdef TEST

	// Test code 
   // Test strcat function

   strcat(af1, Plus);
   strcat(af1, Minus);
   strcat(af1, Period);
   strcat(af1, E);

	Argument1 = 323.1123E+5;
    DoubleToAscii(Argument1, af1);

	Argument1 = -12345.6789;
    DoubleToAscii(Argument1, af1);
	

	Argument1 = pow10(3);
   Argument2 = log10(4500.0);
   Argument2 = log10(10000.0);

	Argument1 = -12345.6789;
    DoubleToAscii(Argument1, af1);
	printf("Argument1 = %lf  af1 = %s\r\n", Argument1, af1);

	Argument1 = 323.1123E+5;
    DoubleToAscii(Argument1, af1);
	printf("Argument1 = %lf  af1 = %s\r\n", Argument1, af1);

	Argument1 = 1234.244E+8;
    DoubleToAscii(Argument1, af1);
	printf("Argument1 = %lf  af1 = %s\r\n", Argument1, af1);

	Argument1 = -31415.023E+3;
    DoubleToAscii(Argument1, af1);
	printf("Argument1 = %lf  af1 = %s\r\n", Argument1, af1);

	Argument1 = -7654.0E-11;
    DoubleToAscii(Argument1, af1);
	printf("Argument1 = %lf  af1 = %s\r\n", Argument1, af1);


   
    Argument1 = 3.5;
    Argument2 = 6.5;
 
	// Invoke calculator for test

	// Test addition

	Result = Calculator(Argument1, '+', Argument2);
	//printf("%f + %f = %f \r\n", Argument1, Argument2, Result);


	// Convert argument1 from float to ascii

	DoubleToAscii(Argument1, af1);
   puts(af1);
   printf(" + ");
	DoubleToAscii(Argument2, af1);
   puts(af1);
   printf(" = ");
	DoubleToAscii(Result, af1);
   puts(af1);
   printf("\r\n");

	// Test subtraction

	Result = Calculator(Argument1, '-', Argument2);
	//printf("%f - %f = %f \r\n", Argument1, Argument2, Result);
	DoubleToAscii(Argument1, af1);
   puts(af1);
   printf(" - ");
	DoubleToAscii(Argument2, af1);
   puts(af1);
   printf(" = ");
	DoubleToAscii(Result, af1);
   puts(af1);
   printf("\r\n");

	// Test multiplication

	Result = Calculator(Argument1, '*', Argument2);
	//printf("%f * %f = %f \r\n", Argument1, Argument2, Result);
	DoubleToAscii(Argument1, af1);
   puts(af1);
   printf(" * ");
	DoubleToAscii(Argument2, af1);
   puts(af1);
   printf(" = ");
	DoubleToAscii(Result, af1);
   puts(af1);
   printf("\r\n");

	// Test division

	Result = Calculator(Argument1, '/', Argument2);
	//printf("%f / %f = %f \r\n", Argument1, Argument2, Result);
	DoubleToAscii(Argument1, af1);
   puts(af1);
   printf(" / ");
	DoubleToAscii(Argument2, af1);
   puts(af1);
   printf(" = ");
	DoubleToAscii(Result, af1);
   puts(af1);
   printf("\r\n");

	Ch = getch();
#endif
#endif

}

//******************************************************************************
//* Calculator - Simple calculator that performs double precision floating point
//* addition, subtraction, multiplication and division.  It does not currently
//* support levels of nesting parentheses for complicated mathematical expressions.
//******************************************************************************

double Calculator(double Argument1, char Operator, double Argument2)
{
	double TheResult;

	switch(Operator)
	{
		case '+':
		{
			TheResult = Argument1 + Argument2;
			break;
		}

		case '-':
		{
			TheResult = Argument1 - Argument2;
			break;
		}

		case '*':
		{
			TheResult = Argument1 * Argument2;
			break;
		}

		case '/':
		{
			TheResult = Argument1 / Argument2;
			break;
		}

		default:
		{
			TheResult = 0;
			break;
		}		

	}

	return (TheResult);

}

//******************************************************************************
//* Normalize - Normalizes the IEEE double precision floating point number in exponential 
//* notation so that only 1 digit appears to the left of the decimal point. The 
//* exponent is adjusted accordingly.  Used by printf to output floating point numbers.
//******************************************************************************

void Normalize(double *Value, int *Exponent)
{
	double TheNormalizedValue;
	int TheNormalizedExponent;
 
    int i;

	TheNormalizedValue = *Value;
   TheNormalizedExponent = *Exponent;

	if (TheNormalizedValue > 10.0)
	{
		while(TheNormalizedValue > 10.0)
		{
			TheNormalizedValue = TheNormalizedValue / 10.0;
			TheNormalizedExponent++;
			//printf("trace 1: TheNormalizedExponent = %d,  TheNormalizedValue = %lf ", TheNormalizedExponent, TheNormalizedValue);
		}
	}
	else
	if ((TheNormalizedValue > 0.0)  && (TheNormalizedValue < 1.0))
	{
		while((TheNormalizedValue > 0.0)  && (TheNormalizedValue < 1.0))
		{
			TheNormalizedValue = TheNormalizedValue * 10.0;
			TheNormalizedExponent--;
			//printf("trace 2: TheNormalizedExponent = %d,  TheNormalizedValue = %lf ", TheNormalizedExponent, TheNormalizedValue);
		}
	}

	*Value = TheNormalizedValue;
	*Exponent = TheNormalizedExponent;
 
}

//******************************************************************************
//* NormalizeFraction - Normalizes the fractional part of an IEEE double precision 
//* floating point number. Used by printf to output floating point numbers.
//******************************************************************************
 
unsigned long NormalizeFraction(double FractionalPart, int NumberOfDigits)
{
	unsigned long TheNormalizedPart;
   double x,y;

  	//TheNormalizedPart = (unsigned long) (pow(10.0, (double) (NumberOfDigits - 1)) * FractionalPart);
   TheNormalizedPart = (unsigned long) (pow10(NumberOfDigits - 1) * FractionalPart);
  
	return TheNormalizedPart;  
}

#ifdef SINGLE_PRECISION

//******************************************************************************
//* FloatToAscii - Converts an IEEE single precision floating point number from 
//* binary format to an ASCII string. Used by printf to output floating point numbers.
//******************************************************************************

void FloatToAscii(float Number, char * s)
{
	// Invoke DoubleToAscii for now

	DoubleToAscii((float)Number, s);
}

//******************************************************************************
//* AsciiToFloat - Converts an IEEE single precision floating point number from 
//* ASCII format to an IEEE float in binary format.  Used by the scanf function
//* to read floating point numbers.  
//******************************************************************************

void AsciiToFloat(float Number, char * s)
{
	// Invoke AsciiToDouble for now

	AsciiToDouble((float) Number, s);
}
#endif

//******************************************************************************
//* MantissaToAscii - Converts the mantissa part of an IEEE double precision
//* floating point number from binary format to an ASCII string.  Used by printf
//* to output floating point numbers.
//******************************************************************************

void MantissaToAscii(double Number, int Sign, char * s)
{
	unsigned long IntegralPart;              // Holds the floating point integral part
	unsigned long NormalizedPart;            // Holds the floating point normalized fractional part  
	double FractionalPart;					     // Holds the floating point fractional part

	int i;                                   // Index 
	int n;                                   // Number of digits in the mantissa
	char s1[BufferSize];                             // Temporary string
	char s2[BufferSize];                             // Temporary string

   
	IntegralPart = (unsigned long) (Number);
	FractionalPart = (double) (Number - (unsigned long) (Number)) ;

 	// Take care of the mantissa's sign

	if (Sign == -1)
	{
#ifdef PIC18C452
		strcpy(s, Minus);
#else
		strcpy(s, Minus);
#endif
	}
	else
	{
		strcpy(s,Blank);
	}
	// Convert integral part of the floating point number to ascii

#ifdef PIC18C452
	ultoa(IntegralPart, s1);
#else
	ultoa(IntegralPart, s1, 10);
#endif

	// Determine how many digits are in the Mantissa

	n = strlen(s1);

    //printf("n = %d \r\n",n);

	// Normalize the fractional part of the floating point number  

	NormalizedPart = NormalizeFraction(FractionalPart, (11 - n));

	// Convert fractional part of the floating point number to ascii (Number 
	// of digits remaining is (11 - n), including the decimal point. 

#ifdef PIC18C452
	ultoa(NormalizedPart, s2);
#else
	ultoa(NormalizedPart, s2, 10);
#endif

	//printf("IntegralPart = %ld \r\n", IntegralPart);
	//printf("FractionalPart = %lf \r\n", FractionalPart);
	//printf("NormalizedPart = %ld \r\n", NormalizedPart);
	//printf("Sign of mantissa = %d \r\n", Sign);

	//printf("s = !%s!",s);
	//printf("s1 = !%s!",s1);

	// Concatenate fractional part to integer part including the decimal point

	strcat(s, s1);
	strcat(s, Period);
	strcat(s, s2); 

}

//******************************************************************************
//* DoubleFloatToAscii - Converts an IEEE double precision floating point number 
//* from binary format to an ASCII string.  Used by printf to output
//* floating point numbers.
//******************************************************************************

void DoubleToAscii(double Number, char * s)
{
	unsigned long IntegralPart;              // Holds the floating point integral part
	unsigned long NormalizedPart;            // Holds the floating point normalized fractional part 
	unsigned long Mantissa;                  // Holds the floating point mantissa used for exponential notation 
   int SignOfMantissa;                      // Holds the sign of the floating point mantissa
   int SignOfExponent;                      // Holds the sign of the floating point exponent

	int Exponent;                            // Holds the floating point exponent 
	double FractionalPart;					     // Holds the floating point fractional part

	int i;                                   // Index 
	int n;                                   // Number of digits in the mantissa
	char s1[BufferSize];                             // Temporary string
	char s2[BufferSize];                             // Temporary string
 
 

	// Determine exponent

	Exponent = (int) log10(Number);

 	// Determine sign of mantissa and exponent

	if (Number >= 0.0)
	{
		SignOfMantissa = 1;
	}
	else
	{
		SignOfMantissa = -1;		
		Number = -Number;
	}


	if (Exponent >= 0)
	{
		SignOfExponent = 1;
	}
	else
	{
		SignOfExponent = -1;
 	}

	if ((Exponent >= 0) && (Exponent <= 10)) 
	{
		// Convert normal float number with no exponent (need to include the sign)

		MantissaToAscii(Number, SignOfMantissa, s);

	}
	else
	{
		// Normalized the float number using exponential notation

		Normalize(&Number, &Exponent);
/*
		printf("Mantissa = %ld \r\n", Number);
	   printf("Exponent = %d \r\n", Exponent);
		printf("Sign of Mantissa = %d \r\n", SignOfMantissa);
		printf("Sign of Exponent = %d \r\n", SignOfExponent);
*/

		// Convert the mantissa (Number) and the exponent to ascii.  Include characters for the 
		// mantissa and exponent signs, and the "E" for the exponent.

		// Convert mantissa to ascii

		MantissaToAscii(Number, SignOfMantissa, s1);

#ifdef PIC18C452

		// Convert exponent to ascii

		ultoa(Exponent, s2);

#else
		// Convert exponent to ascii

		ultoa(Exponent, s2, 10);
#endif

		// Take care of the mantissa's sign

		if (SignOfMantissa == -1)
		{
			strcpy(s, Minus);
		}
		else
		{
			strcpy(s,Blank);
		}
		
		// Concatanate mantissa

		strcat(s, s1);

		// Concatenate the exponent symbol

		strcat(s,E);

		// Take care of the exponent's sign

		if (SignOfMantissa == -1)
		{
			strcat(s, Minus);
		}
		else
		{
			strcat(s,Plus);
		}
		// Concatanate exponent

		strcat(s, s2);

	}
 
}

//******************************************************************************
//* AsciiToDouble - Converts an IEEE double precision floating point number from
//* ASCII format to an IEEE float in binary format. Used by the scanf function to
//* read long floating point
//* numbers.  
//******************************************************************************

void AsciiToDouble(double *Number, char *s)
{
 
   	char Ch;							// Character 
	int Digit;							// Floating point digit (0..9)
	int i;								// Loop index
	int FractionalStart;				// Starting location of fraction
 

	CharacterCount = 0;
 	*Number = 0.0;   
	Ch = ' ';
	
	i = 0;
	
	// Count the numeric characters in the Buffer before the decimal point (if any)

 	while (s[CharacterCount] != 0)
	{
		if (s[CharacterCount] == '.') 
		{
			goto ExitLoop1;
		}
		else
		{
			CharacterCount++;  
		}
		
	 	//i++;
	}
 
ExitLoop1:

	// Convert the Integral Part of an  IEEE Floating Point number

	for (i = 0; i < CharacterCount; i++) 
	{
 		// Get a digit from the floating point work buffer 

		Ch = s[i];    
		Digit = Ch - 48;    // Convert to Binary
 
 		//if (!Member(Ch, Operators)) 
		if (Member(Ch, Operators) == 0) 
		{
			*Number = *Number + (double) Digit * pow10(CharacterCount-i-1);
		}
		
	} 
		
	// Check terminal character for fractional part of floating point number or for operator (if any)

	Ch = s[CharacterCount];
  
	if (Ch == '.')
	{

		FractionalStart = CharacterCount+1;
		
		CharacterCount = 0;
		i = FractionalStart;
	
		// Count the numeric characters in the Buffer after the decimal point (if any)

 		while (s[i] != 0)
		{
			CharacterCount++;  
			
	 		i++;
		}
 
ExitLoop2:

		// CharacterCount = CharacterCount - 1;

		// Convert the Fractional Part of an IEEE Floating Point number (if present)

		for (i = 0; i < CharacterCount; i++) 
		{
			// Get a keystroke from the floating point work buffer 

			Ch = s[FractionalStart+i];    
			Digit = Ch - 48;    // Convert to Binary
 
 			//if (!Member(Ch, Operators)) 
			if ((Member(Ch, Operators) == 0)) 
			{
				*Number = *Number + (double) Digit * pow10(-i-1);
			}
			
		} 

	}

}

//******************************************************************************
//* log10 - Computes the integral log base 10 of argument, x, where x > 0.0, 
//* using a lookup table.
//******************************************************************************
double log10(double x)
{
	double TempValue;
   double TheLog;

	TempValue = x;
	TheLog = 0;

   
   if (TempValue > 0.0)
	{
		while (TempValue >= 10.0)
		{
			TempValue = TempValue / 10.0;
			TheLog++;
		} 
	}

	return TheLog;

}

//******************************************************************************
//* pow10 - Computes 10 raised to the integer power of argument, x, 
//* range -10 <= x <= 10, using a table lookup.
//******************************************************************************
double pow10(int x)
{
	double TempValue;
 
	if ((x >= 0) && (x <= 10))
	{
   		TempValue = PlusPowerOf10[x];
	}
	else
	{
   		TempValue = MinusPowerOf10[-x];
	}

	return TempValue;
}

//******************************************************************************
//* Member - Determines if a character, Ch, is an element of a given set.
//******************************************************************************
int Member(char Ch, char * Set)
{
	int i;
	int FoundFlag;

	FoundFlag =  FALSE;

	for (i = 0; i< strlen(Set); i++)
	{
		if (Ch == Set[i]) 
		{
			FoundFlag = TRUE;
			goto ExitLoop;
		}
	}
ExitLoop:
	return FoundFlag;
}


#ifdef SINGLE_PRECISION

//******************************************************************************
//* GetFloat - Scans a single precision floating point number from the keypad
//* or reads it from the serial port (USART). Also checks for the mantissa sign and 
//* checks for exponent the number is entered in exponential notation.  
//******************************************************************************
float GetFloat(void)
{	
	// Use GetDouble function for now

	return ((float) GetDouble());

}

#endif
										
//******************************************************************************
//* GetDouble - Scans a double precision floating point number from the keypad
//* or reads it from the serial port (USART). Also checks for the mantissa sign and 
//* checks for exponent the number is entered in exponential notation.  
//******************************************************************************
double GetDouble(void)
{													

	double DoubleFloatValue = 0.0;

   char Ch;				// Character 
	int Digit;				// Floating point digit (0..9)
	int i;					// Loop index

 

	CharacterCount = 0;
 	DoubleFloatValue = 0.0;   
	Ch = ' ';

	// Continue scanning for input until an operator: "+","-","*","/","=" is found

	//while (!Member(Ch, Operators))
	while (Member(Ch, Operators) == 0)
	{

		// Store the ASCII IEEE Floating Point number in a buffer use the 
		// operator key +,-,*,/,= as a delimiter.

		do
		{
			// Get a keystroke from the operator using the hex key pad or
			// the serial port (USART) and store it in the floating point work buffer.

#ifdef SERIAL_DRIVER

			//Ch = getchar();                   // Serial Input and Output 
			Ch = GetKey();								// Keypad Input and Serial Output  

#else

			Ch = GetKey();                      // Keypad Input and LED or LCD output  

#endif

			Buffer[CharacterCount++] = Ch;

 
		//} while (!Member(Ch, Operators) && !Member(Ch, NumericChars) && 
		//	    (CharacterCount < BufferSize));
		} while ((Member(Ch, Operators) == 0) && (Member(Ch, NumericChars) == 0) && 
			    (CharacterCount < BufferSize));
		CharacterCount = CharacterCount - 1;
		

		// Convert the Integral Part of an  IEEE Floating Point number

		for (i = 0; i < CharacterCount; i++) 
		{
 			// Get a digit from the floating point work buffer 

			Ch = Buffer[i];    
			Digit = Ch - 48;    // Convert to Binary
 
 			//if (!Member(Ch, Operators)) 
			if (Member(Ch, Operators) == 0) 
			{
				DoubleFloatValue = DoubleFloatValue + (double) Digit * pow10(CharacterCount-i-1);
			}
		
		} 
		
		// Check terminal character for fractional part of floating point number or for operator (if any)

		Ch = Buffer[CharacterCount];
  
		if (Ch == '.')
		{

			CharacterCount = 0;

			do
			{
				// Get a keystroke from the operator using the hex key pad or
				// the serial port (USART) and store it in the floating point work buffer.

#ifdef SERIAL_DRIVER

				//Ch = getchar();                   // Serial Input and Output 
				Ch = GetKey();								// Keypad Input and Serial Output  

#else

				Ch = GetKey();                      // Keypad Input and LED or LCD output  

#endif
				Buffer[CharacterCount++] = Ch;

 
			//} while (!Member(Ch, Operators) && !Member(Ch, NumericChars) && 
			//    (CharacterCount < BufferSize));
			} while ((Member(Ch, Operators) == 0) && (Member(Ch, NumericChars) == 0) && 
			    (CharacterCount < BufferSize));

			CharacterCount = CharacterCount - 1;

			// Convert the Fractional Part of an IEEE Floating Point number (if present)

			for (i = 0; i < CharacterCount; i++) 
			{
				// Get a keystroke from the floating point work buffer 

				Ch = Buffer[i];    
				Digit = Ch - 48;    // Convert to Binary
 
 				//if (!Member(Ch, Operators)) 
				if ((Member(Ch, Operators) == 0)) 
				{
					DoubleFloatValue = DoubleFloatValue + (double) Digit * pow10(-i-1);
				}
			
			} 

			// Check terminal character for operator (if any) or equal sign

			Ch = Buffer[CharacterCount];
 	
			if (Member(Ch, Operators))
			{
				if (Ch == '=')
				{
					TheEqualSign = Ch;
				}
				else
				{
					TheOperator = Ch;
				}
			}
	
		}
		else
		{
			// Check terminal character for operator (if any) or equal sign

			Ch = Buffer[CharacterCount];
 	
			if (Member(Ch, Operators))
			{
				if (Ch == '=')
				{
					TheEqualSign = Ch;
				}
				else
				{
					TheOperator = Ch;
				}
			}
		}

	}

	return DoubleFloatValue;
}

// HP-45 Functions

//******************************************************************************
//* Lift - Pushes the value in register X onto the simulated HP-45 Stack.  The 
//* contents of register T, at the top of the stack is lost, being replaced by
//* the contents register Z. The next number keyed into X overwrites the old X.
//******************************************************************************
void Lift(void)
{
	// Check to make sure that the Lift operation has not been disabled 
	
	if (DisableLift)
	{
		DisableLift = FALSE;								// Enable Lift for the next operation
	}
	else
	{
 		Stack[T_LOCATION] = Stack[Z_LOCATION]; 				// Contents of T are lost
 		Stack[Z_LOCATION] = Stack[Y_LOCATION]; 				// Copy contents of register Y to register Z   
 		Stack[Y_LOCATION] = Stack[X_LOCATION]; 				// Copy contents of register X to register Y  
 		//Stack[Y_LOCATION] = X; 							// Copy contents of register X to register Y
 	}  
}

//******************************************************************************
//* Drop - The rest of the stack is dropped to fill the gap left by the 
//* previous operation.  The value of the Y register is lost. 
//******************************************************************************
void Drop(void)
{
	int i;												// Local loop variable
	
 	//Stack[X_LOCATION] = Stack[Y_LOCATION]; 			// Contents of X are lost
 	Stack[Y_LOCATION] = Stack[Z_LOCATION]; 				// Copy contents of register Y to register Z   
 	Stack[Z_LOCATION] = Stack[T_LOCATION]; 				// Copy contents of register X to register Y  
	//X = Stack[X_LOCATION]; 							// Copy contents of register X to X  

 		
 		
	// Convert the value in register X from float to ascii for display to the monitor, LED or LCD.

	DoubleToAscii(Stack[X_LOCATION], af1);
		
	// Send result to Serial Port
	printf(" = ");
	puts(af1);
/*	
	// Display the contents of the Stack
		
	printf("\r\nStack...\r\n");
	for (i=3; i>=0; i--)
	{
 		putch(StackLabel[i]);
 		printf(": ");
		DoubleToAscii(Stack[i], af1);
		string(af1);
	}
	printf("\r\n> ");					// Calculator prompt
*/

#ifdef SERIAL_DRIVER
	// Hit any  key to continue
	Ch = getchar();                   // Serial Input   
#endif
				
#ifdef KEYPAD_DRIVER
	// Hit any  key to continue
	Ch = GetKey();                      // Keypad Input    
#endif
	
 	lcd_clear();						// clear LCD, home cursor
 	printf("> ");					// Calculator prompt
	
	
}

//******************************************************************************
//* Roll - 	 Rotate the values in the stack while preserving them.    
//******************************************************************************
void Roll(void)
{
 	Stack[X_LOCATION] = Stack[Y_LOCATION]; 				// Copy contents of register Y to register X
 	Stack[Y_LOCATION] = Stack[Z_LOCATION]; 				// Copy contents of register Z to register Y
 	Stack[Z_LOCATION] = Stack[T_LOCATION]; 				// Copy contents of register T to register Z    
 	Stack[T_LOCATION] = X; 								// Copy contents of X to register T  
    X = Stack[X_LOCATION];								// Copy contents of register X to X 
 
}

//******************************************************************************
//* Swap - Swaps the contents of register X with the contents of register Y.
//******************************************************************************
void Swap(void)
{
	double TempValue;									// Temporary storage for swap
	
	TempValue = Stack[X_LOCATION];						// Save contents of register X
	Stack[X_LOCATION] = Stack[Y_LOCATION]; 				// Copy contents of register Y to register X   
  	Stack[Y_LOCATION] = TempValue; 						// Copy contents of TempValue to register Y     	
	X = Stack[X_LOCATION]; 								// Copy contents of register X to X  
 
}

//********************************************************************************
//* ConfigureInterrupts - Configure interrupts for serial I/O and for other peripherals
//* that require them.
//********************************************************************************

void ConfigureInterrupts(void)
{
     // Configure USART interrupts (1 = High Priority, 0 = Low Priority).

    IPR1bits.RCIP = 1;

    // Enable USART asynchronous interrupt

    PIE1bits.RCIE = 1; 

  // Enable peripheral interrupts
 
   INTCONbits.PEIE = 1;                                    

    // Enable general interrupts

    INTCONbits.GIE = 1;                                    // Enable all interrupts	   
}

//******************************************************************************
//* GetKey - Scans a character from the keypad or reads it from the serial 
//* port (USART).    
//******************************************************************************
char GetKey(void)
{
	char TheKey;
	
	TheKey = get_byte();
 
 	return TheKey;
}

