//********************************************************************************
//*                                                                              *
//* Description:                                                                 *
//*                                                                              *
//*        Name:  mathtst.c                                                      *
//*      Author:  Daniel F. Ramirez                                              *
//*        Date:  08/06/2002                                                     *
//*                                                                              *
//* Description:                                                                 *
//*     This application implements floating point math package that includes the* 
//* trigonmetric and scientific functions using IEEE floating point format for   *
//* the Microchip PIC18F452 micro-controller.  Included is support for a version *
//* of printf that formats and displays floating point numbers to a display or   *
//* PC or laptop connected to the PIC serial port or I2C port.  Both single and  *
//* double precision floating point formats are supported using the PIC18 C      *
//* floating point math.  Results are written to a log file in Matlab format so  *
//* that they may be validated by comparing the results from the JMATH library   *
//* to those from Matlab                                                         *
//*                                                                              *
//*                                                                              *
//*      Future upgrades to this application include: Using lookup tables for    *
//* trigonometric functions, and signal processing routines such as FFT(s), FIR  *
//* FFR digital filters.  In addition the possibility for supporting a PIC       *
//* math co-processor will be explored.  In addition an NJU ??? real-time clock  *
//* will be incorporated to obtain benchmark results without overhead from timer * 
//* interrupts.                                                                  * 
//*                                                                              *
//*                                                                              *
//********************************************************************************
  
  
#include "flags.h"                                      // Global flags used for MPSIM debugging 
  

#ifdef PIC18C252     
#include <p18c252.h>                                    // Register definitions for PIC18C252
#endif

#ifdef PIC18C452     
#include <p18c452.h>                                    // Register definitions for PIC18C452
#else
#include <stdio.h>
#include <stdlib.h>
#endif

#include <ctype.h>
#include <string.h>

#ifdef PIC18C452
#include <i2c.h>                                        // I2C library functions
#include <pwm.h>                                        // PWM library functions
#include <timers.h>                                     // Timer library functions
#include <portb.h>                                      // PORTB library functions                                 
#include <usart.h>                                      // USART library functions
#include <delays.h>                                     // High resolution delay functions
//#include "portutil.h"                                 // Port utility functions
#include "myprintf.h"                                   // My version of printf I/O
#include "timeutil.h"                                   // Include time utilities
#include "math.h"                                       // Include floating point math functions
#include "stdio.h"                                      // Serial I/O support functions
#else
#include "constant.h"
#include "jmath.h"
#endif
  
//********************************************************************************
// Constant Definitions
//********************************************************************************

                                          					// Flag used for debug I/O
#define	PI	3.14159265358979
#define	TWO_PI	6.28318530717958
#define	HALF_PI	1.570796326794895
 
 
#define TABLE_SIZE  360                                 // Size of tables (0..359 degrees)
#define SYSTEM_FREQUENCY 20                             // System frequency, Fs = 20 MHz

// Compute timer clock frequency which depends on the system frequency.  This constant is 
// used by DelayUs function. 

#define CLOCK_FREQUENCY   (SYSTEM_FREQUENCY / 4)        //  Fc = Fs / 4 
 
#define TIME_SCALE_FACTOR 100                           // 100 micro-second granularity for delays

 
#define TRUE  1                                         // Boolean TRUE value
#define FALSE 0                                         // Boolean FALSE value

#define READ  1                                         // I2C read operation
#define WRITE 0                                         // I2C write operation

 
//*******************************************************************************
// Define port bit definitions here
//*******************************************************************************

#define INPUT    1                                     // Input pin direction
#define OUTPUT   0                                     // Output pin direction

//*******************************************************************************
// Define port bit definitions here
//*******************************************************************************

// Port C Bits

   
// PORT D Bits

#define LeftEmitter PORTDbits.RD0                                         // Left IR emitter 
#define RightEmitter PORTDbits.RD1                                        // Right IR emitter

//********************************************************************************
// Type definitions
//********************************************************************************
  
#ifndef PIC18C452
typedef unsigned char byte;                              // 8 Bit unsigned integer
#define MATH_TEST
#endif

//  File variable declarations

FILE *f1;												// Log file #1
FILE *f2;												// Log file #2
FILE *f3;												// Log file #3
FILE *f4;												// Log file #4
FILE *f5;												// Log file #5
FILE *f6;												// Log file #6
FILE *f7;												// Log file #7
FILE *f8;												// Log file #8
FILE *f9;												// Log file #9
FILE *f10;												// Log file #10
FILE *f11;												// Log file #11
FILE *f12;												// Log file #12

//********************************************************************************
// Variable declarations
//********************************************************************************
 
double ConvertToRadians;													// Convert from degrees to radians  
double ConvertToDegrees; 													// Convert from radians to degrees  

byte TheCommand = 0;                                            			// The I2C Command read from host

double x, y, z, w;					// Local values 
 
char ax[BufferSize];    // Holds ascii representation of an IEEE floating point number for testing
char ay[BufferSize];    // Holds ascii representation of an IEEE floating point number for testing
char az[BufferSize];    // Holds ascii representation of an IEEE floating point number for testing
char aw[BufferSize];    // Holds ascii representation of an IEEE floating point number for testing
		
// Pointers and arrays

//byte SinTable[TABLE_SIZE];                                           		// Store a 0..TABLE_SIZE lookup table  

 
 
//********************************************************************************
// Function Prototypes
//********************************************************************************

  
// Math Test Functions  

void main(void);                          	// Main application 
 
void InitializeMath(void);
double Factorial(double n);
double my_sqrt(double a, int n); 

#ifdef MATH_TEST
 
//********************************************************************************
//* main - Driver to test all the floating point math functions including the 
//* trigonmetric and scientific functions along with testing the floating point I/O.
//********************************************************************************
   
void main(void)
{
	long int i,j;						// Loop index
	double my_x, my_y,angle;
	char c;
 
#ifdef PIC18C452


   // Configure USART for 20 MHz, 9600 Baud, 8-Bits, No parity, Asynchronous
   
   InitializeUsart();
#else
   // Open log file to write test results to Matlab

   f1 = fopen("sin.log","w");   
   f2 = fopen("cos.log","w");
   f3 = fopen("tan.log","w");
   f4 = fopen("asin.log","w");   
   f5 = fopen("acos.log","w");
   f6 = fopen("atan2.log","w");

   f7 = fopen("sqrt.log","w");   
   f8 = fopen("exp.log","w");
   f9 = fopen("power.log","w");
   f10 = fopen("mod.log","w");   
   f11 = fopen("log.log","w");
   f12 = fopen("log10.log","w");

#endif

	// Initialize math test package 

	InitializeMath();
	
     // Setup timers 

      SetupTimers();  

    i = sizeof(int);
    j = sizeof(long int);
    x = sizeof(float);
    y = sizeof(double);
    z = sizeof(long);
    
    // The my_sqrt funtion works !!!
    
 	x = my_sqrt(4.0, 15);
	y = my_sqrt(16.0, 15);
	z = my_sqrt(144.0, 15);
	
    x = my_sqrt(25.0, 15);
    y = my_sqrt(1024, 15);
    z = my_sqrt(144.0, 15);
    
	x = square_root((double)25.0);
	y = sqrt((double)25.0);
	x = square_root(-4.0);
	w = sqrt(-4.0);
	
	// Test J. Crenshaw's sqrt function
	
	x = sqrt(2.0);
	y = sqrt(4.0);
	z = sqrt(16.0);
	w = sqrt(144.0);
	
	
#ifdef NEW_CODE	

	// Test J. Crenshaw's sin function.  The sin function works !!!
	
	x = sin(0.0*ConvertToRadians);
	y = sin(90.0*ConvertToRadians);
	z = sin(180.0*ConvertToRadians);
	w = sin(270.0*ConvertToRadians);
	
	x = sin(5.0*ConvertToRadians);
	y = sin(94.0*ConvertToRadians);
	z = sin(177.0*ConvertToRadians);
	w = sin(272.0*ConvertToRadians);
	
	angle = 0.0*ConvertToRadians;
	x = sin(angle);
	
	angle = 90.0*ConvertToRadians;
	y = sin(angle);
	
	angle = 180.0*ConvertToRadians;
	z = sin(angle);
	
	angle = 270.0*ConvertToRadians;
	w = sin(angle);
	
	angle = 5.0*ConvertToRadians;
	x = sin(angle);
	
	angle = 94.0*ConvertToRadians;
	y = sin(angle);
	
	angle = 177.0*ConvertToRadians;
	z = sin(angle);
	
	angle = 272.0*ConvertToRadians;
	w = sin(angle);

	// Test J. Crenshaw's cos function.  The cos function works !!!
	
	x = cos(0.0*ConvertToRadians);
	y = cos(90.0*ConvertToRadians);
	z = cos(180.0*ConvertToRadians);
	w = cos(270.0*ConvertToRadians);
	
	x = cos(5.0*ConvertToRadians);
	y = cos(94.0*ConvertToRadians);
	z = cos(177.0*ConvertToRadians);
	w = cos(272.0*ConvertToRadians);
#endif


	printf("PIC18C452 Math Test Application... \r\n");

	printf("Delay 5 seconds ...\r\n");
#ifdef PIC18C452
	pause(5000);
#endif
	printf("End of 5 seconds.\r\n");
	
	// Test the N ! function from 0! to 100! and see how big the PIC18 C Stack is.
	
	//for (i=0; i<100; i++)
	for (i=0; i<12; i++)
	{
		y = Factorial((double) i);
		dec(i);
		printf("! = ");
		
 		// Convert the value in  x, and y from float to ascii for display to the monitor, LED or LCD.

		DoubleToAscii(y, ay);
		puts(ay);
		printf("\r\n");
		//pause(1000);
	}
	
   // Generate table of sines from 0..360 degrees and write the results to a log file (sin.log)

	printf("Generate a Sine table from 0 to 360 degrees  ... \r\n");
	
	for (i=0; i<360; i++)   
	{
		x =  (double)i*ConvertToRadians;
		y = sin(x);
			
		// Send result to Serial Port
 		
#ifdef PIC18C452
		printf("  i = ");
		dec(i);
		printf(" x = ");
		
 		// Convert the value in  x, and y from float to ascii for display to the monitor, LED or LCD.

		DoubleToAscii(x, ax);
		string(ax);
		printf(" sin(x) = ");
		DoubleToAscii(y, ay);
		string(ay);
		printf("\r\n");	
		//pause(500);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  x = %15.8le, sin = %15.8le  \n", i, x, y);
		//fprintf(f1,"  i = %d,  x = %15.8le, sin = %15.8le  \n", i, x, y);
		fprintf(f1,"%15.8le \n", y);
#endif
 
	}
	
	// Generate table of Cosines from 0..360 degrees and write the results to a log file (cos.log)
 
	printf("Generate a Cosine table from 0 to 360 degrees ... \r\n");

 
	for (i=0; i<360; i++)   
	{
		x =  (double)i*ConvertToRadians;
		y =  cos(x);
 
#ifdef PIC18C452
		printf("  i = ");
		dec(i);
		printf(" x = ");
		
 		// Convert the value in  x, and y from float to ascii for display to the monitor, LED or LCD.

		DoubleToAscii(x, ax);
		string(ax);
		printf(" cos(x) = ");
		DoubleToAscii(y, ay);
		string(ay);
		printf("\r\n");	
		//pause(500);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  x = %15.8le, sin = %15.8le  \n", i, x, y);
		//fprintf(f1,"  i = %d,  x = %15.8le, sin = %15.8le  \n", i, x, y);
		fprintf(f1,"%15.8le \n", y);
#endif
	}

#ifdef NEW_CODE

		
	// Generate table of arctangents from 0..359 degrees and write the results to a log file (atan.log)
 
	printf("Generate an arctangent table from 0 to 360 ... \r\n");

	for (i=0; i<360; i++)   
	{
		x =  (double)i*ConvertToRadians;
		y =   tan(x);
		z =  atan(y)*ConvertToDegrees;

#ifdef PIC18C452
		printf("  i = ");
		dec(i);
		printf(" x = ");
		dec(x);
		printf(" y = ");
		dec(y);
		printf(" z = ");
		dec(z);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d, x = %15.8le, tan = %15.8le, atan = %15.8le  \n", i, x, y, z);
		fprintf(f6, "  i = %d, x = %15.8le, tan = %15.8le, atan = %f  \n", i, x, y, z);
		//fprintf(f6,"%15.8le \n", z);
#endif
 
	}
	
	// Generate table of arcsines from 0..359 degrees and write the results to a log file (asin.log)
 
	printf("Generate an arcsine table from 0 to 360 ... \r\n");

	for (i=0; i<360; i++)   
	{
		x =  (double)i*ConvertToRadians;
		y =  sin(x);
		z =  asin(y);

#ifdef PIC18C452
		printf("  i = ");
		dec(i);
		printf(" x = ");
		dec(x);
		printf(" y = ");
		dec(y);
		printf(" z = ");
		dec(z);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  x = %15.8le, sin = %15.8le, asin = %15.8le  \n", i, x, y, z);
		fprintf(f4, "  i = %d,  x = %15.8le, sin = %15.8le, asin = %15.8le  \n", i, x, y, z);
		//fprintf(f4,"%15.8le \n", z);
#endif
 
	}
	



	// Generate table of Tangents from 0..359 degrees and write the results to a log file (tan.log)
 
	printf("Generate a Tangent table from 0 to 360 ... \r\n");

	for (i=0; i<360; i++)   
	{
		x =  (double)i*ConvertToRadians;
		y = tan(x);

#ifdef PIC18C452
		printf("  i = ");
		dec(i);
		printf(" x = ");
		dec(x);
		printf(" y = ");
		dec(y);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  x = %15.8le, tan = %15.8le  \n", i, x, y);
		fprintf(f3, "  i = %d,  x = %15.8le, tan = %15.8le  \n", i, x, y);
		//fprintf(f3,"%15.8le \n", y);
#endif
 
	}
	



	// Generate table of arccosines from 0..359 degrees and write the results to a log file (acos.log)
 
	printf("Generate an arccosine table from 0 to 360 ... \r\n");

	for (i=0; i<360; i++)   
	{
		x =  (double)i*ConvertToRadians;
		y =   cos(x);
		z =  acos(y);

#ifdef PIC18C452
		printf("  i = ");
		dec(i);
		printf(" x = ");
		dec(x);
		printf(" y = ");
		dec(y);
		printf(" z = ");
		dec(z);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  x = %15.8le, cos = %15.8le, acos = %15.8le  \n", i, x, y, z);
		fprintf(f5, "  i = %d,  x = %15.8le, cos = %15.8le, acos = %15.8le  \n", i, x, y, z);
		//fprintf(f5,"%15.8le \n", z);
#endif
 
	}

#endif



	// Generate table of square roots from 0..1024 degrees and write the results to a log file (sqrt.log)
 
	printf("Generate an square root table from 0 to 1024 ... \r\n");

	for (i=0; i<1025; i++)   
	{
		x = (double) i;
		y = sqrt(x);
 
#ifdef PIC18C452
		printf("  i = ");
		dec(i);
		printf(" x = ");
		dec(x);
		printf(" y = ");
		dec(y);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  x = %f,  sqrt = %15.8le  \n", i, x, y);
		fprintf(f7,"  i = %d,  x = %f, sqrt = %15.8le  \n", i, x, y);
		//fprintf(f7,"%15.8le \n", y);
#endif
 
	}

#ifdef NEW_CODE

	// Generate table of exp values from 0..1024 and write the results to a log file (exp.log)
 
	printf("Generate an exponential table from 0 to 1024 ... \r\n");

	for (i=0; i<1025; i++)   
	{
		//x = (double) i;
		y = exp((double) i);
 
#ifdef PIC18C452
		printf("  i = ");
		dec(i);
		printf(" y = ");
		dec(y);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  exp = %15.8le \n", i,y);
		fprintf(f8,"%15.8le \n", y);
#endif
 
	}

#endif

	// Generate table of power values from 0..TABLE_SIZE and write the results to a log file (power.log)
 
	printf("Generate an power of 2 table from 0 to 1024 ... \r\n");

	for (i=0; i<1025; i++)   
	{
		//x = (double) i;
		y = power(2.0, i);
 
#ifdef PIC18C452
		printf("  i = ");
		dec(i);
		printf(" y = ");
		dec(y);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  power = %15.8le \n", i,y);
		fprintf(f9,"%15.8le \n", y);
#endif
 
	}

	// Generate table of power values from 0..1024 and write the results to a log file (mod.log)
 
	printf("Generate an mod (x,360) table from x: 0 to 1024 ... \r\n");

	for (i=0; i<1025; i++)   
	{
		x = (double) i;
		y = mod(x, 360.0);
 
#ifdef PIC18C452
		printf("  i = ");
		dec(i);
		printf(" y = ");
		dec(y);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  mod = %15.8le \n", i,y);
		fprintf(f10,"%15.8le \n", y);
#endif
 
	}

	// Generate table of log values from 0..1024 and write the results to a log file (log.log)
 
	printf("Generate an log table from 0 to 1024 ... \r\n");

	for (i=0; i<1025; i++)   
	{
		x = (double) i;
		y = log(x); 
 
#ifdef PIC18C452
		printf("  i = ");
		dec(i);
		printf(" y = ");
		dec(y);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  log = %15.8le \n", i,y);
		fprintf(f11,"%15.8le \n", y);
#endif
 
	}
#ifdef NEW_CODE
	// Generate table of log10 values from 0..1024 and write the results to a log file (log10.log)
 
	printf("Generate an log10 table from 0 to 1024 ... \r\n");

	for (i=0; i<1025; i++)   
	{
		x = (double) i;
		y = log10(x); 
 
#ifdef PIC18C452
		printf("  i = ");
		dec(i);
		printf(" y = ");
		dec(y);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  log10 = %15.8le \n", i,y);
		fprintf(f12,"%15.8le \n", y);
#endif
 
	}
#endif

#ifdef PIC18C452
   CloseUSART();
#else
   // Close log files

   fclose(f1);
   fclose(f2);
   fclose(f3);
   fclose(f4);
   fclose(f5);
   fclose(f6);
   fclose(f7);
   fclose(f8);
   fclose(f9);
   fclose(f10);
   fclose(f11);
   fclose(f12);
 
#endif

}

#endif
 
//******************************************************************************
//******************************************************************************
//******************************************************************************
//******************************************************************************

//******************************************************************************
//* InitializeMath - Initialize math test package variables.
//******************************************************************************

void            InitializeMath(void)
{
 
	// Initialize the math constants used by the jmath package

	InitializeConstants();

	// Compute the conversion from degrees to radians variable

	ConvertToRadians = PI/180.0;
	ConvertToDegrees = 180.0/PI;

#ifdef PIC18C452
    // Initialize ports  

    PORTA = 0x00;                                          // Clear PORTA
    PORTB = 0x00;                                          // Clear PORTB
    //PORTC = 0x00;                                          // Clear PORTC (Some Port C bits are reserved for USART)
    PORTD = 0x00;                                          // Clear PORTD
    PORTE = 0x00;                                          // Clear PORTE

    // Initialize port I/O direction for output

 
    TRISA = 0x00;                                          // PORT A all outputs.
    TRISB = 0x00;                                          // PORT B all outputs.
    //TRISC = 0x00;                                        // PORTC all outputs (Some Port C bits are reserved for USART).
    TRISD = 0x00;                                          // PORTD all outputs.
    TRISE = 0x00;                                          // PORTE all outputs.

#endif   
 
}

 

//******************************************************************************
// I2CReadByte - Reads a byte from the selected I2C slave device.
//******************************************************************************
  
byte I2CReadByte(byte I2CReadAddress)
{
 	byte TheData;
 
   return(TheData); 
}
//******************************************************************************
// I2CWriteByte - Writes a byte to the selected I2C slave device
//******************************************************************************

void I2CWriteByte(byte I2CWriteAddress, byte TheData)
{
  

   //pause(10);                      // Wait 10 milliseconds (Is this statement needed ????)
}

//******************************************************************************
// Factorial - Finds the value of N!.   
//******************************************************************************

double Factorial(double n)
{
 	
 	// Check for recursion exit condition
 	
	if (n == 0.0)
	{
		return 1.0;
	}
	else
	{
		// Recusion of N!
		return  (n*Factorial(n-1.0));
	}
	
} 

//******************************************************************************
// my_sqrt - Finds the square root by iteration (from Crenshaw's book).  Does not 
// determine the best guess at the initial root.  
//******************************************************************************

double my_sqrt(double a, int n)
{
	double x;
	int i;
	x = a;
	
	if (x < 0.0)
		return x;
		
	for (i=0; i<n; i++)
	{
		x = 0.5 * (a/x + x);
	}
	return x;
} 
