//********************************************************************************
//*                                                                              *
//* Description:                                                                 *
//*                                                                              *
//*     Name:  max7219.c                                                         *
//*   Author:  Daniel F. Ramirez                                                 *
//*     Date:  11/20/2000                                                        *
//*                                                                              *
//* Description:                                                                 *
//*         This is an embedded realtime LED Display application, that uses the  *
//* MAX7219 IC and a PIC18C452 micro-controller from Microchip Technology to     *
//* drive LED displays.  It employs the shiftout and pulsout functions to        *
//* communicate to the MAX7219 IC, and uses reverse, itoa and right_justify      *
//* functions to format data.  Currently it handles up to 2 MAX7219 drivers each *
//* capable of driving up to 5 LED digits.  The specs for the MAX7219 specify    *
//* that a maximum of 8 digits may be driven by it, but I have not been able to  *
//* get large 8 digits working at the same time, maybe because the large LED     *
//* digits are drawing too much current.  The floating point I/O routines        *
//* developed in this application are used for formatting and displaying         *
//* data on LED displays.  They may also be used for both the stdio.c and lcd.c  *    
//* applications to implement the printf %f floating point specifier.            *
//*                                                                              *
//*                                                                              *
//*     Future upgrades to this application include: Implementing large LED      *
//* displays; handling multiple MAX7219 devices; using the USART interface to    * 
//* make a serial LED display controller similar to the Scott Edwards serial LCD.*                                                  
//*                                                                              *
//********************************************************************************
   
#include "flags.h"                                        // Global flags used for MPSIM TESTging 
  
  
#ifdef PIC18C252     
#include <p18c252.h>									  				// Register definitions for PIC18C252
#endif

#ifdef PIC18C452     
#include <p18c452.h>									  				// Register definitions for PIC18C452
#endif

#include <stdlib.h>
#include <string.h>                                       // String functions

#ifdef PIC18C452  
#include <timers.h>                                       // Timer library functions
#include <portb.h>                                        // PORTB library functions                                 
#include <usart.h>                                        // USART library functions
#endif

#ifdef PIC18C452  
#include "stdio.h"                                        // I/O support functions from PIC libraries
#else
#include <stdio.h>                                        // I/O support functions from VC++ libraries
#endif

#include "timeutil.h"                                     // Timer utility functions
#include "flags.h"                                        // Debug flags used for MPSIM
#include "max7219.h"                                      // Max7219 function headers
#include "portutil.h"                                     // Port utilities
#include "display.h"                                      // Realtime display utilities

//********************************************************************************
// Constant Definitions
//********************************************************************************  

//#define TEST 
 
// Declare global constants

#define     MAX7219_IC_0   										// Include MAX7219 IC #0
#define     MAX7219_IC_1   										// Include MAX7219 IC #1

#define     INPUT    1												// Input direction
#define     OUTPUT   0 												// Output direction


#define		HIGH		1
#define		LOW		0


#define 		MAXIMUM_DIGITS  10                           // Maximum number of LED digits to be displayed (up to 5 per MAX7219 device)
//#define     STRING_SIZE  20                              // String size for numeric fields 

// Port C Bits
 
#define SPULSE                   PORTCbits.RC5          // Software timing pulse output (used for benchmarking ISRs)

#define TX                       PORTCbits.RC6           // Used for serial communication to PC Host 
#define RX                       PORTCbits.RC7           // Used for serial communication to PC Host 

#define SPULSEDir               	TRISCbits.TRISC5        // Software timing pulse output pin direction (used for benchmarking ISRs)

// Port D Bits

// Hardware interface with the  MAX7219 (using PIC18C452 Pin numbering)

#define ClockPin1      24    //    BS2 PIN 24 => PORTDbits.RD1      // Data valid on rising 
																		               // edge of this clock (pin RD5).
#define LoadPin1       25     //    BS2 PIN 25 => PORTDbits.RD2      // This control pin tells the 7219 to transfer data to LEDs.  
#define DataPin1       27     //    BS2 PIN 27 => PORTDbits.RD4      // Bits are shifted out this pin # to the MAX7219.

#define ClockPin0      28    //    BS2 PIN 28 => PORTDbits.RD5       // Data valid on rising 
																		         	   // edge of this clock (pin RD5).
#define LoadPin0       29    //    BS2 PIN 29 => PORTDbits.RD6       // This control pin tells the 7219 to transfer data to LEDs.  
#define DataPin0       30    //    BS2 PIN 30 => PORTDbits.RD7       // Bits are shifted out this pin # to the MAX7219.

/*
#define ClockPinDir1   	TRISDbits.TRISD1        // Clock Pin direction   for upper half of LED display (digits 5..9)
#define LoadPinDir1     TRISDbits.TRISD2        // Load Pin direction for upper half of LED display (digits 5..9)
#define DataDir1       	TRISDbits.TRISD4        // Data Pin direction  for upper half of LED display (digits 5..9)

#define ClockPinDir0    TRISDbits.TRISD5        // Clock Pin direction  for lower half of LED display (digits 0..4)
#define LoadPinDir0     TRISDbits.TRISD6        // Load Pin direction for lower half of LED display (digits 0..4)
#define DataDir0        TRISDbits.TRISD7        // Data Pin direction  for lower half of LED display (digits 0..4)
*/  
 
/*
	Register addresses for the MAX7219 to control a given attribute of the
	display, for instance , its brightness or the number shown in a particular
	digit, you write the register address followed by the data.  For example
	to set the brightness you'd send 'brite//followed by a number from 0 (off)
	to 15 (100% bright.)
*/

#define		decode    9									// Decode register, a 1 turns on BCD decoding
#define		brite		10									// Intensity register
#define		scan		11									// scan-limit register
#define		Switch	12									// on/off register
#define		test		15									// Activates test mode (all digits on, 100% bright)


// Declare MAX7219 variables here

byte TheDigit;                                  // Digit to be displayed

char  anum[STRING_SIZE] = "";                   // Numeric field string
char	blank[]="                    ";           // Field of 20 blanks used to reformat numeric I/O
 
//byte  LookupTable[8] = {scan, 3, brite, 15, decode, 0x1F, Switch, 1};     // Lookup table to hold MAX7219 parameters (brightest=15).
//byte  LookupTable[8] = {scan, 3, brite, 3, decode, 0x1F, Switch, 1};      // Lookup table to hold MAX7219 parameters.
byte  LookupTable[8] = {scan, 5, brite, 15, decode, 0x1F, Switch, 1};     // Lookup table to hold MAX7219 parameters.
//byte  LookupTable[8] = {scan,6, brite, 15, decode, 0x1F, Switch, 1};       // Lookup table to hold MAX7219 parameters.
//byte  LookupTable[8] = {scan, 5, test, 15, decode, 0x1F, Switch, 1};      // Lookup table to hold MAX7219 test mode parameters.

//void            itoa(int Value, char *s);
//char            itoh(int x);
//int             htoi(char c);



#ifdef MAX7219_TEST

char TestNumber[]   = "1234567890";
char TestNumber0[]  = "0987654321";
char TestNumber1[]  = "3.14159263";
char TestNumber2[]  = "2.71828282";
char TestNumber3[]  = "6.23456E28";
char TestNumber4[]  = "-3.14E75";
char TestNumber5[]  = "-2.7815";
char TestNumber6[]  = "14533";
char TestNumberL[]  = "HELP  HELP";

/*
char TestNumber7[]  = "----------";
char TestNumber8[]  = "         1";
char TestNumber9[]  = "        2 ";
char TestNumberA[]  = "       3  ";
char TestNumberB[]  = "      4   ";
char TestNumberC[]  = "     5    ";
char TestNumberD[]  = "    6     ";
char TestNumberE[]  = "   7      ";
char TestNumberF[]  = "  8       ";
char TestNumberG[]  = " 9        ";
char TestNumberH[]  = "0         ";
char TestNumberI[]  = "    5.3   ";
char TestNumberJ[]  = "  2.718   ";
char TestNumberK[]  = "   -4.63  ";
char TestNumberL[]  = "HELP  HELP";
*/ 

#ifdef DEBUG
 
void 		Pulsout(byte PortID, byte Pin, word Time);       				               // PIC version of pulsout function                           
      				   // PIC version of count function 
                                            // PIC version of shiftin function                       
void     Shiftout(byte PortID, byte DataPin, byte ClockPin, byte Mode, word Data,
                 byte NumberBits);   	 
 
 
#endif


//******************************************************************************
//* main - main function
//******************************************************************************  
void            main(void)
{

    int             i;
 

	// Configure USART for 20 MHz, 9600 Baud, 8-Bits, No parity, Asynchronous
   
	InitializeUsart();
	
   printf("\r\n PIC18C452 MAX7219 LED Controller \r\n"); 
   
	SetupTimers();                                         // Setup timers and software
                                                          // variables. 
   pause(3000);

   // Initialize the MAX7219
   
   InitializeMAX7219();

   //printf("trace 1: \r\n");
	
   printf("Testing MaxPrint...\r\n");
   
   MaxPrint(TestNumber);
	pause(5000);	  
  	MaxPrint(TestNumber0);
	pause(5000);	
 	MaxPrint(TestNumber1);
	pause(5000);	
  	MaxPrint(TestNumber2);
	pause(5000);
  	MaxPrint(TestNumber3);
	pause(5000);
  	MaxPrint(TestNumber4);
	pause(5000);
  	MaxPrint(TestNumber5);
	pause(5000);
  	MaxPrint(TestNumber6);
	pause(5000);
	MaxPrint(TestNumberL);
	pause(5000);
	
/*
  	MaxPrint(TestNumber7);
	pause(5000);
  	MaxPrint(TestNumber8);
	pause(5000);
  	MaxPrint(TestNumber9);
	pause(5000);
  	MaxPrint(TestNumberA);
	pause(5000);
  	MaxPrint(TestNumberB);
	pause(5000);
  	MaxPrint(TestNumberC);
	pause(5000);
  	MaxPrint(TestNumberD);
	pause(5000);
  	MaxPrint(TestNumberE);
	pause(5000);
  	MaxPrint(TestNumberF);
	pause(5000);
  	MaxPrint(TestNumberG);
	pause(5000);
  	MaxPrint(TestNumberH);
	pause(5000);
  	MaxPrint(TestNumberI);
	pause(5000);
  	MaxPrint(TestNumberJ);
	pause(5000);
  	MaxPrint(TestNumberK);
	pause(5000);
 	MaxPrint(TestNumberL);
	pause(5000);
*/

   while(1)
   {
      for (i=0; i<99999; i++)
      {
#ifdef DEBUG
        itoa(i, TestNumber, 10);										// Convert  i to string
#else
        itoa(i, TestNumber);										// Convert  i to string
#endif
		MaxPrint(TestNumber);
   		pause(10);
   	}
	}     
   
#ifdef TEST 

   printf("Testing pulsout...\r\n");
   
   while(1)
   {
 		pulsout(ClockPin0, 1); 
		pulsout(LoadPin0, 1);                
  		pulsout(DataPin0,1 );  
  		
 		pulsout(ClockPin1, 1); 
		pulsout(LoadPin1, 1);                
  		pulsout(DataPin1, 1 );  

   }
   
   printf("Testing shiftout...\r\n");
   
   while(1)
   {
		shiftout( DataPin0, ClockPin0, MSBFIRST, 0xAA, 8);
		shiftout( DataPin1, ClockPin1, MSBFIRST, 0xAA, 8);
   }
    

    // Pause for keystroke

    // Ch = getch();

 
   
 
#endif
    
}

#endif  

//  MAX7219 functions are implemented here

void InitializeMAX7219(void);

//******************************************************************************
//* InitializeMAX7219 - Initialize the MAX7219 device.
//******************************************************************************  
void   InitializeMAX7219(void)
{
 
	byte MaxData; 
	byte Lsb;
   byte Index;												// Index into setup table
 
     // Setup all pins on port b to output low.

/*
#ifdef PIC18C452 

#ifdef MAX7219_IC_0
 	ClockPinDir0  = OUTPUT;        	        	// Set Clock Pin direction to output 
 	LoadPinDir0 = OUTPUT;            	      // Set Load Pin  direction to output 
 	DataDir0 = OUTPUT;       			        	// Set Data Pin direction to output
#endif

#ifdef MAX7219_IC_1
 	ClockPinDir1  = OUTPUT;        	       	// Set Clock Pin direction to output 
 	LoadPinDir1 = OUTPUT;            	      // Set Load Pin direction to output 
 	DataDir1 = OUTPUT;       			        	// Set Data pin direction to output 
#endif

#endif
*/

    //printf("trace 0: \r\n");
 
    /* Initialize the MAX7219 by using the Lookup table data structure.  Each
     * register is paired with its setting data.  The table sets the scan limit
     * to 4 (5 digits, numbered 0-4); brightness to 3; BCD decoding to the
     * lower 5 digits (the only ones we are displaying), and switches the
     * display on. The MAX7219 expects data in 16-bit packets, but our lookup
     * table holds a series of 8 bit values.  That's why the loop below is
     * designed to pulse the LoadPin line every other byte transmitted.  */


#ifdef MAX7219_IC_0       

    // MAX7219 Device #0 
    
    for (Index = 0; Index < 8; Index++)
    {       
        MaxData = LookupTable[Index];
   
 		  shiftout( DataPin0, ClockPin0, MSBFIRST, MaxData, 8);      // Send the data to the MAX7219 #0

		  // Look at lowest bit of Index

		  Lsb = Index & 1;
        if (Lsb != 0)                               
        {
            pulsout(LoadPin0, 1);                             // If it's 1, pulse the Load Pin 0 line           
        }
    }
#endif

#ifdef MAX7219_IC_1

    // MAX7219 Device #1

    for (Index = 0; Index < 8; Index++)
    
    {
        MaxData = LookupTable[Index];

		  shiftout( DataPin1, ClockPin1, MSBFIRST, MaxData, 8);      // Send the data to the MAX7219 #1

		  // Look at lowest bit of Index

	     Lsb = Index & 1;
        if (Lsb != 0)
        {
            pulsout(LoadPin1, 1);                             // If it's 1, pulse the Load Pin 1 line
        }
    }
#endif
}

//******************************************************************************
//* MaxPrint - Converts a string to separate BCD digits and sends them to the LED 
//* display.  The MAX7219 won't accept a number like 2742 and display it on 
//* the LEDs. Instead it expects the program to send it individual digits preceded 
//* by their position on the display.  It will also send special floating point  
//* characters such as "+","-" "E" and ".".
//******************************************************************************

void     	MaxPrint(char *s)
{

	byte NumberDigits;									// Number of digits in string
	byte Digit;												// Current decimal digit value
	byte AsciiDigit;										// Current ASCII digit value
   byte Index;												// Index into setup table
   byte Position;                               // Position of digit
   byte Decimal_Point;                          // Position of decimal point
   byte i;
   char            s1[STRING_SIZE] = "";			// Temporary workspace string
   char            s2[STRING_SIZE] = "";			// Temporary workspace string
   char            s3[STRING_SIZE] = "";			// Temporary workspace string


   // Save original string

   strcpy(s3, s);

   // Find length of numeric string

   NumberDigits = strlen(s);

   right_justify(MAXIMUM_DIGITS, s);                        // Right Justify and fill with blanks

	// Find location of decimal point
	
	Decimal_Point = 255;
	
   for (i=0; i<MAXIMUM_DIGITS; i++)
	{
	   
	   if (s[i] == '.')
		{
			Decimal_Point = i;
         goto ExitLoop;
		}
	}
 
ExitLoop:

	if (Decimal_Point != 255) 
	{
		// Remove decimal point character from string now that the digit position is known

    	strncpy(s1, s, Decimal_Point);
    	strncpy(s2, s+Decimal_Point+1, (strlen(s)-Decimal_Point));
    	strcpy(s, s1);
    	strcat(s, s2);

		// Adjust decimal point digit location.  The decimal point does not count as a separate digit !

		Decimal_Point = (MAXIMUM_DIGITS - Decimal_Point);
   	//right_justify(MAXIMUM_DIGITS, s);                        // Right Justify and fill with blanks
	}
 
 /*
   printf("****************************\r\n");
   printf("trace 0: NumberDigits = ");
   dec(NumberDigits);
   printf(" Decimal_Point = ");
   dec(Decimal_Point); 
   printf("  s = !");
   string(s);
   printf("! \r\n");
*/ 
 
   
    for (Index = 0; Index < MAXIMUM_DIGITS; Index++)
    {
			if (Index < 5)
 			{

				Position = Index+1; 
 
         	// Digits 0 to 4 using MAX7219 # 0
         	       	
 				shiftout( DataPin0, ClockPin0, MSBFIRST, Position, 8);      // Send the Index

         	AsciiDigit = s[(MAXIMUM_DIGITS-1) - Index];                 // Get ASCII Digit
         	//AsciiDigit = s[MAXIMUM_DIGITS - Index];                      // Get ASCII Digit
 
         	// Check for special characters
              
         	Digit = TranslateCharacter(AsciiDigit);

				// Turn On the Decimal Point 

  				if (Index == Decimal_Point)  
  				{
             	//AsciiDigit = s[(MAXIMUM_DIGITS-1) - Index + 1];             // Get previous ASCII Digit
           		//Digit = TranslateCharacter(AsciiDigit);
/*
           	   printf("Trace 1: AsciiDigit = !");
            	putch(AsciiDigit);
            	printf("!  Digit = ");
            	dec(Digit);       
            	printf("\r\n");
*/
  					Digit = Digit | BIT_7;										// Set decimal for current digit
  				} 
 
/*        
         	printf("trace 2: Index = ");
         	dec(Index);
         	printf("  AsciiDigit = !");
         	putch(AsciiDigit);
         	printf("!  Digit = ");
         	dec(Digit);       
         	printf("\r\n");
        
*/
  				shiftout( DataPin0, ClockPin0, MSBFIRST, Digit, 8);      // Send the data  
        		pulsout(LoadPin0, 1);								   			// Pulse the Load Pin line

			}
        	else
 		  	{
        		// Digits 5 to 9 using MAX7219 # 1
        		
 				Position = Index - 4; 
  
				shiftout( DataPin1, ClockPin1, MSBFIRST, Position, 8);      // Send the Index divided by 2

            AsciiDigit = s[(MAXIMUM_DIGITS-1) - Index];                     // Get ASCII Digit
            //AsciiDigit = s[MAXIMUM_DIGITS - Index];                     // Get ASCII Digit
 
            // Check for special characters
            
            Digit = TranslateCharacter(AsciiDigit);

				// Turn On the Decimal Point 

  				if (Index == Decimal_Point)  
  				{
             	//AsciiDigit = s[(MAXIMUM_DIGITS-1) - Index + 1];             // Get previous ASCII Digit
           		//Digit = TranslateCharacter(AsciiDigit);
/*                     
            	printf("trace 3:  AsciiDigit = !");
            	putch(AsciiDigit);
            	printf("!  Digit = ");
            	dec(Digit);       
            	printf("\r\n");
*/
  					Digit = Digit | BIT_7;										// Set decimal for current digit
  				} 
           
/*                   
            printf("trace 4: Position = ");
            dec(Position);
            printf("\r\n");
            printf("  AsciiDigit = !");
            putch(AsciiDigit);
            printf("!  Digit = ");
            dec(Digit);       
            printf("\r\n");

*/
     			shiftout( DataPin1, ClockPin1, MSBFIRST, Digit, 8);      // Send the data  
       		pulsout(LoadPin1, 1);								   			// Pulse the Load Pin line   
    
			}
        
    }

   // Restore original string

   strcpy(s, s3);

}

//******************************************************************************
//* TranslateCharacter - Translates the character to a digit 0..,"E",".", or "-".
//******************************************************************************

byte TranslateCharacter(char c)
{
	
   // Check for special characters         
      
   switch (c)
   {
   	case '-':
  		{
  			TheDigit = 10;															// "-" character
  			break;
  		}
  	
  		case 'E':
  		{
  			TheDigit = 11;															// "E" character
  			break;
  		}
  		
   	case 'H':
  		{
  			TheDigit = 12;															// "H" character
  			break;
  		}
  		
   	case 'L':
  		{
  			TheDigit = 13;															// "L" character
  			break;
  		}
  		
   	case 'P':
  		{
  			TheDigit = 14;															// "P" character
  			break;
  		}
  		
  		case ' ': 
  		{
  			TheDigit = 15;															// Blank " " character
 			break;
  		}
  			
  		default:
  		{
     	 	TheDigit =  c - 48;                                     // Convert Digit from ASCII to Binary
      	break;
  		}
  	}
    			
	return TheDigit;
}

/*******************************************************************************
 * right_justify - function to right justify a string.
 *******************************************************************************/

void            right_justify(int size, char *s)

{
    int             len;
    char            s1[STRING_SIZE] = "";
    int             nchars;

  
    len = strlen(s);
    if (len < size)
    {
        nchars = size - len;
        strncpy(s1, blank, nchars);
        strcat(s1, s);
        strcpy(s, s1);
    }
}

 
/*
//*******************************************************************************
//* htoi - function to convert an ascii hex digit to decimal.
//*******************************************************************************

int             htoi(char c)
{
    int             x;

    // Convert invalid characters and blanks to blank code 15 for the display
 
    if ((c >= '0') && (c <= '9'))
      x = (int) c - 48;
    else
      x = 15;

    return (x);
}

//*******************************************************************************
//* itoh - function to convert a decimal to an ascii hex digit.
//*******************************************************************************

char            itoh(int x)
{
    char            c;

    c = (char) (x + 48);
    return (c);
}

 
*/
 
#ifdef DEBUG

// declare stubs for functions that are not available in VC++ here

// VC++ version of pulsout function 
void 		pulsout(byte Pin, word Time)
{	 
}

// VC++ version of shiftout function                       
                                                                                         
void     shiftout(byte DataPin, byte ClockPin, byte Mode, word Data,
                 byte NumberBits)
{ 
}

// Converts an integer to an ascii string of numeric characters and writes                                                          
// them to the USART.
int             dec(int Value)
{
	return 0;
}

void            SetupTimers(void)
{
}

// Pause for n units of 1000 instruction

void            pause(unsigned int n)
{
}  
                            
// Initializes the USART
void 		InitializeUsart(void)
{
}

	 	                                             
#endif


