//********************************************************************************
//*        Name:  myprintf.c                                                     *
//*      Author:  Daniel F. Ramirez                                              *
//*        Date:  12/26/2002                                                     *
//*                                                                              *
//*                                                                              * 
//* Description:                                                                 *
//*     This file contains my printf I/O functions used by the PIC "C" compiler  * 
//* Included are low level floating point conversion functions.                  *
//*                                                                              *
//*      Future upgrades to this application include: File I/O, I2c, and         *
//* Internet networking.                                                         *
//*                                                                              *
//*                                                                              *
//********************************************************************************
  
        
#include "flags.h"                                      // Global flags used for MPSIM debugging 
#include "myprintf.h"
//#include "keypad.h"

#ifdef PIC18C252     
#include <p18c252.h>					// Register definitions for PIC18C252
#endif
  
#ifdef PIC18C452     
#include <p18c452.h>					// Register definitions for PIC18C452
#endif

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
//#include <i2c.h>					// I2C library functions
//#include <pwm.h>                                        // PWM library functions

#ifdef PIC18C452   
#include <timers.h>                                     // Timer library functions
#include <portb.h>                                      // PORTB library functions                                 
#include <usart.h>                                      // USART library functions
#include <delays.h>                                     // High resolution delay functions
#endif

#define TRUE 1
#define FALSE 0


char Period[] = ".";
char Plus[] = "+";
char Minus[] = "-";
char Blank[] = " ";
char E[] = "E";
char TheOperator;        // Current operator function key: "+","-","*","/"
char TheEqualSign;       // Equal sign, "="

double Argument1;
double Argument2;
double Result;

// Calculator specific variables

char Buffer[BufferSize];					// Floating point Buffer
char Operators[] = {'+','-','*','/','.','=',0};          // Math operators 
//extern char Operators[];                               // Math operators declared in keypad.c

char NumericChars[] = {'.','E',0};                       // Numeric characters
unsigned char DisableLift = FALSE;						 // Disable the lift flag
int CharacterCount = 0;									// Character count
int BufferCount = 0;									// Buffer count
 
// Power of 10 lookup tables used by the pow10 function

#ifdef PIC18C452
//     10 **                 0     1      2      3     4       5      6      7      8      9      10
const rom double PlusPowerOf10[11] = {1.0, 1.0E1, 1.0E2, 1.0E3, 1.0E4, 1.0E5, 1.0E6, 1.0E7, 1.0E8, 1.0E9, 1.0E10 };
const rom double MinusPowerOf10[11] = {1.0, 1.0E-1, 1.0E-2, 1.0E-3, 1.0E-4, 1.0E-5, 1.0E-6, 1.0E-7, 1.0E-8, 1.0E-9, 1.0E10 };
#else
//     10 **                 0     1      2      3     4       5      6      7      8      9      10
const double PlusPowerOf10[11] = {1.0, 1.0E1, 1.0E2, 1.0E3, 1.0E4, 1.0E5, 1.0E6, 1.0E7, 1.0E8, 1.0E9, 1.0E10 };
const double MinusPowerOf10[11] = {1.0, 1.0E-1, 1.0E-2, 1.0E-3, 1.0E-4, 1.0E-5, 1.0E-6, 1.0E-7, 1.0E-8, 1.0E-9, 1.0E-10 };
#endif

//******************************************************************************
//* Normalize - Normalizes the IEEE double precision floating point number in exponential 
//* notation so that only 1 digit appears to the left of the decimal point. The 
//* exponent is adjusted accordingly.  Used by printf to output floating point numbers.
//******************************************************************************

void Normalize(double *Value, int *Exponent)
{
	double TheNormalizedValue;
	int TheNormalizedExponent;
 
    int i;

	TheNormalizedValue = *Value;
   TheNormalizedExponent = *Exponent;

	if (TheNormalizedValue > 10.0)
	{
		while(TheNormalizedValue > 10.0)
		{
			TheNormalizedValue = TheNormalizedValue / 10.0;
			TheNormalizedExponent++;
			//printf("trace 1: TheNormalizedExponent = %d,  TheNormalizedValue = %lf ", TheNormalizedExponent, TheNormalizedValue);
		}
	}
	else
	if ((TheNormalizedValue > 0.0)  && (TheNormalizedValue < 1.0))
	{
		while((TheNormalizedValue > 0.0)  && (TheNormalizedValue < 1.0))
		{
			TheNormalizedValue = TheNormalizedValue * 10.0;
			TheNormalizedExponent--;
			//printf("trace 2: TheNormalizedExponent = %d,  TheNormalizedValue = %lf ", TheNormalizedExponent, TheNormalizedValue);
		}
	}

	*Value = TheNormalizedValue;
	*Exponent = TheNormalizedExponent;
 
}

//******************************************************************************
//* NormalizeFraction - Normalizes the fractional part of an IEEE double precision 
//* floating point number. Used by printf to output floating point numbers.
//******************************************************************************
 
unsigned long NormalizeFraction(double FractionalPart, int NumberOfDigits)
{
	unsigned long TheNormalizedPart;
   double x,y;

  	//TheNormalizedPart = (unsigned long) (pow(10.0, (double) (NumberOfDigits - 1)) * FractionalPart);
   TheNormalizedPart = (unsigned long) (pow10(NumberOfDigits - 1) * FractionalPart);
  
	return TheNormalizedPart;  
}

#ifdef SINGLE_PRECISION

//******************************************************************************
//* FloatToAscii - Converts an IEEE single precision floating point number from 
//* binary format to an ASCII string. Used by printf to output floating point numbers.
//******************************************************************************

void FloatToAscii(float Number, char * s)
{
	// Invoke DoubleToAscii for now

	DoubleToAscii((float)Number, s);
}

//******************************************************************************
//* AsciiToFloat - Converts an IEEE single precision floating point number from 
//* ASCII format to an IEEE float in binary format.  Used by the scanf function
//* to read floating point numbers.  
//******************************************************************************

void AsciiToFloat(float Number, char * s)
{
	// Invoke AsciiToDouble for now

	AsciiToDouble((float) Number, s);
}
#endif

//******************************************************************************
//* MantissaToAscii - Converts the mantissa part of an IEEE double precision
//* floating point number from binary format to an ASCII string.  Used by printf
//* to output floating point numbers.
//******************************************************************************

void MantissaToAscii(double Number, int Sign, char * s)
{
	unsigned long IntegralPart;              // Holds the floating point integral part
	unsigned long NormalizedPart;            // Holds the floating point normalized fractional part  
	double FractionalPart;					     // Holds the floating point fractional part

	int i;                                   // Index 
	int n;                                   // Number of digits in the mantissa
	char s1[BufferSize];                             // Temporary string
	char s2[BufferSize];                             // Temporary string

   
	IntegralPart = (unsigned long) (Number);
	FractionalPart = (double) (Number - (unsigned long) (Number)) ;

 	// Take care of the mantissa's sign

	if (Sign == -1)
	{
#ifdef PIC18C452
		strcpy(s, Minus);
#else
		strcpy(s, Minus);
#endif
	}
	else
	{
		strcpy(s,Blank);
	}
	// Convert integral part of the floating point number to ascii

#ifdef PIC18C452
	ultoa(IntegralPart, s1);
#else
	ultoa(IntegralPart, s1, 10);
#endif

	// Determine how many digits are in the Mantissa

	n = strlen(s1);

    //printf("n = %d \r\n",n);

	// Normalize the fractional part of the floating point number  

	NormalizedPart = NormalizeFraction(FractionalPart, (11 - n));

	// Convert fractional part of the floating point number to ascii (Number 
	// of digits remaining is (11 - n), including the decimal point. 

#ifdef PIC18C452
	ultoa(NormalizedPart, s2);
#else
	ultoa(NormalizedPart, s2, 10);
#endif

	//printf("IntegralPart = %ld \r\n", IntegralPart);
	//printf("FractionalPart = %lf \r\n", FractionalPart);
	//printf("NormalizedPart = %ld \r\n", NormalizedPart);
	//printf("Sign of mantissa = %d \r\n", Sign);

	//printf("s = !%s!",s);
	//printf("s1 = !%s!",s1);

	// Concatenate fractional part to integer part including the decimal point

	strcat(s, s1);
	strcat(s, Period);
	strcat(s, s2); 

}

//******************************************************************************
//* DoubleFloatToAscii - Converts an IEEE double precision floating point number 
//* from binary format to an ASCII string.  Used by printf to output
//* floating point numbers.
//******************************************************************************

void DoubleToAscii(double Number, char * s)
{
	unsigned long IntegralPart;              // Holds the floating point integral part
	unsigned long NormalizedPart;            // Holds the floating point normalized fractional part 
	unsigned long Mantissa;                  // Holds the floating point mantissa used for exponential notation 
   int SignOfMantissa;                      // Holds the sign of the floating point mantissa
   int SignOfExponent;                      // Holds the sign of the floating point exponent

	int Exponent;                            // Holds the floating point exponent 
	double FractionalPart;					     // Holds the floating point fractional part

	int i;                                   // Index 
	int n;                                   // Number of digits in the mantissa
	char s1[BufferSize];                             // Temporary string
	char s2[BufferSize];                             // Temporary string
 
 

	// Determine exponent

	Exponent = (int) log10(Number);

 	// Determine sign of mantissa and exponent

	if (Number >= 0.0)
	{
		SignOfMantissa = 1;
	}
	else
	{
		SignOfMantissa = -1;		
		Number = -Number;
	}


	if (Exponent >= 0)
	{
		SignOfExponent = 1;
	}
	else
	{
		SignOfExponent = -1;
 	}

	if ((Exponent >= 0) && (Exponent <= 10)) 
	{
		// Convert normal float number with no exponent (need to include the sign)

		MantissaToAscii(Number, SignOfMantissa, s);

	}
	else
	{
		// Normalized the float number using exponential notation

		Normalize(&Number, &Exponent);
/*
		printf("Mantissa = %ld \r\n", Number);
	   printf("Exponent = %d \r\n", Exponent);
		printf("Sign of Mantissa = %d \r\n", SignOfMantissa);
		printf("Sign of Exponent = %d \r\n", SignOfExponent);
*/

		// Convert the mantissa (Number) and the exponent to ascii.  Include characters for the 
		// mantissa and exponent signs, and the "E" for the exponent.

		// Convert mantissa to ascii

		MantissaToAscii(Number, SignOfMantissa, s1);

#ifdef PIC18C452

		// Convert exponent to ascii

		ultoa(Exponent, s2);

#else
		// Convert exponent to ascii

		ultoa(Exponent, s2, 10);
#endif

		// Take care of the mantissa's sign

		if (SignOfMantissa == -1)
		{
			strcpy(s, Minus);
		}
		else
		{
			strcpy(s,Blank);
		}
		
		// Concatanate mantissa

		strcat(s, s1);

		// Concatenate the exponent symbol

		strcat(s,E);

		// Take care of the exponent's sign

		if (SignOfMantissa == -1)
		{
			strcat(s, Minus);
		}
		else
		{
			strcat(s,Plus);
		}
		// Concatanate exponent

		strcat(s, s2);

	}
 
}

//******************************************************************************
//* AsciiToDouble - Converts an IEEE double precision floating point number from
//* ASCII format to an IEEE float in binary format. Used by the scanf function to
//* read long floating point
//* numbers.  
//******************************************************************************

void AsciiToDouble(double *Number, char *s)
{
 
   	char Ch;							// Character 
	int Digit;							// Floating point digit (0..9)
	int i;								// Loop index
	int FractionalStart;				// Starting location of fraction
 

	CharacterCount = 0;
 	*Number = 0.0;   
	Ch = ' ';
	
	i = 0;
	
	// Count the numeric characters in the Buffer before the decimal point (if any)

 	while (s[CharacterCount] != 0)
	{
		if (s[CharacterCount] == '.') 
		{
			goto ExitLoop1;
		}
		else
		{
			CharacterCount++;  
		}
		
	 	//i++;
	}
 
ExitLoop1:

	// Convert the Integral Part of an  IEEE Floating Point number

	for (i = 0; i < CharacterCount; i++) 
	{
 		// Get a digit from the floating point work buffer 

		Ch = s[i];    
		Digit = Ch - 48;    // Convert to Binary
 
 		//if (!Member(Ch, Operators)) 
		if (Member(Ch, Operators) == 0) 
		{
			*Number = *Number + (double) Digit * pow10(CharacterCount-i-1);
		}
		
	} 
		
	// Check terminal character for fractional part of floating point number or for operator (if any)

	Ch = s[CharacterCount];
  
	if (Ch == '.')
	{

		FractionalStart = CharacterCount+1;
		
		CharacterCount = 0;
		i = FractionalStart;
	
		// Count the numeric characters in the Buffer after the decimal point (if any)

 		while (s[i] != 0)
		{
			CharacterCount++;  
			
	 		i++;
		}
 
ExitLoop2:

		// CharacterCount = CharacterCount - 1;

		// Convert the Fractional Part of an IEEE Floating Point number (if present)

		for (i = 0; i < CharacterCount; i++) 
		{
			// Get a keystroke from the floating point work buffer 

			Ch = s[FractionalStart+i];    
			Digit = Ch - 48;    // Convert to Binary
 
 			//if (!Member(Ch, Operators)) 
			if ((Member(Ch, Operators) == 0)) 
			{
				*Number = *Number + (double) Digit * pow10(-i-1);
			}
			
		} 

	}

}

//******************************************************************************
//* log10 - Computes the integral log base 10 of argument, x, where x > 0.0, 
//* using a lookup table.
//******************************************************************************
double log10(double x)
{
	double TempValue;
   double TheLog;

	TempValue = x;
	TheLog = 0;

   
   if (TempValue > 0.0)
	{
		while (TempValue >= 10.0)
		{
			TempValue = TempValue / 10.0;
			TheLog++;
		} 
	}

	return TheLog;

}

//******************************************************************************
//* pow10 - Computes 10 raised to the integer power of argument, x, 
//* range -10 <= x <= 10, using a table lookup.
//******************************************************************************
double pow10(int x)
{
	double TempValue;
 
	if ((x >= 0) && (x <= 10))
	{
   		TempValue = PlusPowerOf10[x];
	}
	else
	{
   		TempValue = MinusPowerOf10[-x];
	}

	return TempValue;
}

//******************************************************************************
//* Member - Determines if a character, Ch, is an element of a given set.
//******************************************************************************
int Member(char Ch, char * Set)
{
	int i;
	int FoundFlag;

	FoundFlag =  FALSE;

	for (i = 0; i< strlen(Set); i++)
	{
		if (Ch == Set[i]) 
		{
			FoundFlag = TRUE;
			goto ExitLoop;
		}
	}
ExitLoop:
	return FoundFlag;
}


#ifdef SINGLE_PRECISION

//******************************************************************************
//* GetFloat - Scans a single precision floating point number from the keypad
//* or reads it from the serial port (USART). Also checks for the mantissa sign and 
//* checks for exponent the number is entered in exponential notation.  
//******************************************************************************
float GetFloat(void)
{	
	// Use GetDouble function for now

	return ((float) GetDouble());

}

#endif
										
//******************************************************************************
//* GetDouble - Scans a double precision floating point number from the keypad
//* or reads it from the serial port (USART). Also checks for the mantissa sign and 
//* checks for exponent the number is entered in exponential notation.  
//******************************************************************************
double GetDouble(void)
{													

	double DoubleFloatValue = 0.0;

   char Ch;				// Character 
	int Digit;				// Floating point digit (0..9)
	int i;					// Loop index

 

	CharacterCount = 0;
 	DoubleFloatValue = 0.0;   
	Ch = ' ';

	// Continue scanning for input until an operator: "+","-","*","/","=" is found

	//while (!Member(Ch, Operators))
	while (Member(Ch, Operators) == 0)
	{

		// Store the ASCII IEEE Floating Point number in a buffer use the 
		// operator key +,-,*,/,= as a delimiter.

		do
		{
			// Get a keystroke from the operator using the hex key pad or
			// the serial port (USART) and store it in the floating point work buffer.

#ifdef SERIAL_DRIVER

			Ch = getchar();                   // Serial Input and Output 
			//Ch = GetKey();								// Keypad Input and Serial Output  

#else

			//Ch = GetKey();                      // Keypad Input and LED or LCD output  
			Ch = getchar();                   // Serial Input and Output 

#endif

			Buffer[CharacterCount++] = Ch;

 
		//} while (!Member(Ch, Operators) && !Member(Ch, NumericChars) && 
		//	    (CharacterCount < BufferSize));
		} while ((Member(Ch, Operators) == 0) && (Member(Ch, NumericChars) == 0) && 
			    (CharacterCount < BufferSize));
		CharacterCount = CharacterCount - 1;
		

		// Convert the Integral Part of an  IEEE Floating Point number

		for (i = 0; i < CharacterCount; i++) 
		{
 			// Get a digit from the floating point work buffer 

			Ch = Buffer[i];    
			Digit = Ch - 48;    // Convert to Binary
 
 			//if (!Member(Ch, Operators)) 
			if (Member(Ch, Operators) == 0) 
			{
				DoubleFloatValue = DoubleFloatValue + (double) Digit * pow10(CharacterCount-i-1);
			}
		
		} 
		
		// Check terminal character for fractional part of floating point number or for operator (if any)

		Ch = Buffer[CharacterCount];
  
		if (Ch == '.')
		{

			CharacterCount = 0;

			do
			{
				// Get a keystroke from the operator using the hex key pad or
				// the serial port (USART) and store it in the floating point work buffer.

#ifdef SERIAL_DRIVER

				Ch = getchar();                   // Serial Input and Output 
				//Ch = GetKey();								// Keypad Input and Serial Output  

#else
				Ch = getchar();                   // Serial Input and Output 

				//Ch = GetKey();                      // Keypad Input and LED or LCD output  

#endif
				Buffer[CharacterCount++] = Ch;

 
			//} while (!Member(Ch, Operators) && !Member(Ch, NumericChars) && 
			//    (CharacterCount < BufferSize));
			} while ((Member(Ch, Operators) == 0) && (Member(Ch, NumericChars) == 0) && 
			    (CharacterCount < BufferSize));

			CharacterCount = CharacterCount - 1;

			// Convert the Fractional Part of an IEEE Floating Point number (if present)

			for (i = 0; i < CharacterCount; i++) 
			{
				// Get a keystroke from the floating point work buffer 

				Ch = Buffer[i];    
				Digit = Ch - 48;    // Convert to Binary
 
 				//if (!Member(Ch, Operators)) 
				if ((Member(Ch, Operators) == 0)) 
				{
					DoubleFloatValue = DoubleFloatValue + (double) Digit * pow10(-i-1);
				}
			
			} 

			// Check terminal character for operator (if any) or equal sign

			Ch = Buffer[CharacterCount];
 	
			if (Member(Ch, Operators))
			{
				if (Ch == '=')
				{
					TheEqualSign = Ch;
				}
				else
				{
					TheOperator = Ch;
				}
			}
	
		}
		else
		{
			// Check terminal character for operator (if any) or equal sign

			Ch = Buffer[CharacterCount];
 	
			if (Member(Ch, Operators))
			{
				if (Ch == '=')
				{
					TheEqualSign = Ch;
				}
				else
				{
					TheOperator = Ch;
				}
			}
		}

	}

	return DoubleFloatValue;
}
