//*********************************************************************************
//*                                                                               *
//*         Name:  portutil.c                                                     *
//*                                                                               *
//*       Author:  Daniel F. Ramirez                                              *
//*                                                                               *
//*         Date:  10/09/2000                                                     *
//*                                                                               *
//*  Description:                                                                 *
//*                                                                               *
//*   	This is a PIC 18C452 "C" test program which will test the following      * 
//* "C" versions of the BS2 Stamp functions: high, low, toggle, input, output,    *
//* pulsin, pulsout, shiftin, shiftout.  For test purposes, switches are          *
//* connected to Port D in order select each pin's state.                         *              
//*                                                                               * 
//*     Future upgrades to this application include: Implementing the Stamp       *
//* BS2 functions: pwm, dtmfout and freqout and diagnostics functions.            *
//*                                                                               *
//* Note:                                                                         *
//* 	1) The names of all BS2 equivalent functions use lower case.              	 *
//*                                                                               *
//*     2) The following BS2 to PIC18C452 I/O pin map is used by most of the      *
//*        Port Utility functions:                                                *                                                                                                             *
//*                                                                               *
//*       BS2        PIC        PIC           PIC                                 *
//*       PIN        PIN        PORT          BIT                                 *
//*     ---------------------------------------------                             *
//*         0         2           0             0                                 *
//*         1         3           0             1                                 *
//*         2         4           0             2                                 *
//*         3         5        A  0             3                                 *
//*         4         6           0             4                                 *
//*         5         7           0             5                                 *
//*         6        14           0             6                                 *
//*     ---------------------------------------------                             *
//*         7        33           1             0                                 *
//*         8        34           1             1                                 *
//*         9        35           1             2                                 *
//*        10        36           1             3                                 *
//*        11        37        B  1             4                                 *
//*        12        38           1             5                                 *
//*        13        39           1             6                                 *
//*        14        40           1             7                                 *
//*     ---------------------------------------------                             *
//*        15        15           2             0                                 *
//*        16        16           2             1                                 *
//*        17        17           2             2                                 *
//*        18        18           2             3                                 *
//*        19        23        C  2             4                                 *
//*        20        24           2             5                                 *
//*        21        25           2             6                                 *
//*        22        26           2             7                                 *
//*     ---------------------------------------------                             *
//*        23        19           3             0                                 *
//*        24        20           3             1                                 *
//*        25        21           3             2                                 *
//*        26        22           3             3                                 *
//*        27        27        D  3             4                                 *
//*        28        28           3             5                                 *
//*        29        29           3             6                                 *
//*        30        30           3             7                                 *
//*     ---------------------------------------------                             *
//*        31         8           4             0                                 *
//*        32         9        E  4             1                                 *
//*        33        10           4             2                                 *
//*     ---------------------------------------------                             *
//*                                                                               *
//*                                                                               *
//*********************************************************************************
     
#include "flags.h"                                        // Global flags used for MPSIM debugging 
  
//#define DISPLAY_FUNCTIONS_ENABLED
 
#ifdef PIC18C252     
#include <p18c252.h>					  // Register definitions for PIC18C252
#endif

#ifdef PIC18C452     
#include <p18c452.h>					  // Register definitions for PIC18C452
#endif

#include <stdlib.h>
#include <string.h>
#include <timers.h>                                       // Timer library functions
#include <capture.h>                                      // Capture library functions
#include <portb.h>                                        // PORTB library functions                                 
#include <usart.h>                                        // USART library functions
#include <delays.h>                                       // High resolution delay functions
#include "timeutil.h"                                     // Timer utility functions
#include "stdio.h"                                        // I/O support functions
#include "flags.h"                                        // Debug flags used for MPSIM
#include "portutil.h"                                     // Port utilities
#ifdef DISPLAY_FUNCTIONS_ENABLED
#include "display.h"                                      // Realtime display utilities
#endif

//********************************************************************************
// Constant Definitions
//********************************************************************************

//#define PORT_FUNCTIONS_TEST                             	// Test port functions
//#define BIT_TEST                                        	// Test bit operations
//#define PORT_DISPLAY_TEST                               		// Test port display
//#define REGISTER_DISPLAY_TEST                           	// Test register display 
//#define STATUS_DISPLAY_TEST											// Test Status register display     
#define DIAGNOSTICS                                        // Diagnostics 
//#define DISPLAY_SUPPORT                                     // Support for register display functions

#define SYSTEM_FREQUENCY 20         			// System frequency, Fs = 20 MHz

// Compute timer clock frequency which depends on the system frequency.  This constant is 
// used by DelayUs function. 

#define CLOCK_FREQUENCY   (SYSTEM_FREQUENCY / 4)     	//  Fc = Fs / 4 
 
#define  SCALE_FACTOR 10      			// 10 micro-second granularity for delays


#define T_2_Us     1  // 1 = (2  /  SCALE_FACTOR * CLOCK_FREQUENCY) =  2 micro-second delay
 
//********************************************************************************
// Port Utility general variables
//********************************************************************************
 
// Pin lookup table type

// Pin_T  PinLookup[34] ; 				// Pin lookup table

  
// Declare strings and numeric tables in ROM to save precious RAM space
  
// Port name and addresse tables

rom char PortName[][7]    =  {"PORTA \0", "PORTB \0", "PORTC \0", "PORTD \0", "PORTE \0"};       	// Port names table
rom int *PortAddress[]    =  { 0xF80,  0xF81,   0xF82,   0xF83,   0xF84};  		// Port address table                           

// Port direction name and address tables

rom char PortDirName[][7] =  {"TRISA \0","TRISB \0", "TRISC \0", "TRISD \0", "TRISE \0"};		// Port direction names table
rom int *PortDirAddress[] =  { 0xF92,  0xF93,   0xF94,   0xF95,   0xF96};    	// Port  direction address table                           

#ifdef DISPLAY_SUPPORT

// Register name table

rom char RegisterName[][10]    = {"TOSU \0","TOSH \0","TOSL \0","STKPTR \0","PCLATU \0","PCLATH \0","PCL \0",              
                                  "TBLPTRU \0","TBLPTRH \0","TBLPTRL \0","TABLAT \0","PRODH \0",
                                  "PRODL \0","INTCON \0","INTCON2 \0","INTCON3 \0","INDF0 \0",  
                                  "POSTINC0 \0","POSTDEC0 \0","PREINC0 \0","PLUSW0 \0","FSR0H \0", 
                                  "FSR0L \0","WREG \0","INDF1 \0","POSTINC1 \0","POSTDEC1 \0",
                                  "PREINC1 \0","PLUSW1 \0","FSR1H \0","FSR1L \0","BSR \0","INDF2 \0", 
                                  "POSTINC2 \0","POSTDEC2 \0","PREINC2 \0","PLUSW2 \0","FSR2H \0",
                                  "FSR2L \0","STATUS \0"};
 
// Register addresses

rom byte *RegisterAddress[] =  {0xFF,0xFE,0xFD,0xFC,0xFB,0xFA,0xF9,0xF8, 
                                0xF7,0xF6,0xF5,0xF4,0xF3,0xF2,0xF1,0xF0,
                                0xEF,0xEE,0xED,0xEC,0xEB,0xEA,0xE9,0xE8,
                                0xE7,0xE6,0xE5,0xE4,0xE3,0xE2,0xE1,0xE0,
                                0xDF,0xDE,0xDD,0xDC,0xDB,0xDA,0xD9,0xD8}; 

// STATUS Bits


rom char StatusBitsName[][4] = {"N \0","OV \0","Z \0","DC \0","C \0"};
rom byte StatusBitsID[] = {0x04,0x03,0x02,0x01,0x00 };  

#endif

// Capture Status Byte
struct capstatus CapStatus;

#ifdef BIT_TEST

// These variables are used for Bit tests
byte *x;
byte pos;
byte *Register;
byte Position;
byte Bit0, Bit1, Bit2; 
 
#endif 
 
#ifdef PORT_DISPLAY_TEST   
char Title1[] = "Current Port Values:";
#endif

// Declare function headers here

//#ifdef STDIO_INTERRUPTS

// Include stdio interrupt handlers for USART

//#include "stdio.inc"
//#endif



#ifdef PORTUTIL_TEST

//******************************************************************************
//* main - main function
//****************************************************************************** 
void main(void)
{
	int i,j; 						// Loop index variables
   byte PinID, PortID;			// Pin number and port (A=0..E=4)
   byte ProcessorFlag = 0;		// Processor flag (PIC=0, BS2 = 1)
   char Command;              // Menu command
   word Result;               // Data read from pulsin and shiftin

   byte Data = 0xAA;          // Used for shiftin, shiftout and pulsin

	byte DataPinID;				// Data pin for shiftin and shiftout function 
   byte ClockPinID;				// Clock pin for shiftin and shiftout function 
   byte Mode = MSBPRE;			// Mode for shiftin and shiftout function
   byte NumberOfBits = 8; 		// Number of of bits to be shifted in or out

	byte DutyCycle = 0x80;     // Duty cycle for PWM set to 50%
   byte Duration = 100;       // Duration for PWM

   //static byte TheValue = 255;

	// Note: Avoid modifiying PORT C which is used by the USART !

	// Initialize Port Utility variables

	InitializePorts();

#ifdef DIAGNOSTICS

		// Diagnostic menu

Menu:
      printf("BS2: Port Utilities \r\n");

      printf(" 0 - Input   \r\n");
      printf(" 1 - Output  \r\n");
      printf(" 2 - Reverse \r\n");
      printf(" 3 - Low     \r\n"); 
      printf(" 4 - High    \r\n");
      printf(" 5 - Toggle \r\n");
      printf(" 6 - Pulsout \r\n");      
      printf(" 7 - Pulsin  \r\n");
      printf(" 8 - Shiftout\r\n");
      printf(" 9 - Shiftin \r\n");
      printf(" A - Pwm     \r\n");
      printf(" Z - Exit    \r\n");
      

	 	Command = getchar();
	 	
      printf("Enter Processor (PIC=0, BS2 = 1: ");
      ProcessorFlag = ReadHexByte();
      printf("\r\n");
	 	
      printf("Enter Port ID: ");
      PortID = ReadHexByte();
      printf("\r\n");

      printf("Enter Pin ID: ");
      PinID = ReadHexByte();
      printf("\r\n");

		// Execute the Port Utility command selected 

		switch (Command)
		{
			case '0': 
			{ 
				// Set the pin selected to input 

            if (ProcessorFlag == 1)
            {
					input(PinID);          // BS2
				}
				else
				{
					Input(PortID, PinID);    // PIC
				}
				break;
			}

			case '1': 
			{

				// Set the pin selected to output
 
            if (ProcessorFlag == 1)
            {
					output(PinID);         // BS2
				}
				else
				{
					Output(PortID, PinID);   // PIC
				}
				break;
			}

			case '2': 
			{

				// Set the pin selected to output
 
            if (ProcessorFlag == 1)
            {
					reverse(PinID);        // BS2
				}
				else
				{
					Reverse(PortID, PinID);  // PIC
				}
				break;
			}

			case '3': 
			{
				// Set the pin selected to low
 
            if (ProcessorFlag == 1)
            {
					low(PinID);        // BS2
				}
				else
				{
					Low(PortID, PinID);  // PIC 
				}
				break;
			}

			case '4': 
			{
				// Set the pin selected to high
 
            if (ProcessorFlag == 1)
            {
					high(PinID);        // BS2
				}
				else
				{
					High(PortID, PinID);  // PIC
				}
				
				break;
			}

			case '5': 
			{
				// Toggle the pin selected 
 
            if (ProcessorFlag == 1)
            {
					toggle(PinID);          // BS2
				}
				else
				{
					Toggle(PortID, PinID);    // PIC
				}
				break;
			}

			case '6': 
			{
				// Generate a 1000 cycles square wave 
 
				for (i=0; i<1000; i++)
				{
            	if (ProcessorFlag == 1)
            	{
						pulsout(PinID, 1);        // BS2
					}
					else
					{
						Pulsout(PortID, PinID, 1);  // PIC
					}
				}
				break;
			}

			case '7': 
			{
				// Read 500 cycles of a signal    
 
				for (i=0; i<500; i++)
				{
            	if (ProcessorFlag == 1)
            	{
						pulsin(PinID, 1, &Result);             // BS2
					}
					else
					{
						Pulsin(PortID, PinID, 1, &Result);     // PIC
					}
               printf("Result = ");
               dec(Result);
               printf("\r\n");
				}
				break;
			}

			case '8': 
			{
				// Shift out 1000 cycles of 0xAA     
 
				for (i=0; i<1000; i++)
				{
            	if (ProcessorFlag == 1)
            	{
						shiftout(DataPinID, ClockPinID, Mode, Data, NumberOfBits);    // BS2
					}
					else
            	{
						Shiftout(PortID, DataPinID, ClockPinID, Mode, Data, NumberOfBits);    // PIC
					}
					
  				}
 				break;
			}
			case '9': 
			{
				// Shift in 1000 cycles of 0xAA     
 
				for (i=0; i<1000; i++)
				{
            	if (ProcessorFlag == 1)
            	{
						shiftin(DataPinID, ClockPinID, Mode, &Result, NumberOfBits);           // BS2
					}
					else
            	{
						Shiftin(PortID, DataPinID, ClockPinID, Mode, &Result, NumberOfBits);   // PIC
					}
					
               printf("Result = ");
               dec(Result);
               printf("\r\n");

  				}
				break;
			}
			case 'A': 
			{
 				// Send 1000 cycles of a PWM pulse train using a 50% duty cycle     
 
				for (i=0; i<1000; i++)
				{
            	if (ProcessorFlag == 1)
            	{
						pwm(PinID, DutyCycle, Duration);              // BS2
					}
					else
           		{
						Pwm(PortID, PinID, DutyCycle, Duration);      // PIC
					}
					
  				}
 
				break;
			}
			case 'Z': 
			{
 				// Exit debug menu     
 				
 				goto MainLoop;
 
				break;
			}		 
		}
		
		// Display contents of registers 
		
		printf("trace 2: PORTD = ");
		hex2(PORTD);
      printf("\r\n");
      
 		printf("TRISD = ");
		hex2(TRISD);
      printf("\r\n");
      pause(5000);
      
 		
      goto Menu;
      
MainLoop:

#endif

#ifdef BIT_TEST

 			// Test reverse function

		  input(1);
        output(2);
	     reverse(1);
        reverse(2);

        // Test pulsout function

        pulsout(1,1);
        pulsout(3,2);
        pulsout(5,10);

        // Test shiftout function



        // Test PinLookup function

        PinLookup(0, &PortID, &PinID);   // Port A
        PinLookup(3, &PortID, &PinID);   // Port A
        PinLookup(6, &PortID, &PinID);   // Port A
        PinLookup(7, &PortID, &PinID);   // Port B 
        PinLookup(9, &PortID, &PinID);   // Port B
        PinLookup(14, &PortID, &PinID);  // Port B
        PinLookup(15, &PortID, &PinID);  // Port C 
        PinLookup(16, &PortID, &PinID);  // Port C 
        PinLookup(22, &PortID, &PinID);  // Port C 
        PinLookup(23, &PortID, &PinID);  // Port D 
        PinLookup(24, &PortID, &PinID);  // Port D 
        PinLookup(27, &PortID, &PinID);  // Port D 
        PinLookup(30, &PortID, &PinID);  // Port D 
        PinLookup(31, &PortID, &PinID);  // Port E 
        PinLookup(33, &PortID, &PinID);  // Port E 


	// Test btoh function

        //btoh(TheValue, Hex);
        //printf(Hex);
        //printf(" \r\n");

       	// Test the DisplayPort function 

       	//DisplayPort(PORT_A);
       	//DisplayPort(PORT_B);
  
        // Set bit 2 in Port B register

        PORTB = 0;
  	     TRISB = 0x00;
        PORTD = 0;
  	     TRISD = 0x00;

       	x = &PORTB;  // Port B register
       	pos = BIT_2;
       	SetBit(x, pos); 

       	SetBit((byte *)&PORTD,  BIT_0); 
       	ClearBit((byte *)&PORTD,  BIT_1); 
       	SetBit((byte *)&PORTD,  BIT_2); 

	// PIC version

        Bit0 = GetBit((byte *)&PORTD,  BIT_0); 
       	Bit1 = GetBit((byte *)&PORTD,  BIT_1); 
       	Bit2 = GetBit((byte *)&PORTD,  BIT_2); 

	// BS2 version
 
        Bit0 = getpin(23); 
       	Bit1 = getpin(24); 
       	Bit2 = getpin(25); 

       	ClearBit((byte *)&PORTD,  BIT_0); 
       	ClearBit((byte *)&PORTD,  BIT_1); 
       	ClearBit((byte *)&PORTD,  BIT_2); 

      	Bit0 = GetBit((byte *)&PORTD,  BIT_0); 
       	Bit1 = GetBit((byte *)&PORTD,  BIT_1); 
       	Bit2 = GetBit((byte *)&PORTD,  BIT_2); 

       	ComplementBit((byte *)&PORTD,  BIT_0); 
       	ComplementBit((byte *)&PORTD,  BIT_1); 
       	ComplementBit((byte *)&PORTD,  BIT_2); 


#endif

 
 
 
   printf("\r\n PIC18C452 Port Controller Program \r\n");
   
   PORTB = 0x00;
	DDRB = 0x00;   // Set port B to output
	
	
	while (TRUE)
	{
	   for (i=0; i < 255; i++)
	   {
	      PORTB = i;
	      printf("trace 0: i = ");
	      dec(i);
	      printf("\r\n");
	      pause(500);
	   }
	}
	
#ifdef  PORT_DISPLAY_TEST   
                            
	// Test port display function here

	printf("Port display test ...\r\n");

	// Display state of ports A-E

  	PORTA = 0x00;
  	PORTB = 0x00;
  	//PORTC = 0x00;
  	PORTD = 0x00;
  	PORTE = 0x00;

 	TRISA = 0xFF;
  	TRISB = 0xFF;
  	//TRISC = 0xFF;
  	TRISD = 0xFF;
  	TRISE = 0xFF;

  
	ClearDisplay();
   
   OutTextXY(10,1,Title1);

	while (TRUE)
	{
  		DisplayPorts();
	}

/*
	// Display state of port D only

	PORTD = 0x00;															// Initialize PORT D to 0
   TRISD = 0xFF;                                          	// PORT D all inputs (switches select each pin state)

	while (TRUE)
	{
		printf("PORTD = ");
		hex2(PORTD);
      printf("\r\n");
   }

	while (TRUE)
	{
		DisplayPort(PORT_D);
      printf("\r\n");
   }
*/
#endif

#ifdef  REGISTER_DISPLAY_TEST   
                            
	// Test register display function here

   printf("Register display test ...\r\n");

	// Display state of all CPU registers

	ClearDisplay();
   OutTextXY(10,1,"Current CPU Register Values:");

	while (TRUE)
	{
		// Display all CPU registers 

      DisplayRegisters();

	}
#endif

#ifdef  STATUS_DISPLAY_TEST   
                            
	// Test Status register display function here

   printf("Status register display test ...\r\n");

	// Display state of all CPU registers

	ClearDisplay();
   OutTextXY(10,1,"Current CPU Status Register Bits:");

	while (TRUE)
	{
		// Display all the Status register bits   

      DisplayStatus();

	}
#endif

#ifdef  PORT_FUNCTIONS_TEST                             // Test port functions

//********************************************************************************
//***************************** PIC Port Function Tests ************************** 
//********************************************************************************
       
  	// Test PIC input function

 	printf("***************************************************\r\n");
  	printf("Test PIC input function ...\r\n");

  	PORTA = 0x00;
  	PORTB = 0x00;
  	PORTC = 0x00;   
  	PORTD = 0x00;
  	PORTE = 0x00;

  	TRISA = 0x00;
  	TRISB = 0x00;
  	TRISC = 0x00;
  	TRISD = 0x00;
  	TRISE = 0x00;

        // Display the ports before

	printf("PORTS (A-E) (before): \r\n ");
  	DisplayPorts();
 
        for (i=PORT_A; i<=PORT_E; i++)
	{
        	for (j=0; j<8; j++)
		{
			Input(i,j);
		}
	}

       // Display the ports after

	printf("PORTS (A-E) (after): \r\n ");
  	DisplayPorts();


 	printf("***************************************************\r\n");
  	printf("Test PIC output function ...\r\n");


  	PORTA = 0x00;
  	PORTB = 0x00;
  	PORTC = 0x00;
  	PORTD = 0x00;
  	PORTE = 0x00;

 	TRISA = 0xFF;
  	TRISB = 0xFF;
  	TRISC = 0xFF;
  	TRISD = 0xFF;
  	TRISE = 0xFF;

       // Display the ports before

	printf("PORTS (A-E) (before): \r\n ");
  	DisplayPorts();
        for (i=PORT_A; i<=PORT_E; i++)
	{
        	for (j=0; j<8; j++)
		{
			Output(i,j);
		}
	}

       	// Display the ports after

	printf("PORTS (A-E) (after): \r\n ");
  	DisplayPorts();

 	printf("***************************************************\r\n");
  	printf("Test PIC low function ...\r\n");


  	PORTA = 0xFF;
  	PORTB = 0xFF;
  	PORTC = 0xFF;
  	PORTD = 0xFF;
  	PORTE = 0xFF;

       // Display the ports before

	printf("PORTS (A-E) (before): \r\n ");
  	DisplayPorts();

        for (i=PORT_A; i<=PORT_E; i++)
	{
        	for (j=0; j<8; j++)
		{
			Low(i,j);
		}
	}
 
       // Display the ports after

	printf("PORTS (A-E) (after): \r\n ");
  	DisplayPorts();


  	// Test PIC high function

  	printf("***************************************************\r\n");
  	printf("Test PIC high function ...\r\n");

  	PORTA = 0x00;
  	PORTB = 0x00;
  	PORTC = 0x00;
  	PORTD = 0x00;
  	PORTE = 0x00;

       // Display the ports before

	printf("PORTS (A-E) (before): \r\n ");
  	DisplayPorts();

       for (i=PORT_A; i<=PORT_E; i++)
	{
        	for (j=0; j<8; j++)
		{
			High(i,j);
		}
	}

       	// Display the ports after

	printf("PORTS (A-E) (after): \r\n ");
  	DisplayPorts();

  	// Test PIC toggle function

  	printf("***************************************************\r\n");
  	printf("Test PIC toggle function ...\r\n");

  	PORTA = 0x00;
  	PORTB = 0x00;
  	PORTC = 0x00;
  	PORTD = 0x00;
  	PORTE = 0x00;

       // Display the ports before

	printf("PORTS (A-E) (before): \r\n ");
  	DisplayPorts();

       for (i=PORT_A; i<=PORT_E; i++)
	{
        	for (j=0; j<8; j++)
		{
			Toggle(i,j);
		}
	}

       // Display the ports after

	printf("PORTS (A-E) (after): \r\n ");
  	DisplayPorts();

 

//**************************************************************************
//***************************** BS2 Function Tests************************** 
//**************************************************************************

	// Test input function
  	printf("***************************************************\r\n");
  	printf("Test BS2 input function ...\r\n");

  	PORTA = 0x00;
  	PORTB = 0x00;
  	PORTC = 0x00;   
  	PORTD = 0x00;
  	PORTE = 0x00;

  	TRISA = 0x00;
  	TRISB = 0x00;
  	TRISC = 0x00;
  	TRISD = 0x00;
  	TRISE = 0x00;

       // Display the ports before

	printf("PORTS (A-E) (before): \r\n ");
  	DisplayPorts();

        for (i=0; i<MAX_PINS; i++)
	{
		input(i);
	}


       // Display the ports after

	printf("PORTS (A-E) (after): \r\n ");
  	DisplayPorts();



  	// Test output command

  	printf("***************************************************\r\n");
  	printf("Test BS2 output function ...\r\n");

  	PORTA = 0x00;
  	PORTB = 0x00;
  	PORTC = 0x00;
  	PORTD = 0x00;
  	PORTE = 0x00;

 	TRISA = 0xFF;
  	TRISB = 0xFF;
  	TRISC = 0xFF;
  	TRISD = 0xFF;
  	TRISE = 0xFF;

       // Display the ports before

	printf("PORTS (A-E) (before): \r\n ");
  	DisplayPorts();

  	for (i=0; i<MAX_PINS; i++)
  	{
	 	output(i);
 
  	}

       	// Display the ports after

	printf("PORTS (A-E) (after): \r\n ");
  	DisplayPorts();


  	// Test low function

  	printf("***************************************************\r\n");
  	printf("Test BS2 low function ...\r\n");

  	PORTA = 0xFF;
  	PORTB = 0xFF;
  	PORTC = 0xFF;
  	PORTD = 0xFF;
  	PORTE = 0xFF;

       	// Display the ports before

	printf("PORTS (A-E) (before): \r\n ");
  	DisplayPorts();


  	for (i=0; i<MAX_PINS; i++)
  	{
	 	low(i);
 
  	}

       	// Display the ports after

	printf("PORTS (A-E) (after): \r\n ");
  	DisplayPorts();


  	// Test high function

  	printf("***************************************************\r\n");
  	printf("Test BS2 high function ...\r\n");

  	PORTA = 0x00;
  	PORTB = 0x00;
  	PORTC = 0x00;
  	PORTD = 0x00;
  	PORTE = 0x00;

       	// Display the ports before

	printf("PORTS (A-E) (before): \r\n ");
  	DisplayPorts();


  	for (i=0; i<MAX_PINS; i++)
  	{
	 	high(i);
  	}

       	// Display the ports after

	printf("PORTS (A-E) (after): \r\n ");
  	DisplayPorts();

 
   	// Test BS2 toggle function

  	printf("***************************************************\r\n");
  	printf("Test BS2 toggle function ...\r\n");

  	PORTA = 0x00;
  	PORTB = 0x00;
  	PORTC = 0x00;
  	PORTD = 0x00;
  	PORTE = 0x00;

       	// Display the ports before

	printf("PORTS (A-E) (before): \r\n ");
  	DisplayPorts();


  	for (i=0; i<MAX_PINS; i++)
  	{
	 	toggle(i);
  	}

       	// Display the ports after

	printf("PORTS (A-E) (after): \r\n ");
  	DisplayPorts();

  
#endif
 
}

#endif

//******************************************************************************
//********************** PIC Low Level Functions *******************************
//******************************************************************************

//******************************************************************************
//* Initialize BS2 I/O pin lookup table
//******************************************************************************
void InitializePorts(void)
{
    int i;						// Loop index
/*
    // Initialize pin lookup table

    for (i=0; i<MAX_PINS; i++)
    {
        if ((i >=0) && (i<7))  
        {
            // Port A
            PinLookup[i].PortID = 0; 
            PinLookup[i].PinID = i;
        }
        else
        if ((i >= 7) && (i<15))
        {
	    // Port B
            PinLookup[i].PortID = 1; 
            PinLookup[i].PinID = i-7;
        }  
        else    
        if ((i >= 15) && (i<23))
        {
	    // Port C
            PinLookup[i].PortID = 2; 
            PinLookup[i].PinID = i-15;
        }
        else    
        if ((i >= 23) && (i<31))
        {
	    // Port D
            PinLookup[i].PortID = 3; 
            PinLookup[i].PinID = i-23;
        }
        else    
        if ((i >= 31) && (i<34))
        {
	    // Port E
            PinLookup[i].PortID = 4; 
            PinLookup[i].PinID = i-31;
        }
     }

*/
   
    // Initialize the register address table here
/*
    for (i = 0xFFF; i>=0xF80; i--)
    {
        *RegisterAddress[i] = i;
    }

*/
    

    // configure port A for all digital inputs

    //OpenADC(ADC_FOSC_32 &
    //        ADC_RIGHT_JUST & ADC_0ANA_0REF, ADC_CH0 & ADC_CH1 & ADC_CH2 & ADC_CH3 & ADC_CH4 & ADC_CH5 & ADC_CH6 & ADC_CH7 & ADC_INT_OFF);
    OpenADC(ADC_0ANA_0REF, ADC_INT_OFF);

    // Configure PORTB interrupt with Interrupt on change off and pull-up resistors disabled

    OpenPORTB(PORTB_CHANGE_INT_OFF & PORTB_PULLUPS_OFF);
 

    // Initialize ports

    PORTA = 0x00;                                          // Clear PORTA
    PORTB = 0x00;                                          // Clear PORTB
    //PORTC = 0x00;                                          // Clear PORTC. Some bits of PORTC are used by the USART.
    PORTD = 0x00;                                          // Clear PORTD
    PORTE = 0x00;                                          // Clear PORTE

    // Initialize port I/O direction

    TRISA = 0xFF;                                          // PORT A all inputs.
    TRISB = 0xFF;                                          // PORT B all inputs.
    //TRISC = 0x00;                                          // PORT C all outputs. Some bits of PORTC are used by the USART.
    TRISD = 0xFF;                                          // PORT D all inputs (switches select each pin state)
    TRISE = 0x00;                                          // PORT E all outputs.

	// Configure Timer 1 for pulsin function (Capture)

	OpenTimer1(TIMER_INT_OFF&T1_SOURCE_INT); 

	// Configure Timer 3 for pulsin function (Capture)

	OpenTimer3(TIMER_INT_OFF&T3_SOURCE_INT); 

 	// Configure USART for 20 MHz, 9600 Baud, 8-Bits, No parity, Asynchronous
   
   InitializeUsart();

}

//******************************************************************************
//* PinLookup - Determines the port ID and the pin ID from a BS2 pin number.
//******************************************************************************
void PinLookup (byte Pin, byte *PortID, byte *PinID)
{
	// Determine the  port ID and the pin ID from a BS2 pin number.
 
	if ((Pin >=0) && (Pin <7))
	{
		// Set pin ID (0..6) for Port A

		*PinID = Pin;  

		// Set port ID for Port A

		*PortID  = 0;
        
		}
		else
		{
         
			// Determine which port (B..E) 

			*PortID  = ((Pin - 7) / 8) + 1;

			// Determine which pin (0..7)

			*PinID = (Pin-7) % 8;       
		}          

}

//******************************************************************************
//* GetBit - Gets the bit state from the selected register using a read operation.
//* RegisterValue is any SFR register (RAM), and BitPosition is a power of 2 
//* specifying the location of the selected bit. 
//******************************************************************************
byte 		GetBit(byte *RegisterValue, byte BitPosition)
{
  	byte Bit;
  	byte Mask;                                                    // Mask used to get the selected bit state

 
  	// Copy bit state  from register to a local register value (read)

  	if ((*RegisterValue & BitPosition) == BitPosition)
  	{
     	Bit = TRUE;
  	}
  	else
  	{
     	Bit = FALSE;
  	}
  
  	return Bit;
  
}
			   

//******************************************************************************
//* SetBit - Set a bit in the selected register using a read/modify/write operation.
//* RegisterValue is any SFR register (RAM), and BitPosition is a power of 2 
//* specifying the location of the selected bit to be set.  This function assumes
//* that the pin direction is set to output.
//******************************************************************************
void SetBit(byte *RegisterValue, byte BitPosition)
{
  	byte LocalRegisterValue = 0;  				// Local register value for read/modify/write     
 
 
  	// Copy value from register to a local register value (read)

  	LocalRegisterValue = *RegisterValue;

  	// Set the selected bit specified by BitPosition.  Leave all other bits alone. (modify)

  	LocalRegisterValue = LocalRegisterValue | BitPosition;

  	// Write the local register value back to the specified FSR register (write)
	
  	*RegisterValue = LocalRegisterValue;
  
}

//******************************************************************************
//* ClearBit - Clears a bit in the selected register using a read/modify/write operation.
//* RegisterValue is any SFR register (RAM), and BitPosition is a power of 2 
//* specifying the location of the selected bit to be cleared.  This function assumes
//* that the pin direction is set to output.
//******************************************************************************
void ClearBit(byte *RegisterValue, byte BitPosition)
{
  	byte LocalRegisterValue = 0;  				// Local register value for read/modify/write     
 
  	byte Mask;                                                    // Mask used to clear the selected bit

  	Mask = ~BitPosition;						// Create the mask

  	// Copy value from register to a local register value (read)

  	LocalRegisterValue = *RegisterValue;

  	// Clear the selected bit specified by BitPosition.  Leave all other bits alone. (modify)

  	LocalRegisterValue = LocalRegisterValue & Mask;

  	// Write the local register value back to the specified FSR register (write)

  	*RegisterValue = LocalRegisterValue;
  
}

//******************************************************************************
//* ComplementBit - Complements a bit in the selected register using a read/modify/write operation.
//* RegisterValue is any SFR register (RAM), and BitPosition is a power of 2 
//* specifying the location of the selected bit to be complemented.  This function assumes
//* that the pin direction is set to output.
//******************************************************************************
void ComplementBit(byte *RegisterValue, byte BitPosition)
{
  byte LocalRegisterValue = 0;  				// Local register value for read/modify/write     
 
  
  	// Copy value from register to a local register value (read)
	
  	LocalRegisterValue = *RegisterValue;

  	// Clear the selected bit specified by BitPosition.  Leave all other bits alone. (modify)

  	LocalRegisterValue = LocalRegisterValue ^ BitPosition;

  	// Write the local register value back to the specified FSR register (write)

  	*RegisterValue = LocalRegisterValue;
  
}

//******************************************************************************
//* High - Set specified bit, PinID range (0..7), to high in selected port, 
//* PortID range (PORT_A..PORTE). This is a PIC version of  the BS2 "high" function.
//* This function assumes that the pin direction is set to output.
//******************************************************************************

void           	High(byte PortID, byte Pin)
{
 	byte PinID;     															// I/O Pin  

	// Set pin direction in selected port to output

	Output(PortID, Pin);

	// Convert pin number to power of 2 for bit position.

	PinID = 1 << Pin;
 
	// Set specified pin in selected port to high

	SetBit((byte *)PortAddress[PortID],  PinID); 
}

//******************************************************************************
//* Low - Set specified bit, PinID range (0..7), to low in selected port, 
//* PortID range (PORT_A..PORTE). This is a PIC version of the BS2 "Low" function.
//* This function assumes that the pin direction is set to output.
//******************************************************************************

void           	Low(byte PortID, byte Pin)
{
 	byte PinID;     															// I/O Pin ID 

	// Set pin direction in selected port to output

	Output(PortID, Pin);

	// Convert pin number to power of 2 for bit position.

	PinID = 1 << Pin;
 
	// Clear specified pin in selected port 

	ClearBit((byte *)PortAddress[PortID],  PinID); 
}

//******************************************************************************
//* Input - Set specified bit, PinID range (0..7), to input in selected port, 
//* PortID range (PORT_A..PORTE). This is a PIC version of the BS2 "input" function.
//******************************************************************************

void           	Input(byte PortID, byte Pin)
{
 	byte PinID;     															// I/O Pin ID

	// Convert pin number to power of 2 for bit position.

	PinID = 1 << Pin;
 
	// Set selected pin direction to input

	SetBit((byte *) PortDirAddress[PortID],  PinID); 
}
	 		 
//******************************************************************************
//* Output - Set specified bit, PinID range (0..7), toOutput in selected port, 
//* PortID range (PORT_A..PORTE). This is a PIC version of the BS2 "output" function.
//******************************************************************************
void           	Output(byte PortID, byte Pin)
{
 	byte PinID;     															// I/O Pin  ID
  
	// Convert pin number to power of 2 for bit position.

	PinID = 1 << Pin;
 
	// Set selected pin direction to output

	ClearBit((byte *) PortDirAddress[PortID],  PinID); 

}
			 
//******************************************************************************
//* Toggle - Toggle  specified bit, PinID range (0..7), in selected port, 
//* PortID range (PORT_A..PORTE). This is a PIC version of the BS2 "toggle" function.
//******************************************************************************
void            Toggle(byte PortID, byte Pin)
{
 	byte PinID;     															// I/O Pin ID 

   // Convert pin number to power of 2 for bit position.

   PinID = 1 << Pin;
 
	// Toggle selected pin 

   ComplementBit((byte *) PortAddress[PortID],  PinID); 
}  

//******************************************************************************
//* Reverse - Makes selected input pins outputs and output pins inputs.  Similar
//* to BS2 reverse function.
//******************************************************************************					
void           Reverse(byte PortID, byte Pin)
{  
 
	byte PinID;      
                                                               // Pin ID
 
	// Convert pin number to power of 2 for bit position.

	PinID = 1 << Pin;

   // Check to see if pin direction is input or output, then change the direction
   // to the opposite.

   if (GetBit((byte *) PortDirAddress[PortID],  PinID) == INPUT)
   {
		// Set selected pin direction to output

		ClearBit((byte *) PortDirAddress[PortID],  PinID); 
	}
	else
	{
		// Set selected pin direction to input

		SetBit((byte *) PortDirAddress[PortID],  PinID); 
	}
}

//******************************************************************************
//* Pulsout - This is a PIC version of the pulsout function, that outputs a timed 
//* pulse.  Pin will be made an output opposite of its OUTx value for Time*2us.  
//* Then Pin's value will revert to its original value in output mode.
//******************************************************************************   
void 		Pulsout(byte PortID, byte Pin, word Time)       								  
{	
 	byte PinID;     															// I/O Pin ID
                            
	// Make pin an output

   Output(PortID, Pin);
                                   
	// Convert pin number to power of 2 for bit position.

	PinID = 1 << Pin;

   // Complement the selected pin 

	ComplementBit((byte *) PortAddress[PortID],  PinID); 

   // Delay for approximatly Time increments of two micro-seconds (@ 20 Mhz and a granularity of 10)

   Delay10TCYx(Time * T_2_Us);

	// Complement the selected pin on port B to restore original value

	ComplementBit((byte *) PortAddress[PortID],  PinID); 

}
//******************************************************************************
//* Pulsin - PIC version of pulsin function.  Currently this function is limited
//* to capturing input from PIC/Stamp BS2/BSX pins 16 (RC1/CCP2), 17 (RC2/CCP1) and 
//* pin 36 (RB3/CCP2 alternate, PIC18C452 only).  It so happens that pin numbering for 
//* pins 16 and 17 are the same for the PIC and the Stamp.  Timers 1 and 3 may be
//* used to count pulses.
//****************************************************************************** 
void 		Pulsin(byte PortID, byte Pin, byte State, word *Result)       				  
{
 	byte PinID;     															// Pin ID 

	// Make pin an input

   //input(Pin);
   
   PORTCbits.RC1 =  INPUT;                 		   // RC1/CCP2
   PORTCbits.RC2 =  INPUT;                 		   // RC2/CCP1
  
	// Determine which port (A..E) and pin (0..7) 

 	//PinLookup (Pin, &PortID, &PinID);

	if (Pin == 16)
	{
		// 
		// Use Capture 2 register (CCP2) which has been assigned to PIC18C452 Pin 16 (RC1/CCP2)

		if (State == 1)
		{

			// Configure Capture 2  for rising edge (0 -> 1 transitions)

			OpenCapture2(C2_EVERY_RISE_EDGE);
		}
		else
		{
			// Configure Capture 2 for falling edge (1 -> 0 transitions)

			OpenCapture2(C2_EVERY_FALL_EDGE);
		}

		// Configure Timer3
		OpenTimer3(TIMER_INT_OFF&T3_SOURCE_INT);
 
		while(!PIR2bits.CCP2IF);					// Wait for event

      PIR2bits.CCP2IF = 0;                   // Clear the capture interrupt flag bit 
      //PIE2bits.CCP2IE = 0;                   //  Clear the capture interrupt enable bit  (CloseCapture2 function does this !)

		*Result = ReadCapture2();				// read result from Capture 2 register
 
		if(!CapStatus.Cap2OVF)
		{
			CloseCapture2();					// Close Capture 2  
		}
		else
		{
			*Result = 0;						// Capture 1 register overflow
		}
    }
	else
	if (Pin == 17)
	{
		// Use Capture 1 register (CCP1) which is assigned to PIC18C452 Pin 17 (RC2/CCP1) 

		if (State == 1)
		{

			// Configure Capture1 for rising edge (0 -> 1 transition)

			OpenCapture1(C1_EVERY_RISE_EDGE);
		}
		else
		{
			// Configure Capture1 for falling edge (1 -> 0 transition)

			OpenCapture1(C1_EVERY_FALL_EDGE);
		}

		// Configure Timer3
	   OpenTimer3(TIMER_INT_OFF&T3_SOURCE_INT);
 
		while(!PIR1bits.CCP1IF);					// Wait for event

      PIR1bits.CCP1IF = 0;                   // Clear the capture interrupt flag bit 
      //PIE1bits.CCP1IE = 0;                   //  Clear the capture interrupt enable bit  (CloseCapture1 function does this !)
           
		*Result = ReadCapture1();				// read result from capture 1 register
 
		if(!CapStatus.Cap1OVF)
		{
			CloseCapture1();					// Close capture 1  
		}
		else
		{
			*Result = 0;						// Capture 1 register overflow
		}
  
	}
	else 
	if (Pin == 36)
	{
		// Use Capture 2 alternate registers (CCP2)
	}
		


}  

//******************************************************************************
//* Shiftin - PIC version of shiftin function.  
//****************************************************************************** 
void     Shiftin(byte PortID, byte DataPin, byte ClockPin, byte Mode, word *Result, byte NumberBits)  	
{
	byte	i;													// Loop index
   byte DataPortID;										// Data port ID
	byte DataPinID;										// Data pin ID
   byte ClockPortID;										// Clock port ID
	byte ClockPinID;										// Clock pin ID

   word LastBit;           							// Last bit position (power of 2) 
 	
	*Result = 0;

	// Force Data pin an input

   Input(PortID, DataPin);

	// Force Clock pin an output

   Output(PortID, ClockPin);  

	// Convert Data pin number to power of 2 for bit position.

	DataPinID = 1 << DataPin;	

	// Convert Clock pin number to power of 2 for bit position.

	ClockPinID = 1 << ClockPin;	

	for (i = 0; i < NumberBits;  i++)
  	{
    
		switch (Mode)
    	{ 
			case MSBPRE:
			{
				// Pre-clock/msb-first mode for latching bits on falling edge (1-0 transition).

        		// Sample the MSB bit first

				*Result = *Result | GetBit((byte *) PortAddress[DataPortID],  DataPinID); 
				*Result = *Result << 1; 

				// Clock in the data after sampling. Assume Pre-clock mode for latching bits on 
				// falling edge (1-0 transition)

				// Set the clock line

				//high(ClockPin);
				SetBit((byte *)PortAddress[PortID],  ClockPinID); 

				// Clear the clock line

				//low(ClockPin);
				ClearBit((byte *) PortAddress[PortID],  ClockPinID); 

				break;
			}

			case LSBPRE:
      	{

				// Pre-clock/lsb-first mode for latching bits on falling edge (1-0 transition)

        		// Sample the LSB bit first

				*Result = *Result | (GetBit((byte *) PortAddress[DataPortID],  DataPinID) << NumberBits); 
				*Result = *Result >> 1; 
  
				// Clock in the data after sampling. Assume Pre-clock mode for latching bits on 
				// falling edge (1-0 transition)

				// Set the clock line

				//high(ClockPin);
				SetBit((byte *)PortAddress[PortID],  ClockPinID); 

				// Clear the clock line

				//low(ClockPin);
				ClearBit((byte *) PortAddress[PortID],  ClockPinID); 

				break;
			}
    
			case MSBPOST:
			{ 

				// Post-clock/msb-first mode for latching bits on rising edge (0-1 transition)

         	// Sample the MSB bit first

				*Result = *Result | GetBit((byte *) PortAddress[DataPortID],  DataPinID); 
				*Result = *Result << 1; 

				// Clock in the data 

				// Clear the clock line

				//low(ClockPin);
				ClearBit((byte *) PortAddress[PortID],  ClockPinID); 

 				// Set the clock line

				//high(ClockPin);
				SetBit((byte *)PortAddress[PortID],  ClockPinID); 

				break;
			}

			case LSBPOST:
			{ 

				// Post-clock/lsb-first mode for latching bits on rising edge (0-1 transition)

       		// Sample the LSB bit first

				*Result = *Result | (GetBit((byte *) PortAddress[PortID],  DataPinID) << NumberBits); 
				*Result = *Result >> 1; 
   

 				// Clock in the data 

				// Clear the clock line

				//low(ClockPin);
				ClearBit((byte *) PortAddress[PortID],  ClockPinID); 

 				// Set the clock line

				//high(ClockPin);
				SetBit((byte *)PortAddress[PortID],  ClockPinID); 

        break;
		}
    }
	}
}   
                   
//******************************************************************************
//* Shiftout - PIC version of shiftout function that shift bits out synchronously.  
//****************************************************************************** 
void     Shiftout(byte PortID, byte DataPin, byte ClockPin, byte Mode, word Data, byte NumberBits)  
{
   byte	i;													// Loop index
   byte DataPortID;										// Data port ID
	byte DataPinID;										// Data pin ID
   byte ClockPortID;										// Clock port ID
	byte ClockPinID;										// Clock pin ID

   word LastBit;           							// Last bit position (power of 2) 
 
/*   
   // MPSIM Debug logic

   static byte LocalDataPin;
   static byte LocalClockPin;
   static byte LocalMode;  
   static word LocalData;
   static byte LocalNumberBits;

   // Copy parameters to local variables so that they may be examined by MPSIM

   LocalDataPin = DataPin;
   LocalClockPin = ClockPin;
   LocalMode = Mode;
   LocalData = Data;
   LocalNumberBits = NumberBits;
*/

	// Force Data pin an output

   Output(PortID, DataPin);

	// Force Clock pin an output

   Output(PortID, ClockPin);  
                                
	// Convert Data pin number to power of 2 for bit position.

	DataPinID = 1 << DataPin;	

	// Convert Clock pin number to power of 2 for bit position.

	ClockPinID = 1 << ClockPin;	

	if (Mode == MSBFIRST)
	{
 		// Send data bits to the target using MSB..LSB order.  LastBit represents the most significant bit (MSB).  

 		LastBit = ((word) 1 << (NumberBits - 1));

		for (i = 0; i < NumberBits; i++)
		{
/*
			// Copy parameters to local variables so that they may be examined by MPSIM

   		LocalDataPin = DataPin;
   		LocalClockPin = ClockPin;
   		LocalMode = Mode;
   		LocalData = Data;
   		LocalNumberBits = NumberBits;
*/

			if ((Data & LastBit) == LastBit)
			{
				// Send a logic-1 to target device

				//high(DataPin);
 
				SetBit((byte *)PortAddress[PortID],  DataPinID); 

			} 
			else
			{
				// Send a logic-0 to target device

				//low(DataPin);
 
				ClearBit((byte *) PortAddress[PortID],  DataPinID); 			
			}
   			
   		// Delay for approximatly two micro-seconds (@ 20 Mhz and a granularity of 10)

   		//Delay10TCYx(T_2_Us);
   		
			// Assume Pre-clock mode for latching bits on falling edge (1-0 transition)

			// Set the clock line

			//high(ClockPin);
			SetBit((byte *)PortAddress[PortID],  ClockPinID); 
 
   		// Delay for approximatly two micro-seconds (@ 20 Mhz and a granularity of 10)

   		//Delay10TCYx(T_2_Us);
 
			// Clear the clock line

      	//low(ClockPin);
			ClearBit((byte *) PortAddress[PortID],  ClockPinID); 
			
  	  	   // Delay for approximatly two micro-seconds (@ 20 Mhz and a granularity of 10)

   		//Delay10TCYx(T_2_Us);
 
			// Get next bit by shifting Data word to the left one bit

			Data = Data << 1;                            
		}
   } 
	else
	{

		// Send data bits to the target using LSB..MSB order  

		for (i = 0; i < NumberBits; i++)
		{
			if (Data & BIT_0 == 1)
			{
				// Send a logic-1 to target device

				//high(DataPin);
				SetBit((byte *)PortAddress[PortID],  DataPinID); 

			} 
			else
			{
				// Send a logic-0 to target device

				//low(DataPin);
				ClearBit((byte *) PortAddress[PortID],  DataPinID); 			
			}
			
   		// Delay for approximatly two micro-seconds (@ 20 Mhz and a granularity of 10)

   		//Delay10TCYx(T_2_Us);
 
			// Assume Pre-clock mode for latching bits on falling edge (1-0 transition)

			// Set the clock line

			//high(ClockPin);
			SetBit((byte *)PortAddress[PortID],  ClockPinID); 
			
   		// Delay for approximatly two micro-seconds (@ 20 Mhz and a granularity of 10)

   		//Delay10TCYx(T_2_Us);
 
			// Clear the clock line

      	//low(ClockPin);
			ClearBit((byte *) PortAddress[PortID],  ClockPinID); 			
  
  	   	// Delay for approximatly two micro-seconds (@ 20 Mhz and a granularity of 10)

   		//Delay10TCYx(T_2_Us);
 
			// Get next bit by shifting Data word to the right one bit

			Data = Data >> 1;                            
		}
	}
 
}

//******************************************************************************
//************************ STAMP BS2 Bit I/O Functions *************************
//******************************************************************************

//******************************************************************************
//* high - Set specified BS2 I/O pin range (0..33) to high. Similar
//* to BS2 high function.  The pin direction is forced to output.
//******************************************************************************
void           high(byte Pin)
{
	byte PortID;                                                                    // Port ID
	byte PinID;                                                                     // Pin ID

	// Set pin direction to output

	output(Pin);

	// Determine which port (A..E) and pin (0..7) 

 	PinLookup (Pin, &PortID, &PinID);

	// Convert pin number to power of 2 for bit position.

	PinID = 1 << PinID;
 
	// Set selected pin to high

	SetBit((byte *)PortAddress[PortID],  PinID); 
 	
}
 
//******************************************************************************
//* setpin - Set specified BS2 I/O pin range (0..33) to high. Similar
//* to BS2 high function except that it does not force the pin direction.
//******************************************************************************
void           setpin(byte Pin)
{
	byte PortID;                                                                    // Port ID
	byte PinID;                                                                     // Pin ID

	// Determine which port (A..E) and pin (0..7) 

 	PinLookup (Pin, &PortID, &PinID);

	// Convert pin number to power of 2 for bit position.

   PinID = 1 << PinID;
 
	// Set selected pin to high

   SetBit((byte *)PortAddress[PortID],  PinID); 
 	
}

//******************************************************************************
//* low - set specified BS2 I/O pin range (0..33) to low in selected port. Similar to 
//* BS2 low function.
//******************************************************************************                        
void           low(byte Pin)
{
	byte PortID;                                                                    // Port ID
	byte PinID;                                                                     // Pin ID

	// Set pin direction to output

	output(Pin);

	// Determine which port (A..E) and pin (0..7) 

 	PinLookup (Pin, &PortID, &PinID);

	// Convert pin number to power of 2 for bit position.

	PinID = 1 << PinID;

	// Clear selected pin  

	ClearBit((byte *) PortAddress[PortID],  PinID); 

  
}
 
//******************************************************************************
//* clearpin - Clear the  specified BS2 I/O pin range (0..33) to low in 
//* selected port. Similar to  BS2 low function except that it does not force the
//* pin direction.
//******************************************************************************                        
void           clearpin(byte Pin)
{
	byte PortID;                                                                    // Port ID
	byte PinID;                                                                     // Pin ID

	// Determine which port (A..E) and pin (0..7) 

 	PinLookup (Pin, &PortID, &PinID);

	// Convert pin number to power of 2 for bit position.

	PinID = 1 << PinID;

	// Clear selected pin  

	ClearBit((byte *) PortAddress[PortID],  PinID); 

}
 
//******************************************************************************
//* toggle - Complement specified BS2 I/O pin range (0..33) to low in selected port. Similar to 
//* BS2 toggle function.
//******************************************************************************                        
void           toggle(byte Pin)
{
	byte PortID;                                                                    // Port ID
	byte PinID;                                                                     // Pin ID

	// Set pin direction to output

	output(Pin);

	// Determine which port (A..E) and pin (0..7) 

 	PinLookup (Pin, &PortID, &PinID);

	// Convert pin number to power of 2 for bit position.

	PinID = 1 << PinID;

	// Toggle selected pin 

	ComplementBit((byte *) PortAddress[PortID],  PinID); 

  
}
//******************************************************************************
//* getpin - Gets the current value of the specified BS2 I/O pin range (0..33). 
//* Similar to BS2 Pin functions.  This function assumes that the pin direction 
//* is set to input.
//******************************************************************************					

byte 		getpin(byte Pin)				 
{
	byte PinValue;					             // The I/O pin value
	byte PortID;                                                 // Port ID
	byte PinID;                                                  // Pin ID 

	// Determine which port (A..E) and pin (0..7) 

 	PinLookup (Pin, &PortID, &PinID);

	// Convert pin number to power of 2 for bit position.

	PinID = 1 << PinID;

	// Read selected pin value

	PinValue = GetBit((byte *) PortAddress[PortID],  PinID); 

	return (PinValue);

}    

//******************************************************************************
//* input - Set specified BS2 I/O pin range (0..33) to input in selected port.  
//* Similar to BS2 input function.
//******************************************************************************					
void           input(byte Pin)
{ 
	byte PortID;                                                                    // Port ID
	byte PinID;                                                                     // Pin ID

	// Determine which port (A..E) and pin (0..7) 

 	PinLookup (Pin, &PortID, &PinID);

	// Convert pin number to power of 2 for bit position.

	PinID = 1 << PinID;

	// Set selected pin direction to input

	SetBit((byte *) PortDirAddress[PortID],  PinID); 

}

//******************************************************************************
//* output - Set specified BS2 I/O pin range (0..33) to output  in selected port. 
//* Similar to BS2 output function.
//******************************************************************************					
void           output(byte Pin)
{
	byte PortID;                                           // Port ID
	byte PinID;      
                                                               // Pin ID
	// Determine which port (A..E) and pin (0..7) 

 	PinLookup (Pin, &PortID, &PinID);

	// Convert pin number to power of 2 for bit position.

	PinID = 1 << PinID;

	// Set selected pin direction to output

	ClearBit((byte *) PortDirAddress[PortID],  PinID); 


}

//******************************************************************************
//* reverse - Makes selected input pins outputs and output pins inputs.  Similar
//* to BS2 reverse function.
//******************************************************************************					
void           reverse(byte Pin)
{  
	byte PortID;                                           		// Port ID
	byte PinID;      
                                                               // Pin ID
	// Determine which port (A..E) and pin (0..7) 

 	PinLookup (Pin, &PortID, &PinID);

	// Convert pin number to power of 2 for bit position.

	PinID = 1 << PinID;

   // Check to see if pin direction is input or output, then change the direction
   // to the opposite.

   if (GetBit((byte *) PortDirAddress[PortID],  PinID) == INPUT)
   {
		// Set selected pin direction to output

		ClearBit((byte *) PortDirAddress[PortID],  PinID); 
	}
	else
	{
		// Set selected pin direction to input

		SetBit((byte *) PortDirAddress[PortID],  PinID); 
	}
}

//******************************************************************************
//* Count - PIC version of count function.  
//****************************************************************************** 
void 		Count(byte PortID, byte Pin, word Time, word Variable)       				
{
	// Force Data pin an input

   input(Pin);

}
//******************************************************************************
//* Freqout - PIC version of freqout function.  
//****************************************************************************** 
void     Freqout(byte PortID, byte Pin, word Duration, word Freq1, word Freq2)     		 
{
}

//******************************************************************************
//* Pwm - PIC version of pwm function.  
//****************************************************************************** 
void     Pwm(byte PortID, byte Pin, byte DutyCycle, word Duration)					
{
}

//******************************************************************************
//******************************************************************************
//*                       Stamp BS2 Advanced functions
//******************************************************************************
//******************************************************************************

//******************************************************************************
//* pulsin - BS2 version of pulsin function.  Currently this function is limited
//* to capturing input from PIC/Stamp BS2/BSX pins 16 (RC1/CCP2), 17 (RC2/CCP1) and 
//* pin 36 (RB3/CCP2 alternate, PIC18C452 only).  It so happens that pin numbering for 
//* pins 16 and 17 are the same for the PIC and the Stamp.  Timers 1 and 3 may be
//* used to count pulses.
//****************************************************************************** 
void 		pulsin(byte Pin, byte State, word *Result)       				  
{
	byte PortID;                                           		// Port ID
	byte PinID;     															// Pin ID 

	// Make pin an input

   //input(Pin);
   
   PORTCbits.RC1 =  INPUT;                 		   // RC1/CCP2
   PORTCbits.RC2 =  INPUT;                 		   // RC2/CCP1
  
	// Determine which port (A..E) and pin (0..7) 

 	//PinLookup (Pin, &PortID, &PinID);

	if (Pin == 16)
	{
		// 
		// Use Capture 2 register (CCP2) which has been assigned to PIC18C452 Pin 16 (RC1/CCP2)

		if (State == 1)
		{

			// Configure Capture 2  for rising edge (0 -> 1 transitions)

			OpenCapture2(C2_EVERY_RISE_EDGE);
		}
		else
		{
			// Configure Capture 2 for falling edge (1 -> 0 transitions)

			OpenCapture2(C2_EVERY_FALL_EDGE);
		}

		// Configure Timer3
		OpenTimer3(TIMER_INT_OFF&T3_SOURCE_INT);
 
		while(!PIR2bits.CCP2IF);					// Wait for event

      PIR2bits.CCP2IF = 0;                   // Clear the capture interrupt flag bit 
      //PIE2bits.CCP2IE = 0;                   //  Clear the capture interrupt enable bit  (CloseCapture2 function does this !)

		*Result = ReadCapture2();				// read result from Capture 2 register
 
		if(!CapStatus.Cap2OVF)
		{
			CloseCapture2();					// Close Capture 2  
		}
		else
		{
			*Result = 0;						// Capture 1 register overflow
		}
    }
	else
	if (Pin == 17)
	{
		// Use Capture 1 register (CCP1) which is assigned to PIC18C452 Pin 17 (RC2/CCP1) 

		if (State == 1)
		{

			// Configure Capture1 for rising edge (0 -> 1 transition)

			OpenCapture1(C1_EVERY_RISE_EDGE);
		}
		else
		{
			// Configure Capture1 for falling edge (1 -> 0 transition)

			OpenCapture1(C1_EVERY_FALL_EDGE);
		}

		// Configure Timer3
	   OpenTimer3(TIMER_INT_OFF&T3_SOURCE_INT);
 
		while(!PIR1bits.CCP1IF);					// Wait for event

      PIR1bits.CCP1IF = 0;                   // Clear the capture interrupt flag bit 
      //PIE1bits.CCP1IE = 0;                   //  Clear the capture interrupt enable bit  (CloseCapture1 function does this !)
           
		*Result = ReadCapture1();				// read result from capture 1 register
 
		if(!CapStatus.Cap1OVF)
		{
			CloseCapture1();					// Close capture 1  
		}
		else
		{
			*Result = 0;						// Capture 1 register overflow
		}
  
	}
	else 
	if (Pin == 36)
	{
		// Use Capture 2 alternate registers (CCP2)
	}
		


}  
                        
//******************************************************************************
//* pulsout - This is a BS2 version of the pulsout function, that outputs a timed 
//* pulse.  Pin will be made an output opposite of its OUTx value for Time*2us.  
//* Then Pin's value will revert to its original value in output mode.
//******************************************************************************   
void 		pulsout(byte Pin, word Time)       								  
{	
	byte PortID;                                           		// Port ID
	byte PinID;     															// Pin ID 
                            
	// Make pin an output

   output(Pin);
                                   
	// Determine which port (A..E) and pin (0..7) 

 	PinLookup (Pin, &PortID, &PinID);
/* 	
 	printf("trace 1: pulsout :  Pin = ");
 	dec(Pin);
 	printf("  PortID = ");
 	dec(PortID);
 	printf("  PinID = ");
 	dec(PinID);
 	printf("\r\n");
*/ 	

	// Convert pin number to power of 2 for bit position.

	PinID = 1 << PinID;

   // Complement the selected pin 

	ComplementBit((byte *) PortAddress[PortID],  PinID); 

   // Delay for approximatly Time increments of two micro-seconds (@ 20 Mhz and a granularity of 10)

   Delay10TCYx(Time * T_2_Us);

	// Complement the selected pin on port B to restore original value

	ComplementBit((byte *) PortAddress[PortID],  PinID); 

}


                
//******************************************************************************
//* count - BS2 version of count function.  
//****************************************************************************** 
void 		count(byte Pin, word Time, word Variable)       				
{
	// Force Data pin an input

   input(Pin);

}
  
//******************************************************************************
//* shiftin - BS2 version of shiftin function.  
//****************************************************************************** 
void     shiftin(byte DataPin, byte ClockPin, byte Mode, word *Result, byte NumberBits)  	
{
	byte	i;													// Loop index
   byte DataPortID;										// Data port ID
	byte DataPinID;										// Data pin ID
   byte ClockPortID;										// Clock port ID
	byte ClockPinID;										// Clock pin ID

   word LastBit;           							// Last bit position (power of 2) 
 	
	*Result = 0;

	// Force Data pin an input

   input(DataPin);

	// Force Clock pin an output

   output(ClockPin);  

	// Determine which port (A..E) and pin (0..7) 

 	PinLookup (DataPin, &DataPortID, &DataPinID);

	// Convert Data pin number to power of 2 for bit position.

	DataPinID = 1 << DataPinID;	

	// Determine which port (A..E) and pin (0..7) 

 	PinLookup (ClockPin, &ClockPortID, &ClockPinID);

	// Convert Clock pin number to power of 2 for bit position.

	ClockPinID = 1 << ClockPinID;	

	for (i = 0; i < NumberBits;  i++)
  	{
    
		switch (Mode)
    	{ 
			case MSBPRE:
			{
				// Pre-clock/msb-first mode for latching bits on falling edge (1-0 transition).

        		// Sample the MSB bit first

				*Result = *Result | GetBit((byte *) PortAddress[DataPortID],  DataPinID); 
				*Result = *Result << 1; 

				// Clock in the data after sampling. Assume Pre-clock mode for latching bits on 
				// falling edge (1-0 transition)

				// Set the clock line

				//high(ClockPin);
				SetBit((byte *)PortAddress[ClockPortID],  ClockPinID); 

				// Clear the clock line

				//low(ClockPin);
				ClearBit((byte *) PortAddress[ClockPortID],  ClockPinID); 

				break;
			}

			case LSBPRE:
      	{

				// Pre-clock/lsb-first mode for latching bits on falling edge (1-0 transition)

        		// Sample the LSB bit first

				*Result = *Result | (GetBit((byte *) PortAddress[DataPortID],  DataPinID) << NumberBits); 
				*Result = *Result >> 1; 
  
				// Clock in the data after sampling. Assume Pre-clock mode for latching bits on 
				// falling edge (1-0 transition)

				// Set the clock line

				//high(ClockPin);
				SetBit((byte *)PortAddress[ClockPortID],  ClockPinID); 

				// Clear the clock line

				//low(ClockPin);
				ClearBit((byte *) PortAddress[ClockPortID],  ClockPinID); 

				break;
			}
    
			case MSBPOST:
			{ 

				// Post-clock/msb-first mode for latching bits on rising edge (0-1 transition)

         	// Sample the MSB bit first

				*Result = *Result | GetBit((byte *) PortAddress[DataPortID],  DataPinID); 
				*Result = *Result << 1; 

				// Clock in the data 

				// Clear the clock line

				//low(ClockPin);
				ClearBit((byte *) PortAddress[ClockPortID],  ClockPinID); 

 				// Set the clock line

				//high(ClockPin);
				SetBit((byte *)PortAddress[ClockPortID],  ClockPinID); 

				break;
			}

			case LSBPOST:
			{ 

				// Post-clock/lsb-first mode for latching bits on rising edge (0-1 transition)

       		// Sample the LSB bit first

				*Result = *Result | (GetBit((byte *) PortAddress[DataPortID],  DataPinID) << NumberBits); 
				*Result = *Result >> 1; 
   

 				// Clock in the data 

				// Clear the clock line

				//low(ClockPin);
				ClearBit((byte *) PortAddress[ClockPortID],  ClockPinID); 

 				// Set the clock line

				//high(ClockPin);
				SetBit((byte *)PortAddress[ClockPortID],  ClockPinID); 

        break;
		}
    }
	}
}   
                   
//******************************************************************************
//* shiftout - BS2 version of shiftout function that shift bits out synchronously.  
//****************************************************************************** 
void     shiftout(byte DataPin, byte ClockPin, byte Mode, word Data, byte NumberBits)  
{
	static byte	i;													// Loop index
   static byte DataPortID;										// Data port ID
	static byte DataPinID;										// Data pin ID
   static byte ClockPortID;										// Clock port ID
	static byte ClockPinID;										// Clock pin ID

   static word LastBit;           							// Last bit position (power of 2) 
 
/*   
   // MPSIM Debug logic

   static byte LocalDataPin;
   static byte LocalClockPin;
   static byte LocalMode;  
   static word LocalData;
   static byte LocalNumberBits;

   // Copy parameters to local variables so that they may be examined by MPSIM

   LocalDataPin = DataPin;
   LocalClockPin = ClockPin;
   LocalMode = Mode;
   LocalData = Data;
   LocalNumberBits = NumberBits;
*/

	// Force Data pin an output

   output(DataPin);

	// Force Clock pin an output

   output(ClockPin);  
                                 
	// Determine which port (A..E) and pin (0..7) 

 	PinLookup (DataPin, &DataPortID, &DataPinID);

	// Convert Data pin number to power of 2 for bit position.

	DataPinID = 1 << DataPinID;	

	// Determine which port (A..E) and pin (0..7) 

 	PinLookup (ClockPin, &ClockPortID, &ClockPinID);
/* 	
 	printf("trace 0: shiftout: ClockPin = ");
 	dec(ClockPin);
 	printf("  ClockPortID = ");
 	dec(ClockPortID);
 	printf("  ClockPinID = ");
 	dec(ClockPinID);
 	printf("\r\n");
*/ 	

	// Convert Clock pin number to power of 2 for bit position.

	ClockPinID = 1 << ClockPinID;	

	if (Mode == MSBFIRST)
	{
 		// Send data bits to the target using MSB..LSB order.  LastBit represents the most significant bit (MSB).  

 		LastBit = ((word) 1 << (NumberBits - 1));

		for (i = 0; i < NumberBits; i++)
		{
/*
			// Copy parameters to local variables so that they may be examined by MPSIM

   		LocalDataPin = DataPin;
   		LocalClockPin = ClockPin;
   		LocalMode = Mode;
   		LocalData = Data;
   		LocalNumberBits = NumberBits;
*/

			if ((Data & LastBit) == LastBit)
			{
				// Send a logic-1 to target device

				//high(DataPin);
 
				SetBit((byte *)PortAddress[DataPortID],  DataPinID); 
			} 
			else
			{
				// Send a logic-0 to target device

				//low(DataPin);
 
				ClearBit((byte *) PortAddress[DataPortID],  DataPinID); 			
			}

			// Assume Pre-clock mode for latching bits on falling edge (1-0 transition)

			// Set the clock line

			//high(ClockPin);
			SetBit((byte *)PortAddress[ClockPortID],  ClockPinID); 

			// Clear the clock line

      	//low(ClockPin);
			ClearBit((byte *) PortAddress[ClockPortID],  ClockPinID); 			

			// Get next bit by shifting Data word to the left one bit

			Data = Data << 1;                            
		}
   } 
	else
	{

		// Send data bits to the target using LSB..MSB order  

		for (i = 0; i < NumberBits; i++)
		{
			if (Data & BIT_0 == 1)
			{
				// Send a logic-1 to target device

				//high(DataPin);
				SetBit((byte *)PortAddress[DataPortID],  DataPinID); 

			} 
			else
			{
				// Send a logic-0 to target device

				//low(DataPin);
				ClearBit((byte *) PortAddress[DataPortID],  DataPinID); 			
			}

			// Assume Pre-clock mode for latching bits on falling edge (1-0 transition)

			// Set the clock line

			//high(ClockPin);
			SetBit((byte *)PortAddress[ClockPortID],  ClockPinID); 

			// Clear the clock line

      	//low(ClockPin);
			ClearBit((byte *) PortAddress[ClockPortID],  ClockPinID); 			

			// Get next bit by shifting Data word to the right one bit

			Data = Data >> 1;                            
		}
	}
 
}
                 
//******************************************************************************
//* freqout - BS2 version of freqout function.  
//****************************************************************************** 
void     freqout(byte Pin, word Duration, word Freq1, word Freq2)     		 
{
}

//******************************************************************************
//* pwm - BS2 version of pwm function.  
//****************************************************************************** 
void     pwm(byte Pin, byte DutyCycle, word Duration)					
{
}

#ifdef DISPLAY_FUNCTIONS_ENABLED

//******************************************************************************
//* DisplayPorts - Display the contents of the all PIC I/O ports on the host's screen.
//****************************************************************************** 
void DisplayPorts(void)
{
	byte i;

	// Display each port 

	for (i=PORT_A; i<= PORT_E; i++)
	{
		GotoXY(10,2+i);
      DisplayPort(i);
	}
  	
}
			
//******************************************************************************
//* DisplayPort - Display the contents of the selected PIC I/O port.
//****************************************************************************** 

void DisplayPort(byte PortID)
{
	byte PortValue;                    	// Contents of selected port 
   
	// Read the selected port 

	PortValue =   *PortAddress[PortID];

	// Display the port contents in hex

   printf(PortName[PortID]);
  	printf("  = ");
   hex2(PortValue);
   printf("\r\n");

}

//******************************************************************************
//* DisplayDirPorts - Display the contents of the all PIC I/O port direction 
//* registers  on the host's screen.
//****************************************************************************** 
void DisplayDirPorts(void)
{
	byte i;

	// Display each port 

	for (i=PORT_A; i<= PORT_E; i++)
	{
		GotoXY(10,2+i);
      DisplayDirPort(i);
	}
  	
}

//******************************************************************************
//* DisplayDirPort - Display the contents of the selected PIC I/O port direction 
//* register (TRISA..TRISE).
//****************************************************************************** 

void DisplayDirPort(byte PortID)
{
	byte PortValue;                    	// Contents of selected port 
   
	// Read the selected port 

	PortValue =   *PortDirAddress[PortID];

	// Display the port contents in hex

   printf(PortDirName[PortID]);
  	printf("  = ");
   hex2(PortValue);
   printf("\r\n");

}

//******************************************************************************
//* DisplayRegister - Display the contents of the selected CPU register (SFR or RAM).
//****************************************************************************** 

void DisplayRegister(byte RegisterID)
{
  	byte RegisterValue;                    	// Contents of selected SFR register or RAM 
 	byte *LocalRegisterAddress;
        int Address;

	// Read the selected port 

	//Address =  (int)RegisterAddress[RegisterID] | (int)0x0F00;
	//LocalRegisterAddress = (byte *)Address;
 
	LocalRegisterAddress = (byte *)((int)RegisterAddress[RegisterID] | (int)0x0F00);

	RegisterValue = *LocalRegisterAddress;

	// Display the port contents in hex

   printf(RegisterName[RegisterID]);
  	printf("  = ");
   hex2(RegisterValue);
   printf("\r\n");


} 

  
//******************************************************************************
//* DisplayRegisters - Display the contents of the all CPU registers (SFRs).
//****************************************************************************** 

void DisplayRegisters(void)
{
	byte i;                                 			// Loop index

	for (i=TOSU_ID; i<= STATUS_ID; i++)
	{
		GotoXY(10,2+i);
		DisplayRegister(i);
	}
   
}

//******************************************************************************
//* DisplayStatus - Display the contents of the all the Status register bit flags.
//****************************************************************************** 

void DisplayStatus(void)
{
	byte i;                                 			// Loop index
	byte Bit;													// Current Status register bit

	for (i=0; i<8 ; i++)
	{
		GotoXY(10,2+i*10);

		// Get selected bit

	
      if (STATUS & (1<<i))
		{
			putchar('1');
		}
 		else
		{
			putchar('0');
		}
 	}
   
}
#endif 
