//*********************************************************************************
//*                                                                               *
//*         Name:  portutil.h                                                     *
//*                                                                               *
//*       Author:  Daniel F. Ramirez                                              *
//*                                                                               *
//*         Date:  10/09/2000                                                     *
//*                                                                               *
//*  Description:                                                                 *
//*                                                                               *
//*   	This is a PIC 18C452 "C" test program which will test the following      * 
//* "C" versions of the BS2 Stamp functions: high, low, toggle, input, output,    *
//* pulsin, pulsout, shiftin, shiftout.  For test purposes, switches are          *
//* connected to Port D in order select each pin's state.                         *                                                        
//*                                                                               * 
//*     Future upgrades to this application include: Implementing the Stamp       *
//* BS2 functions: pwm, dtmfout and freqout and diagnostics functions.            *
//*                                                                               *
//* Note:                                                                         *
//* 	1) The names of all BS2 equivalent functions use lower case.                *
//*                                                                               *
//*     2) The following BS2 to PIC18C452 I/O pin map is used by most of the      *
//*        Port Utility functions:                                                *                                                                                                             
//*                                                                               *
//*       BS2        PIC        PIC           PIC                                 *
//*       PIN        PIN        PORT          BIT                                 *
//*     ---------------------------------------------                             *
//*         0         2           0             0                                 *
//*         1         3           0             1                                 *
//*         2         4           0             2                                 *
//*         3         5        A  0             3                                 *
//*         4         6           0             4                                 *
//*         5         7           0             5                                 *
//*         6        14           0             6                                 *
//*     ---------------------------------------------                             *
//*         7        33           1             0                                 *
//*         8        34           1             1                                 *
//*         9        35           1             2                                 *
//*        10        36           1             3                                 *
//*        11        37        B  1             4                                 *
//*        12        38           1             5                                 *
//*        13        39           1             6                                 *
//*        14        40           1             7                                 *
//*     ---------------------------------------------                             *
//*        15        15           2             0                                 *
//*        16        16           2             1                                 *
//*        17        17           2             2                                 *
//*        18        18           2             3                                 *
//*        19        23        C  2             4                                 *
//*        20        24           2             5                                 *
//*        21        25           2             6                                 *
//*        22        26           2             7                                 *
//*     ---------------------------------------------                             *
//*        23        19           3             0                                 *
//*        24        20           3             1                                 *
//*        25        21           3             2                                 *
//*        26        22           3             3                                 *
//*        27        27        D  3             4                                 *
//*        28        28           3             5                                 *
//*        29        29           3             6                                 *
//*        30        30           3             7                                 *
//*     ---------------------------------------------                             *
//*        31         8           4             0                                 *
//*        32         9        E  4             1                                 *
//*        33        10           4             2                                 *
//*     ---------------------------------------------                             *
//*                                                                               *
//*                                                                               *
//*                                                                               *
//*********************************************************************************
  
#ifndef PORTUTIL_LOADED  /* prevent multiple loading */
#define PORTUTIL_LOADED
     
#include "flags.h"                                        // Global flags used for MPSIM debugging 
  

#ifdef PIC18C252     
#include <p18c252.h>					  // Register definitions for PIC18C252
#endif

#ifdef PIC18C452     
#include <p18c452.h>					  // Register definitions for PIC18C452
#endif

#include <stdlib.h>
#include <string.h>

#ifdef PIC18C452  
#include <timers.h>                                       // Timer library functions
#include <capture.h>                                      // Capture library functions
#include <portb.h>                                        // PORTB library functions                                 
#include <usart.h>                                        // USART library functions
#include <delays.h>                                       // High resolution delay functions
#endif

#include "timeutil.h"                                     // Timer utility functions
#include "stdio.h"                                        // I/O support functions
#include "flags.h"                                        // Debug flags used for MPSIM
  
     
//********************************************************************************
// Constant Definitions
//********************************************************************************
  
  
#define TRUE  1                                             // Boolean TRUE value
#define FALSE 0                                             // Boolean FALSE value
    
//*******************************************************************************
// Define port bit definitions here
//*******************************************************************************

#define INPUT    1                                          // Input pin direction
#define OUTPUT   0                                          // Output pin direction

#define HIGH    1                                           // High value   
#define LOW   0                                             // Low value   

#define MAX_PINS 34                                          // Maximum number of I/O pins allowed


//*******************************************************************************
// Modes used for shiftin function (similar to BS2 Basic Stamp shiftin instruction)
//*******************************************************************************

#define MSBPRE   	0                                 			// msb-first/pre-clock
#define LSBPRE   	1                                			// lsb-first/pre-clock
#define MSBPOST   2                               				// msb-first/post-clock
#define LSBPOST   3                               				// lsb-first/post-clock

//
// Note:
//     Use pre-clock for the shiftin mode when bits on the device are latched on the falling
// edge (1-0 level transition).  Use post-clock for the shiftin mode when the device latches the
// data onto the serial data output on the rising edge (0-1 level transition)
// of the clock line.
//
//     For more detailed information on the shiftin and shiftout
// instructions See Parallax StampII Note 2:"Using Shiftin & Shiftout", pp. 75

//*******************************************************************************
// Modes used for shiftout function (similar to BS2 Basic Stamp shiftout instruction)
//*******************************************************************************

#define LSBFIRST   	0                              			// lsb-first
#define MSBFIRST     1                         					// msb-first


//*******************************************************************************
// Define Keypad typedef structures
//*******************************************************************************
  
typedef unsigned char bit;
typedef unsigned char nib;
typedef unsigned char byte;
typedef unsigned int word;

// Define bit positions as powers of 2

#define BIT_0 1 
#define BIT_1 2 
#define BIT_2 4 
#define BIT_3 8 
#define BIT_4 16 
#define BIT_5 32 
#define BIT_6 64
#define BIT_7 128
  
 
/* Declare global keypad types here */

#define  MAXPORTS  5                       // Maximum number of port registers

#ifdef PIC18C452 

// Port enumeration types

rom enum Port_T {PORT_A, PORT_B, PORT_C, PORT_D, PORT_E}; 

rom enum Register_T  {TOSU_ID,TOSH_ID,TOSL_ID,STKPTR_ID,PCLATU_ID,PCLATH_ID,PCL_ID              
                        ,TBLPTRU_ID,TBLPTRH_ID,TBLPTRL_ID,TABLAT_ID,PRODH_ID
                        ,PRODL_ID,INTCON_ID,INTCON2_ID,INTCON3_ID,INDF0_ID  
                        ,POSTINC0_ID,POSTDEC0_ID,PREINC0_ID,PLUSW0_ID,FSR0H_ID 
                        ,FSR0L_ID,WREG_ID,INDF1_ID,POSTINC1_ID,POSTDEC1_ID
                        ,PREINC1_ID,PLUSW1_ID,FSR1H_ID,FSR1L_ID,BSR_ID,INDF2_ID 
                        ,POSTINC2_ID,POSTDEC2_ID,PREINC2_ID,PLUSW2_ID,FSR2H_ID
                       ,FSR2L_ID,STATUS_ID};
               


// Port A types  
/*
typedef volatile near union  {
    byte PORTA;
    struct {
        unsigned RA0:1;
        unsigned RA1:1;
        unsigned RA2:1;
        unsigned RA3:1;
        unsigned RA4:1;
        unsigned RA5:1;
        unsigned RA6:1;
        unsigned RA7:1;
    } ;
 
} PORTAbits_T ;   

  // Pin lookup table type

  typedef struct 
  {
        byte PortID:4;               			// Port ID      
        byte PinID:4;               			// Pin ID
 
  }   Pin_T; 						// Pin lookup table type
*/

//********************************************************************************
// Port Utility general variables
//********************************************************************************

// Pin lookup table type

// extern Pin_T  PinLookup[] ; 				// Pin lookup table
  
// Declare strings and numeric tables in ROM to save precious RAM space

//extern rom const char PortName[][6];
  
// Port name and addresse tables

rom extern char PortName[][7];       	// Port names table
 
rom extern int *PortAddress[];  		// Port address table                           

// Port direction name and address tables

rom extern char PortDirName[][7];		// Port direction names table

rom extern int *PortDirAddress[];    	// Port  direction address table                           

// Register name table

extern rom char RegisterName[][10];
 
// Register addresses

extern rom byte *RegisterAddress[]; 

// STATUS Bits

extern rom char StatusBitsName[][4];
extern rom byte StatusBitsID[];

#endif

extern struct capstatus CapStatus;

// Declare function headers here

// PIC Bit functions

void 		PinLookup (byte Pin, byte *PortID, byte *PinID);            // Gets port ID and pin ID from BS2 pin number
byte 		GetBit(byte *RegisterValue, byte BitPosition);					// Gets the bit for a given port   
void 		SetBit(byte *RegisterValue, byte BitPosition);					// Sets the bit for a given port   
void 		ClearBit(byte *RegisterValue, byte BitPosition);				// Clears the bit  for a given port 
void 		ComplementBit(byte *RegisterValue, byte BitPosition);  		// Complements the bit for a given port

// PIC Bit I/O functions

void 		InitializePorts(void);                          				// Initialize port variables 

void		High(byte PortID, byte Pin);              						// PIC version of high function
void		Low(byte PortID, byte Pin);											// PIC version of low function
void		Input(byte PortID, byte Pin);		        					   	// PIC version of input function
void		Output(byte PortID, byte Pin);										// PIC version of output function
void		Reverse(byte PortID, byte Pin);              				   // PIC version of reverse function
void		Toggle(byte PortID, byte Pin);              					   // PIC version of toggle function

// PIC Advanced functions
void 		Pulsin(byte PortID, byte Pin, byte State, word *Result);       				// PIC version of pulsin function                           
void 		Pulsout(byte PortID, byte Pin, word Time);       				               // PIC version of pulsout function                           
void 		Count(byte PortID, byte Pin, word Time, word Variable);       				   // PIC version of count function 
void     Shiftin(byte PortID, byte DataPin, byte ClockPin, byte Mode, word *Result, 
                 byte NumberBits);                                                  // PIC version of shiftin function                       
void     Shiftout(byte PortID, byte DataPin, byte ClockPin, byte Mode, word Data,
                 byte NumberBits);   	                                             // PIC version of shiftout function                       
void     Freqout(byte PortID, byte Pin, word Duration, word Freq1, word Freq2);    	// PIC version of freqout function   
void     Pwm(byte PortID, byte Pin, byte DutyCycle, word Duration);				   	// PIC version of pwm function


// Stamp BS2 Bit I/O functions

void		high(byte Pin);                             						// BS2 version of high function
void		low(byte Pin);																// BS2 version of low function
void     toggle(byte Pin);                               				// BS2 version of toggle function
void		input(byte Pin);															// BS2 version of input function
void 		output(byte Pin);															// BS2 version of output function
void     reverse(byte Pin);                                          // BS2 version of reverse function
byte 		getpin(byte Pin);															// BS2 version, gets the pin value     
void     setpin(byte Pin); 														// BS2 version, same as high
void     clearpin(byte Pin);														// BS2 version, same as low

// Stamp BS2 Advanced functions

void 		pulsin(byte Pin, byte State, word *Result);       				// BS2 version of pulsin function                           
void 		pulsout(byte Pin, word Time);       								// BS2 version of pulsout function                           
void 		count(byte Pin, word Time, word Variable);       				// BS2 version of count function 
void     shiftin(byte DataPin, byte ClockPin, byte Mode, word *Result, byte NumberBits);   	// BS2 version of shiftin function                       
void     shiftout(byte DataPin, byte ClockPin, byte Mode, word Data, byte NumberBits);   		// BS2 version of shiftout function                       
void     freqout(byte Pin, word Duration, word Freq1, word Freq2);     		// BS2 version of freqout function   
void     pwm(byte Pin, byte DutyCycle, word Duration);					// BS2 version of pwm function

// PIC Diagnostic Display functions

void 		DisplayPorts(void);														// Display the contents of all PIC I/O ports
void 		DisplayPort(byte PortID);                       				// Display the contents of a selected  PIC I/O port
void 		DisplayRegister(byte RegisterID);               				// Display the contents of a selected PIC register (SFR) (RAM)
void 		DisplayRegisters(void);               								// Display the contents of a all the PIC registers (SFR)  
void 		DisplayStatus(void);														// Display Status register flags

#endif


