//********************************************************************************
//*        Name:  timeutil.c                                                     *
//*      Author:  Daniel F. Ramirez                                              *
//*        Date:  09/06/2000                                                     *
//*                                                                              *
//*                                                                              * 
//* Description:                                                                 *
//*     This application tests the PIC18C452 Timer Code by running a series of   *
//* PIC timer tests.  The first test implements a general purpose stop watch     *
//* using TIMER0, which can be used to time events or profile code. It allows    *
//* allows the operator to push a button connected to the PortB RB3 pin,         *
//* and then reports the elapsed time.  A timer interrupt test uses Timers 0..3  *
//* to generate interrupts and blink an LED corresponding to each timer at       *
//* specific frequencies. Finally a simple clock function allows the operator    *
//* to set the current time,  and proceeds to display the time in HH:MM:SS       *
//* format.                                                                      *
//*                                                                              *
//*     The system clock frequency is assumed to be set to 20 Mhz, which makes   *
//* the Timer clock frequency 5 MHz. Timer 0 is configured to count down the     *
//* total number of milli-seconds, using a prescalar value of 1:256. Timer 1  is *
//* configured to count down the total number of micro-seconds, using a          *
//* prescalar value of 1:8. These default values may be changed by selecting     *
//* the   OSC_4_MHZ for 4 MHz system clock and OSC_40_MHZ for a  40 MHz system   * 
//* clock.                                                                       *
//*                                                                              *
//* 	The following Stamp and general timing functions are tested by this      *
//* application: rctime, pause, sleep, nap, Delay_ms, Delay_us, Delay_s.         *
//*                                                                              *
//*      Future upgrades to this application include: Implementing the Stamp     *
//* BS2 RCTIME function using TIMER0;  using the PIC high resolution timers for  *
//* remote control by transmitting Sony IR Remote codes to appliances;           *
//* implementing a task scheduler for simple state machine processes using       *
//* the timers;  building an X-10 Timer module.                                  *
//*                                                                              *
//*                                                                              *
//********************************************************************************
  
  
#include "flags.h"                                      // Global flags used for MPSIM debugging 
  

#ifdef PIC18C252     
#include <p18c252.h>					// Register definitions for PIC18C252
#endif

#ifdef PIC18C452     
#include <p18c452.h>					// Register definitions for PIC18C452
#endif

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <i2c.h>					// I2C library functions
#include <pwm.h>                                        // PWM library functions
#include <adc.h>                                        // ADC library functions
#include <timers.h>                                     // Timer library functions
#include <portb.h>                                      // PORTB library functions                                 
#include <usart.h>                                      // USART library functions
#include <delays.h>                                     // High resolution delay functions
#include "timeutil.h"                                   // Timer library functions 
#include "stdio.h"                                      // USART library functions                                     
#include "portutil.h"                                   // Port library functions

//********************************************************************************
// Constant Definitions
//********************************************************************************
  
//#define ADVANCED_TIME_FUNCTIONS							// Include support for advanced time utility functions
                                                        // including RCTIME and a Stopwatch function. 


//#define DELAY_TEST                                    // Select the Delay function test  
//#define PAUSE_TEST                                    // Select the Pause function test
//#define STOPWATCH_TEST                                // Select the stop watch function test (works)
//#define CLOCK_TEST                                    // Select realtime clock 
//#define TIMER_INTERRUPTS                              // Select the timer interrupt test (works)
//#define PORTBINT_TEST                                 // Select the port B interrupt function test
//#define RCTIME_TEST   				                    // Select the RCTIME function test                               
//#define RCTIME_FLAG                                   // If defined species RC circuits otherwise CR circuits
                                                        // are assumed to be in operation

// Define timer constants for various delays

#ifdef  OSC_4_MHZ                                       // 4 MHz system clock
#define TIMER_COUNT_1MS   		0x18FC						  // 1 millisecond count
#define TIMER_COUNT_1S 	   	0xF0BE						  // 1 second count
#endif

#ifdef  OSC_20_MHZ                                      // 20 MHz system clock   
        
#define TIMER_COUNT_1MS   		0xFD8F						  // 1 millisecond count
#define TIMER_COUNT_1S   		0xB3B2						  // 1 second count
 
#endif

#ifdef OSC_40_MHZ                                       // 40 MHz system clock           
#define TIMER_COUNT_1MS   		0xD8F0						  // 1 millisecond count
#define TIMER_COUNT_1S        0x6A67						  // 1 second count
#endif

#ifdef OSC_50_MHZ                                       // 50 MHz system clock           
#define TIMER_COUNT_1MS   		0xCF2C						  // 1 millisecond count
#define TIMER_COUNT_1S        0x4144						  // 1 second count
#endif

// Port A Bits

 
// Port E Bits
  
   
 
//********************************************************************************
// Variable declarations
//********************************************************************************


// Timer general variables


byte Command;



#ifdef TIMER_INTERRUPTS

unsigned int             TimerCount0 = 0;                  // Timer interrupt 0 count (Seconds)
unsigned int             TimerCount1 = 0;                  // Timer interrupt 1 count
unsigned int             TimerCount2 = 0;                  // Timer interrupt 2 count
unsigned int             TimerCount3 = 0;                  // Timer interrupt 3 count
unsigned int             InterruptCount0 = 0;              // Port B interrupt 0 count
unsigned int             InterruptCount1 = 0;              // Port B interrupt 1 count
unsigned int             InterruptCount2 = 0;              // Port B interrupt 2 count

unsigned int             Hours = 0;                        // Hours counter
unsigned int             Minutes = 0;                      // Minutes counter
unsigned int             Seconds = 0;                      // Seconds counter

#endif

int             result;


extern byte Command = 0;					// Timer command read from host (S=set time, G = get time)
extern char Ch; 						// Character read from USART
extern byte HexNumber[];					// Hex string

//********************************************************************************
// Declare function headers here
//********************************************************************************

void main(void);
 

#ifdef TIMER_INTERRUPTS

//********************************************************************************
// Interrupt Code
//********************************************************************************


// Designate ServiceInterrupt as an interrupt function and save key registers

#pragma interrupt ServiceInterrupt save = PRODL,PRODH,FSR0L,FSR0H

// Locate ISR handler code at interrupt vector

#pragma code isrcode=0x0008

void            isrhandler(void)                           // This function directs
 // execution to the
{                                                          // actual interrupt code
    _asm
    goto ServiceInterrupt
    _endasm
}

#pragma code

//********************************************************************************
//* ServiceInterrupt - Process each type of interrupt (TMR0IF..TMR3IF).
//********************************************************************************
void            ServiceInterrupt(void)
{
    SPULSE = 1;                                            // Toggle output pin for
    // ISR code timing

#ifdef PORTBINT_TEST
 
   // Check for the type of Port B interrupt (INT0..INT2) and process it

    if (INTCONbits.INT0IF == 1)
    {
        ProcessInterrupt0();
    } else if (INTCON3bits.INT1IF == 1)
    {
        ProcessInterrupt1();
    } else if (INTCON3bits.INT2IF == 1)
    {
        ProcessInterrupt2();
    }
#endif

    // Check for the type of timer interrupt (TMR0IF..TMR3IF) and process it

    if (INTCONbits.TMR0IF == 1)
    {
        ProcessTimer0();
    } 
    else 
    if (PIR1bits.TMR1IF == 1)
    {
        ProcessTimer1();
    } 
    else 
    if (PIR1bits.TMR2IF == 1)
    {
        ProcessTimer2();
    } 
    else if (PIR2bits.TMR3IF == 1)
    {
        ProcessTimer3();
    }
    SPULSE = 0;                                            // Toggle output pin for
    // ISR code timing
}

#endif

//********************************************************************************
//* SetupTimers - Initializes program variables and peripheral registers
//* the timer interrupt test code.
//********************************************************************************
void            SetupTimers(void)
{
 

#ifdef TIMER_INTERRUPTS


    Hours = 0;                                             // Reset hours counter
    Minutes = 0;                                           // Reset minutes counter
    Seconds = 0;                                           // Reset seconds counter

    TimerCount0 = 0;                                       // Reset Timer Interrupt 0 count (Elapsed seconds)
    TimerCount1 = 0;                                       // Reset Timer Interrupt 1 count (Elapsed milli-seconds)
    TimerCount2 = 0;                                       // Reset Timer Interrupt 2 count
    TimerCount3 = 0;                                       // Reset Timer Interrupt 3 count

    InterruptCount0 = 0;                                   // Reset Interrupt 0 count
    InterruptCount1 = 0;                                   // Reset Interrupt 1 count
    InterruptCount2 = 0;                                   // Reset Interrupt 2 count
#endif

    // configure port A for all digital inputs

    //OpenADC(ADC_FOSC_32 &
    //        ADC_RIGHT_JUST & ADC_0ANA_0REF, ADC_CH0 & ADC_CH1 & ADC_CH2 & ADC_CH3 & ADC_CH4 & ADC_CH5 & ADC_CH6 & ADC_CH7 & ADC_INT_OFF);
    OpenADC(ADC_0ANA_0REF, ADC_INT_OFF);

   // Configure PORTB interrupt with Interrupt on change off and pull-up resistors disabled

    OpenPORTB(PORTB_CHANGE_INT_OFF & PORTB_PULLUPS_OFF);

 

    // Initialize ports

    //PORTA = 0x00;                                          // Clear PORTC
    //PORTB = 0x00;                                          // Clear PORTC
    //PORTC = 0x00;                                          // Clear PORTC.  Port C used by USART.
    //PORTD = 0x00;                                          // Clear PORTD
    //PORTE = 0x00;                                          // Clear PORTE

    // Initialize port I/O direction
/*
#ifdef PORTBINT_TEST
    TRISA = 0xFF;                                          // PORT A all inputs.
    TRISB = 0xFF;                                          // PORT B all inputs.
#else
    TRISA = 0x00;                                          // PORT A all outputs.
    TRISB = 0x00;                                          // PORT B all outputs.
#endif
*/

    //TRISC = 0x00;                                          // PORTC all outputs. Port C used by USART.
    //TRISD = 0xFF;                                          // PORTD all inputs
    //TRISE = 0x00;                                          // PORTE all outputs.

    // Clear timer registers

    TMR0L = 0;
    TMR0H = 0;
    TMR1L = 0;
    TMR1H = 0;
    TMR2 = 0;
    TMR3L = 0;
    TMR3H = 0;


    // Turn LEDs OFF

    LED0 = 0;
    LED1 = 0;
    LED2 = 0;
    LED3 = 0;

#ifdef USE_TIMER1

    // Use timer 1 for BS2 pause function
    
    // configure timer0 with interrupt off, internal source and a pre-scaler of 1:256 @ 20 MHz

    OpenTimer0(TIMER_INT_OFF & T0_SOURCE_INT & T0_16BIT & T0_PS_1_256);

    // configure timer1, internal source, 16-Bits and a pre-scaler of 1:8  @ 20 MHz, for BS2 pause function

    OpenTimer1(TIMER_INT_OFF & T1_16BIT_RW & T1_SOURCE_INT & T1_PS_1_8 & T1_OSC1EN_OFF & T1_SYNC_EXT_OFF);
     
    // Configure timers and timer interrupts for Timer 2 and a pre-scaler of 1:16 @ 20 MHz

    //OpenTimer2(TIMER_INT_OFF & T2_PS_1_16);

    // Configure timers and timer interrupts for Timer 3 and a pre-scaler of 1:8 @ 20 MHz

    OpenTimer3(TIMER_INT_OFF & T3_SOURCE_INT & T3_16BIT_RW
               & T3_PS_1_8 & T3_SYNC_EXT_OFF & T3_OSC1EN_OFF);
 
#else

    // Use timer 3 for BS2 pause function

    // configure timer0 with interrupt off, internal source and a pre-scaler of 1:256 @ 20 MHz

    OpenTimer0(TIMER_INT_OFF & T0_SOURCE_INT & T0_16BIT & T0_PS_1_256);

    // configure timer1, internal source, 16-Bits and a pre-scaler of 1:8 @ 20 MHz 

    OpenTimer1(TIMER_INT_OFF & T1_16BIT_RW & T1_SOURCE_INT & T1_PS_1_8 & T1_OSC1EN_OFF & T1_SYNC_EXT_OFF);
     
    // Configure timers and timer interrupts for Timer 2 and a pre-scaler of 1:16 @ 20 MHz

    OpenTimer2(TIMER_INT_OFF & T2_PS_1_16);

    // Configure timers and timer interrupts for Timer 3 and a pre-scaler of 1:8 @ 20 MHz, for BS2 pause function 

    OpenTimer3(TIMER_INT_OFF & T3_SOURCE_INT & T3_16BIT_RW
               & T3_PS_1_8 & T3_SYNC_EXT_OFF & T3_OSC1EN_OFF);
#endif

 
   // EnablePullups();				            // Enable PORTB pullups (inputs)
    DisablePullups();                                       // Disable PORTB pullups

}

//********************************************************************************
//* SetupInterrupts - Initializes the timers and interrupts for the timer
//* interrupt test.
//********************************************************************************
void            SetupInterrupts(void)
{

    // Configure timers and timer interrupts for Timer 0, with a 1:128 pre-scale
    // for an interrupt frequency of 1 Hz using a 20 MHz system clock.

    OpenTimer0(TIMER_INT_ON & T0_16BIT & T0_EDGE_RISE &
               T0_SOURCE_INT & T0_PS_1_128);

    //OpenTimer0(TIMER_INT_ON & T0_16BIT & T0_EDGE_RISE &
    //           T0_SOURCE_INT & T0_PS_1_256);

    // Configure timers and timer interrupts for Timer 1

    OpenTimer1(TIMER_INT_ON & T1_SOURCE_INT & T1_16BIT_RW
               & T1_PS_1_8 & T1_SYNC_EXT_OFF & T1_OSC1EN_OFF);

    // Configure timers and timer interrupts for Timer 2

    OpenTimer2(TIMER_INT_ON & T2_PS_1_16);

    // Configure timers and timer interrupts for Timer 3

    OpenTimer3(TIMER_INT_ON & T3_SOURCE_INT & T3_16BIT_RW
               & T3_PS_1_8 & T3_SYNC_EXT_OFF & T3_OSC1EN_OFF);

    // Configure interrupts

     // Configure timer interrupt priority (1 = High Priority, 0 = Low
    // Priority).

    INTCON2bits.TMR0IP = 1;                                // Set timer interrupt 0
    // priority to high
    IPR1bits.TMR1IP = 1;                                   // Set timer interrupt 1
    // priority to high
    IPR1bits.TMR2IP = 1;                                   // Set timer interrupt 2
    // priority to high
    IPR2bits.TMR3IP = 1;                                   // Set timer interrupt 3
    // priority to high

    // configure port A for all digital inputs

    //OpenADC(ADC_FOSC_32 &
    //        ADC_RIGHT_JUST & ADC_0ANA_0REF, ADC_CH0 & ADC_CH1 & ADC_CH2 & ADC_CH3 & ADC_CH4 & ADC_CH5 & ADC_CH6 & ADC_CH7 & ADC_INT_OFF);
    OpenADC(ADC_0ANA_0REF, ADC_INT_OFF);

#ifdef PORTBINT_TEST

    // Configure Port B interrupts

    // Configure PORTB interrupts (INT0..INT2) using the PORTB RB0..RB2 pins
    // with rising edge and pull-up resistors enabled.

    OpenRB0INT(RISING_EDGE_INT);
    OpenRB1INT(RISING_EDGE_INT);
    OpenRB2INT(RISING_EDGE_INT);
 
    EnablePullups();                                       // Enable PORTB pullups
   
   // Configure interrupts (1 = High Priority, 0 = Low Priority).  Interrupt 0 is always
    // set to high priority

    INTCON3bits.INT1IP = 1;                                // Set interrupt 1 priority to high
    INTCON3bits.INT2IP = 1;                                // Set interrupt 2 priority to high

#endif

 
    // Load timer 0 and timer 1 registers here

    WriteTimer0(TIMER_COUNT_1S);                           // Value for a 1 second period (1 Hz) 
                                                           // (Use LSB..MSB order to compensate for Microchip 
                                                           // timer library bug)  
 
    WriteTimer1(TIMER_COUNT_1MS);                       // Value for a 1 milli-second delay
                                                           // (Use LSB..MSB order to compensate for Microchip 
                                                           // timer library bug)  
                                                           // (Use "WriteTimer1" instead of "WriteTimer1" to compensate for another Microchip 
 
    // Enable peripheral interrupts

    INTCONbits.PEIE = 1;                                   // Enable peripheral

    // Enable general interrupts

    INTCONbits.GIE = 1;                                    // Enable all interrupts
   
}
 
#ifdef TIMEUTIL_TEST

//********************************************************************************
//* main - Displays the latest interrupt counts from interrupts Timer0..Timer3.
//********************************************************************************

void            main(void)
{

   // Setup timers, configure ports and initialize variables

   SetupTimers();  

   // Configure USART for 20 MHz, 9600 Baud, 8-Bits, No parity, Asynchronous
   
    InitializeUsart();
                                      
 
    // Display message to operator

    printf("PIC18C452 Timer Test Application \r\n");


#ifdef RCTIME_TEST

   printf("PIC18C452 RcTime Test Application \r\n");

#endif

#ifdef PORTBINT_TEST

    printf("PIC18C452 Port B Interrupt Test Application \r\n");

 

    SetupInterrupts();                                     // Setup interrupts and software
    // variables.



    while (TRUE)                                           // Loop forever
    {
        // ClrWdt();				// Clear the WDT

        // Display the latest interrupt counts

        printf("InterruptCount0 = ");
        dec(InterruptCount0);                        // Convert interrupt count 0 to a string
        printf("\r\n");

  
        printf("InterruptCount1 = ");
        dec(InterruptCount1);                        // Convert interrupt count 1 to a string
        printf("\r\n");

        printf("InterruptCount2 = ");
        dec(InterruptCount2);                        // Convert interrupt count 2 to a string
        printf("\r\n");


    }                                                      // end while(1)

#endif



#ifdef PAUSE_TEST

    // Test the Pause function

    printf("PIC18C452 Start Pause Test ... \r\n");

    PauseTest();
#endif

#ifdef DELAY_TEST

    // Test the Delay function

    printf("PIC18C452 Start Delay Test ... \r\n");

    DelayTest();
#endif

 
#ifdef STOPWATCH_TEST
 
   //SetupInterrupts();                                     // Setup interrupts and software
    // variables.


    // Test the Stopwatch function

    printf("PIC18C452 Start Stopwatch Test ... \r\n");

    Stopwatch();
#endif


#ifdef CLOCK_TEST

   // Clock Test  

    printf("PIC18C452 Start Clock Test ... \r\n");

    //TimerInterruptTest();

    // Configure the timers and interrupts

    SetupInterrupts();

    //ClockTest();

    while (TRUE)
    {
	// Get the time and send it to the host

        //GetTime();

        printf("Hours = ");                              // Display number of hours elapsed
        dec(Hours );                                    // convert to string
    
        printf("  Minutes = ");                          // Display Number of minutes elapsed
        dec(Minutes );                                  // convert to string
 
        printf("  Seconds = ");                          // Display number of seconds elapsed 
        dec(Seconds);                                  // convert to string
 
	printf("\r\n");                                     // Send carriage return
 
   }

#endif

#ifdef TIMER_INTERRUPTS

    // Test the Timer interrupts

    printf("PIC18C452 Start Timer Interrupt Test ... \r\n");

    //TimerInterruptTest();
    // Configure the timers and interrupts

    SetupInterrupts();

    while (TRUE)
   {
  

        printf("TimerCount0 = ");                         // Display timer 0 counts
        dec(TimerCount0 );                              // convert to string
   
        printf("  TimerCount1 = ");                         // Display timer 0 counts
        dec(TimerCount1);                              // convert to string
  
        printf("  TimerCount2 = ");                         // Display timer 0 counts
        dec(TimerCount2 );                              // convert to string
  
        printf("  TimerCount3 = ");                         // Display timer 0 counts
        dec(TimerCount3 );                              // convert to string
	printf("\r\n");                                          // Send carriage return

 
  }

#endif

    CloseTimer0();                                         // close modules
    CloseUSART();
    return;
}


//********************************************************************************
//* ProcessTimer0 - Processes timer 0 interrupt, serviced every second at 
//* a 1 Hz frequency.
//********************************************************************************

void            ProcessTimer0(void)
{  
   // Disable perpheral interrupts

    INTCONbits.PEIE = 0;                                   // Disable peripheral

    // Disable general interrupts

    INTCONbits.GIE = 0;                                    // Disable all interrupts
 
    TimerCount0++;                                         // Increment timer counter 0

#ifdef CLOCK_TEST

    // Keep track of elapsed hours, minutes and seconds 
 
    Seconds = (Seconds++) % 60;				   // Increment seconds counter

    if (Seconds == 0)                                      
    {
	Minutes = (Minutes++) % 60;                        // Increment minutes counter
 
    	if (Minutes == 0)                                  
        {
           Hours = (Hours++) % 24;   			   // increment hours counter (24 hour clock)                         
        }
    }
#endif    

    // count
    LED0 = ~LED0;                                          // Toggle the timer 0 LED
                                                           // interrupt  LED indicator
   
    INTCONbits.TMR0IF = 0;                                 // Clear the Timer 0 overflow flag

    WriteTimer0(TIMER_COUNT_1S);                           // 2's complement of 39063 (0x6769) for a 1 second period (1 Hz) 
                                                           // using timer 0 and a 1:128 pre-scale.
                                                           // (Use LSB..MSB order to compensate for Microchip 
                                                           // timer library bug)  
   // Enable perpheral interrupts

    INTCONbits.PEIE = 1;                                   // Enable peripheral

    // Enable general interrupts

    INTCONbits.GIE = 1;                                    // Enable all interrupts
 
 
}
  
//********************************************************************************
//* ProcessTimer1 - Processes timer 1 interrupt, serviced every milli-second for  
//* at a 1 KHz frequency.
//********************************************************************************

void            ProcessTimer1(void)
{
  // Disable perpheral interrupts

    INTCONbits.PEIE = 0;                                   // Disable peripheral

    // Disable general interrupts

    INTCONbits.GIE = 0;                                    // Disable all interrupts
 
    TimerCount1++;                                         // Increment timer counter 1 count

    LED1 = ~LED1;                                          // Toggle the timer 1 LED
                                                           // interrupt LED indicator
 
    PIR1bits.TMR1IF = 0;                                   // Clear timer 1 Interrupt Flag.

    WriteTimer1(TIMER_COUNT_1MS);                          // 2's complement of 625 (0xFD8F) for a 1 milli-second delay using timer 1
                                                           // and a 1:1 pre-scale.
                                                           // (Use LSB..MSB order to compensate for Microchip 
                                                           // timer library bug)  
                                                           // (Use "WriteTimer1" instead of "WriteTimer1" to compensate for another Microchip 
                                                           // timer library bug)  
   // Enable perpheral interrupts

    INTCONbits.PEIE = 1;                                   // Enable peripheral

    // Enable general interrupts

    INTCONbits.GIE = 1;                                    // Enable all interrupts
 
    
 
}

//********************************************************************************
//* ProcessTimer2 - Processes timer 2 interrupt.
//********************************************************************************

void            ProcessTimer2(void)
{
  // Disable perpheral interrupts

    INTCONbits.PEIE = 0;                                   // Disable peripheral

    // Disable general interrupts

    INTCONbits.GIE = 0;                                    // Disable all interrupts
 
    TimerCount2++;                                         // Increment timer 2 count

    LED2 = ~LED2;                                          // Toggle the timer 2 LED
                                                           // interrupt LED indicator

    PIR1bits.TMR2IF = 0;                                   // Clear timer Interrupt 2 Flag.

   // Enable perpheral interrupts

    INTCONbits.PEIE = 1;                                   // Enable peripheral

    // Enable general interrupts

    INTCONbits.GIE = 1;                                    // Enable all interrupts
 

}

//********************************************************************************
//* ProcessTimer3 - Processes timer 3 interrupt.
//********************************************************************************

void            ProcessTimer3(void)
{
  // Disable perpheral interrupts

    INTCONbits.PEIE = 0;                                   // Disable peripheral

    // Disable general interrupts

    INTCONbits.GIE = 0;                                    // Disable all interrupts
 
    TimerCount3++;                                         // Increment timer 3 count

    LED3 = ~LED3;                                          // Toggle the timer 3
                                                           // interrupt LED indicator
 
   PIR2bits.TMR3IF = 0;                                   // Clear timer Interrupt 3 Flag.

   // Enable perpheral interrupts

    INTCONbits.PEIE = 1;                                   // Enable peripheral

    // Enable general interrupts

    INTCONbits.GIE = 1;                                    // Enable all interrupts
 

}
//********************************************************************************
//* ProcessInterrupt0 - Processes Port B interrupt 0 (INT0).
//********************************************************************************

void            ProcessInterrupt0(void)
{
    InterruptCount0++;                                     // Increment interrupt 0 count
    LED0 = 1;                                              // Toggle the interrupt 0 LED indicator
    INTCONbits.INT0IF = 0;                                 // Clear Interrupt 0 Flag.

}

//********************************************************************************
//* ProcessInterrupt1 - Processes  Port B interrupt 1 (INT1).
//********************************************************************************

void            ProcessInterrupt1(void)
{
    InterruptCount1++;                                     // Increment interrupt 1 count
    LED1 = 1;                                              // Toggle the interrupt 1 LED indicator
    INTCON3bits.INT1IF = 0;                                // Clear Interrupt 1 Flag.

}

//********************************************************************************
//* ProcessInterrupt2 - Processes Port B interrupt 2 (INT2).
//********************************************************************************

void            ProcessInterrupt2(void)
{
    InterruptCount2++;                                     // Increment interrupt 2 count
    LED2 = 1;                                              // Toggle the interrupt 2 LED indicator
    INTCON3bits.INT2IF = 0;                                // Clear Interrupt 2 Flag.

}

 
#endif



// Stamp BS2/BSX Compatible function implementation

// Stub out delay routines

#ifdef DEBUG

void pause(word Delay)
{
}

#else

//********************************************************************************
//* pause - Will pause for n units of 1000 instruction cycles (milli-seconds at 20 MHz)
//* Works the same was as the Stamp "pause" function.  This function uses Timer 1 or Timer 3, 
//* with with the prescaler set to 1:8 and a system clock of 20 Mhz.
//********************************************************************************
void            pause(unsigned int n)
{


#ifdef USE_TIMER1

    // Use Timer 1
    
    // Clear the Timer 1 overflow flag 

    PIR1bits.TMR1IF = 0;

    do
    {

        WriteTimer1(TIMER_COUNT_1MS);                   // 2's complement of 625 (0xFD8F) for a 1 milli-second delay
                                                           // (Use LSB..MSB order to compensate for Microchip 
                                                           // timer library bug)  
                                                           // (Use "WriteTimer1" instead of "WriteTimer1" to compensate for another Microchip 
                                                           // timer library bug)  
            

       // Wait until Timer 1 counts down to 0 or the Timer 1 overflow flag is set
 
       //while ((Count = ReadTimer1()) != 0xFFFF);

        while (!PIR1bits.TMR1IF);
 
        // Clear the Timer 1 overflow flag 

        PIR1bits.TMR1IF = 0;

    } while (--n);
#else

   // Use Timer 3
    
    // Clear the Timer 3 overflow flag 

    PIR2bits.TMR3IF = 0;

    do
    {

        WriteTimer3(TIMER_COUNT_1MS);                      // 2's complement of 625 (0xFD8F) for a 1 milli-second delay
                                                           // (Use LSB..MSB order to compensate for Microchip 
                                                           // timer library bug)  
                                                           // (Use "WriteTimer3_16" instead of "WriteTimer3" to compensate for another Microchip 
                                                           // timer library bug)  
            

       // Wait until Timer 3 counts down to 0 or the Timer 3 overflow flag is set
 
       //while ((Count = ReadTimer3()) != 0xFFFF);

        while (!PIR2bits.TMR3IF);
 
        // Clear the Timer 3 overflow flag 

        PIR2bits.TMR3IF = 0;

    } while (--n);
    
#endif



}    
#endif

//********************************************************************************
//* Sleep1 - Sleep for n units of 2.3 seconds.  Works the same was as the Stamp
//* "sleep" function.
//********************************************************************************
void            Sleep1(int n)
{
    do
    {
        DelayMs(2300);                                     // Sleep for 2.3 seconds
        // inside loop

        //******* Missing logic to put processor in a sleep state ***********
    } while (n--);
}
  

//********************************************************************************
//* Nap - Nap for a short duration of n delay periods.  Each delay period is 
//* set to 2**Period * 18 milli-seconds.  Works the same was as the Stamp 
//* "nap" function.
//********************************************************************************
void            nap(int n)
{
    int             Duration;

    Duration = n * 18;
    DelayMs(Duration);                                     // Nap for 2**n * 18
    // milli-seconds ????

}

// General timing functions implementation

//******************************************************************************
//* DelayUs - Delay for n units of 100 TCY (micro-seconds at 4 MHz) using Timer 1.
//******************************************************************************
void            DelayUs(unsigned int n)
{


    /* Delay10TCYx Delay multiples of 10 Tcy Passing 0 (zero) results in a delay of 2560 cycles. The 18Cxxx version of
     * this function supports the full range [0,255] The 17Cxxx version supports [2,255] and 0. */

    //Delay10TCYx(n);                    // Use this call for delay of 10 TCY
 
  /* Delay100TCYx Delay multiples of 10 Tcy Passing 0 (zero) results in a delay of 2560 cycles. The 18Cxxx version of
     * this function supports the full range [0,255] The 17Cxxx version supports [2,255] and 0. */

    Delay100TCYx(n);

}

//******************************************************************************
//* DelayMs - Delay for n milli-seconds using Timer 1 or Timer 3, with the prescaler
//* set to 1:8 and a system clock of 20 Mhz.
//******************************************************************************
void            DelayMs(unsigned int n)
{

#ifdef USE_TIMER1

    // Use Timer 1
    
    // Clear the Timer 1 overflow flag 

    PIR1bits.TMR1IF = 0;

    do
    {

        WriteTimer1(TIMER_COUNT_1MS);                   // 2's complement of 625 (0xFD8F) for a 1 milli-second delay
                                                           // (Use LSB..MSB order to compensate for Microchip 
                                                           // timer library bug)  
                                                           // (Use "WriteTimer1" instead of "WriteTimer1" to compensate for another Microchip 
                                                           // timer library bug)  
            

       // Wait until Timer 1 counts down to 0 or the Timer 1 overflow flag is set
 
       //while ((Count = ReadTimer1()) != 0xFFFF);

        while (!PIR1bits.TMR1IF);
 
        // Clear the Timer 1 overflow flag 

        PIR1bits.TMR1IF = 0;

    } while (--n);
#else

   // Use Timer 3
    
    // Clear the Timer 3 overflow flag 

    PIR2bits.TMR3IF = 0;

    do
    {

        WriteTimer3(TIMER_COUNT_1MS);                      // 2's complement of 625 (0xFD8F) for a 1 milli-second delay
                                                           // (Use LSB..MSB order to compensate for Microchip 
                                                           // timer library bug)  
                                                           // (Use "WriteTimer3_16" instead of "WriteTimer3" to compensate for another Microchip 
                                                           // timer library bug)  
            

       // Wait until Timer 3 counts down to 0 or the Timer 3 overflow flag is set
 
       //while ((Count = ReadTimer3()) != 0xFFFF);

        while (!PIR2bits.TMR3IF);
 
        // Clear the Timer 3 overflow flag 

        PIR2bits.TMR3IF = 0;

    } while (--n);
    
#endif



}
  
//******************************************************************************
//* DelayS - Delay for n seconds using Timer 0, with the prescaler set to 1:256
//* and a system clock of 20 Mhz.
//******************************************************************************
void            DelayS(unsigned int n)
{
      static unsigned int Count = 0;             // Timer count

     
    INTCONbits.TMR0IF = 0;                       // Clear the Timer 0 overflow flag
 
    do
    {
 
 
        WriteTimer0(TIMER_COUNT_1S);             // Initialize Timer 0 registers 

        // Wait until Timer 0 counts to 0xFFFF or the timer 0 overflow flag is set
 
        //while (ReadTimer0() != 0xFFFF);
        //while ((Count = ReadTimer0()) != 0xFFFF);

        while (!INTCONbits.TMR0IF);

        // Clear the Timer 0 overflow flag 

        INTCONbits.TMR0IF = 0;
  
    } while (--n);

}

#ifdef ADVANCED_TIMER_FUNCTIONS

//********************************************************************************
//* Stopwatch - This function uses TIMER0 to implement a general purpose stopwatch
//* that can be used to time events or profile code. It allows the operator to push
//* a button connected to the PortB RB3 pin, and then reports the elapsed time.
//********************************************************************************
void            Stopwatch(void)
{
    unsigned int             StartTime = 0;                         // Start time
    unsigned int             StopTime = 0;                          // Stop time
    unsigned int             ElapsedTime = 0;                       // Elapsed time

    // Enable Port B weak pullups

    EnablePullups();

    while (TRUE)
    {
        printf("Push the START button to start stopwatch \n");
        printf("Push the STOP button to stop the watch and display the elapsed time. \n\n");

        // Wait for operator to press the START button

        while (BUTTON0 != 0);
        //while (InterruptCount0 == 1);

        //StartTime = ReadTimer0();                          // Read Timer 0 elapsed seconds
	StartTime = TimerCount0;

        // Wait for operator to press the STOP button

        while (BUTTON1 != 0);
        //while (InterruptCount1 == 1);

        //StopTime = ReadTimer0();                           // Read Timer 0
        StopTime = TimerCount0;                              // Read Timer 0 elapsed seconds

        // Compute elapsed time

        ElapsedTime = StopTime - StartTime;

        Beep(3);
        printf("StartTime = ");
        dec(StartTime);                                // convert to string
        printf("\r\n");  

        printf("StopTime = ");
        dec(StopTime);                                 // convert to string
        printf("\r\n");  

        printf("ElapsedTime = ");
        dec(ElapsedTime );                              // convert to string
        printf("\r\n");  

    }
}
  
//********************************************************************************
//* rctime - Determines the time it takes for a capacitor to discharge from 5 Volts
//* to 0 Volts.  If the time exceeds 65535 time units then the fuction returns a
//* count of 0.  Works the same was as the Stamp "rctime" function.  This function
//* uses the BS2 pin notations which range from 0..33.
//********************************************************************************
void            rctime(byte Pin, byte State, word *Count)
{
/*  
      byte PinValue;
      
#ifdef RCTIME_FLAG

     // RC circuit code
     
    	// Make the selected BS2 pin an output to charge capacitor for CR cuircuit time
 
     //low(Pin);
     
    TRISE = 0x00;                                          // PORTE all outputs
    XJoystickPin = 0;                                     // Discharge the capacitor
     
     
    	pause(5);                                              // Allow time for discharge

    	// Make the selected BS2 pin an input
	
	  //input(Pin);

    TRISE = 0xFF;                                          // PORTE all inputs
    	

        //WriteTimer0(0);                                    // Reset timer 0
        WriteTimer1(0);                                   // Reset timer 1
        
        //while (getpin(Pin) != State)
        //while (TRUE)
        while(XJoystickPin == State);
        
        
        // wait for Pin to equal state

        *Count  = ReadTimer1();                              // read timer 1
        
      	printf("  Count = ");                         // Display timer 0 counts
        hex4(*Count);                              // convert to string
        printf("\r\n");
        
#else

     // CR circuit code (This logic works !!!!)
     
    	// Make the selected BS2 pin an output to charge capacitor for CR cuircuit time
 
     //high(Pin);
     
    TRISE = 0x00;                                          // PORTE all outputs
    XJoystickPin = 1;                                     // Charge the capacitor
     
     
    	pause(1);                                              // Allow time for charge

    	// Make the selected BS2 pin an input
	
	  //input(Pin);

    TRISE = 0xFF;                                          // PORTE all inputs
    	

        //WriteTimer0(0);                                    // Reset timer 0
        WriteTimer1(0);                                   // Reset timer 1
        
        //while (getpin(Pin) != State)
        //while (TRUE)
        while(XJoystickPin == State);
        
        
        // wait for Pin to equal state

        *Count  = ReadTimer1();                              // read timer 1
        
      	printf("  Count = ");                         // Display timer 0 counts
        hex4(*Count);                              // convert to string
        printf("\r\n");

#endif
*/

}

/*

//********************************************************************************
//* ReadJoystick - Reads the connected joystick using the rctime function and returns
//* the raw x and y axis counts.
//********************************************************************************
void ReadJoystick(word *XCount, word *YCount)
{

   byte PinValue;
   byte State;

   State = 0;
     
   // Read Joystick X axis using RC circuit code
     
   TRISE = 0x00;                                          // PORTE all outputs
   XJoystickPin = 0;                                      // Discharge the capacitor
     
   //pause(5);                                              // Allow time for discharge
   pause(10);                                              // Allow time for discharge

   // Make the selected BS2 pin an input
	
   //input(Pin);
   TRISE = 0xFF;                                          // PORTE all inputs
    	
   WriteTimer1(0);                                     // Reset timer 1
       
   while(XJoystickPin == State);
          
   // wait for Pin to equal state

   *XCount  = ReadTimer1();                              // read timer 1
  
   // Read Joystick Y axis using RC circuit code
 	// Make the selected BS2 pin an output to charge capacitor for CR cuircuit time
 
   //low(Pin);
     
   TRISE = 0x00;                                          // PORTE all outputs
   YJoystickPin = 0;                                      // Discharge the capacitor
     
   //pause(5);                                              // Allow time for discharge
   pause(10);                                              // Allow time for discharge

   // Make the selected BS2 pin an input
	
   //input(Pin);
   TRISE = 0xFF;                                          // PORTE all inputs
    	
   WriteTimer1(0);                                     // Reset timer 1
       
   while(YJoystickPin == State);
          
   // wait for Pin to equal state

   *YCount  = ReadTimer1();                              // read timer 1
        
}
*/
  
//********************************************************************************
//* Beep - Emits n beeps.
//********************************************************************************
void            Beep(int n)
{
    char            Bell[2] = {0x07, 0x00};                // Bell character
#ifdef DEBUG
    return;
#endif

    do
    {
        string(Bell);                                   // print string
    } while (--n);

}

//********************************************************************************
//* Sound - Emits a sound of a specified frequency and duration to a speaker
//* connected to any I/O pin. Works the same was as the Stamp "sound" function.
//********************************************************************************
void            Sound(int n)
{
    char            Bell[2] = {0x07, 0x00};                // Bell character
#ifdef DEBUG
    return;
#endif

    do
    {
        string(Bell);                                   // print string
    } while (--n);

}

#ifdef TIMER_INTERRUPTS
//********************************************************************************
//* GetTime - Gets the clock time and sends it to the host using (HH:MM:SS) format.
//********************************************************************************
void            GetTime(void)               // Get the time and send it to the host
{

        printf("Hours = ");                              // Display number of hours elapsed
        dec(Hours);                                    // convert to string
   
        printf("  Minutes = ");                          // Display Number of minutes elapsed
        dec(Minutes);                                  // convert to string
   
        printf("  Seconds = ");                          // Display number of seconds elapsed 
        dec(Seconds );                                  // convert to string
        printf("\r\n");
 
}

//********************************************************************************
//* SetTime - Sets the clock time using (HH:MM:SS) format.
//********************************************************************************
void            SetTime(void)               // Set the time sent from host 
{

	        // Get hours from host 

                HexNumber[0] = toupper(get_byte());        // Get first hex digit (MSB)
                HexNumber[1] = toupper(get_byte());        // Get second hex digit (LSB)
                Ch = get_byte();                           // Get colon delimiter

               // Combine to a two digit hex value into Hours

                Hours = htoi(HexNumber);

	        // Get Minutes from host 

                HexNumber[0] = toupper(get_byte());        // Get first hex digit (MSB)
                HexNumber[1] = toupper(get_byte());        // Get second hex digit (LSB)
                Ch = get_byte();                          // Get colon delimiter

                // Combine to a two digit hex value into minutes

                Minutes = htoi(HexNumber);

	        // Get Seconds from host 

                HexNumber[0] = toupper(get_byte());        // Get first hex digit (MSB)
                HexNumber[1] = toupper(get_byte());        // Get second hex digit (LSB)
                Ch = get_byte();                           // Get semi-colon delimiter

                // Combine to a two digit hex value into Seconds

                Seconds = htoi(HexNumber);


}
#endif

#ifdef PAUSE_TEST

//********************************************************************************
//* pauseTest - Performs the pause function test.
//********************************************************************************
void            pauseTest(void)
{
    int             i;
    unsigned int pauseTime = 1;


    // This is a simple 15 second delay test using the pause function

    Beep(3);
    printf("Starting a 15 second pause test .. \n");

    for (i = 0; i < 15; i++)
    {
        pause(pauseTime);
    }

    Beep(5);

    printf("End of 15 second pause test. \n");
    printf("Start of LED flashing, every second... \n");


    // Test to flash LED every second

    while (TRUE)
    {
        // Toggle an LED

        // Toggle(LED0);

        LED0 = ~LED0;
        pause(pauseTime);                                        // Pause for 0.5 seconds
    }


}
#endif

#ifdef DELAY_TEST

//********************************************************************************
//* DelayTest - Performs the Delay function test.
//********************************************************************************
void            DelayTest(void)
{
    int             i;
    unsigned int TimeDelay = 1;     

    // This is a simple 25 second delay test using the DelayS function

    Beep(3);
    printf("Starting a 25 second delay test.. \n");

    for (i = 0; i < 25; i++)
    {
        DelayMs(1000);                              // Delay for 1 second
        //DelayS(TimeDelay);                            // Delay for 1 second
   }

    Beep(5);
    printf("End of 25 second delay test. \n");
    printf("Start of LED flashing, every second... \n");


    // Test to flash LED every second

    while (TRUE)
    {
        // Toggle an LED

        // Toggle(LED0);

        LED0 = ~LED0;
        DelayMs(1000);                                      // Delay for 1 second
        //DelayS(TimeDelay);                                    // Delay for 1 second
    }

}
#endif

#ifdef CLOCK_TEST

//********************************************************************************
//* ClockTest - Performs the clock controller function test.
//********************************************************************************
void ClockTest(void)
{

    // Main Timer Controller menu

menu:

    printf("        The following Timer Controller commands are available: 				\r\n");
    printf(" 		 									\r\n");
    printf("       'SHH:MM:SS;   -  Set the time using HHMMSS format.                         	\r\n");
    printf("											\r\n");
    printf("        G;'          -  Get the current time in HH:MM:SS format     	                \r\n");
    printf("                        format.	 							\r\n");
    printf("			 	 							\r\n");
    printf("        'Z;'         -  Reset the Timer.	 			                \r\n");
    printf("Enter timer command:  								        \r\n");
 
   // Configure the timers and interrupts

    SetupInterrupts();
    return;

  
    do
    {
        // Get next command from RS-232 Port

        Command = toupper(get_byte());

        // Execute command

        switch (Command)
        {
 
        case 'S':
        {
                // Set the time in HH:MM:SS format from the host

 		SetTime();
 
#ifdef TEST
//*******************************************************************************
// Test code
                send_byte('S');                            // Verify that the set time command was received

//*******************************************************************************
#endif

                break;
        }

 
        case 'G':
        {    
           
                Ch = get_byte();                           // Get terminator ";" character from host

		// Get the current time in HH:MM:SS format and send it to the host

 		GetTime();


#ifdef TEST
//*******************************************************************************
// Test code
                send_byte('G');                            // Verify that the get time command was received

//*******************************************************************************
#endif
                break;

        }
   

        case 'Z':
        {
		SetupTimers();

#ifdef TEST
//*******************************************************************************
// Test code
                send_byte('Z');                            // Verify that the reset command was received
//*******************************************************************************
#endif


		break;
	}
	}
 
        } while (TRUE);  

    CloseTimer0();                                     // close modules
    CloseUSART();

}

#endif

#ifdef TIMER_INTERRUPTS

//********************************************************************************
//* TimerInterruptTest - Performs the Timer Interrupt test.  It will generate interrupts
//* at selected periods of 250 milli-seconds, 500 milli-seconds, and 1 second.  Each
//* ISR will flash and LED and increment a timer count corresponding to the individual
//* timer interrupt.  Timers 0..3 are used for this test.
//* The interrupt periods for each timer are set as follows:
//*  	Timer 0  - 1 Second 
//*     Timer 1  - 1 milli-second  
//*     Timer 2  - 500 micro-seconds
//*     Timer 3  - 250 milli-seconds
//*
//********************************************************************************
void            TimerInterruptTest(void)
{
    // Configure the timers and interrupts

    SetupInterrupts();

                                                       
}

#endif
 
#endif
 
